@@ -1,8 +1,10 @@
-/* SPDX-License-Identifier: Apache-2.0
- * Copyright 2011-2022 Blender Foundation */
+/* SPDX-FileCopyrightText: 2011-2022 Blender Foundation
+ *
+ * SPDX-License-Identifier: Apache-2.0 */
 
 #include "device/cpu/device.h"
 #include "device/cpu/device_impl.h"
+#include "integrator/denoiser_oidn.h"
 
 /* Used for `info.denoisers`. */
 /* TODO(sergey): The denoisers are probably to be moved completely out of the device into their
@@ -47,8 +49,9 @@ string device_cpu_capabilities()
   capabilities += system_cpu_support_sse2() ? "SSE2 " : "";
   capabilities += system_cpu_support_sse41() ? "SSE41 " : "";
   capabilities += system_cpu_support_avx2() ? "AVX2" : "";
-  if (capabilities[capabilities.size() - 1] == ' ')
+  if (capabilities[capabilities.size() - 1] == ' ') {
     capabilities.resize(capabilities.size() - 1);
+  }
   return capabilities;
 }
 
