@@ -1,5 +1,6 @@
-/* SPDX-License-Identifier: Apache-2.0
- * Copyright 2011-2022 Blender Foundation */
+/* SPDX-FileCopyrightText: 2011-2022 Blender Foundation
+ *
+ * SPDX-License-Identifier: Apache-2.0 */
 
 /*
  * ASHIKHMIN SHIRLEY BSDF
@@ -22,6 +23,7 @@ ccl_device int bsdf_ashikhmin_shirley_setup(ccl_private MicrofacetBsdf *bsdf)
   bsdf->alpha_x = clamp(bsdf->alpha_x, 1e-4f, 1.0f);
   bsdf->alpha_y = clamp(bsdf->alpha_y, 1e-4f, 1.0f);
 
+  bsdf->fresnel_type = MicrofacetFresnel::NONE;
   bsdf->type = CLOSURE_BSDF_ASHIKHMIN_SHIRLEY_ID;
   return SD_BSDF | SD_BSDF_HAS_EVAL;
 }
@@ -41,36 +43,37 @@ ccl_device_inline float bsdf_ashikhmin_shirley_roughness_to_exponent(float rough
 
 ccl_device_forceinline Spectrum bsdf_ashikhmin_shirley_eval(ccl_private const ShaderClosure *sc,
                                                             const float3 Ng,
-                                                            const float3 I,
-                                                            const float3 omega_in,
+                                                            const float3 wi,
+                                                            const float3 wo,
                                                             ccl_private float *pdf)
 {
   ccl_private const MicrofacetBsdf *bsdf = (ccl_private const MicrofacetBsdf *)sc;
-  const float cosNgI = dot(Ng, omega_in);
+  const float cosNgO = dot(Ng, wo);
   float3 N = bsdf->N;
 
-  float NdotI = dot(N, I);        /* in Cycles/OSL convention I is omega_out */
-  float NdotO = dot(N, omega_in); /* and consequently we use for O omaga_in ;) */
+  float NdotI = dot(N, wi);
+  float NdotO = dot(N, wo);
 
   float out = 0.0f;
 
-  if ((cosNgI < 0.0f) || fmaxf(bsdf->alpha_x, bsdf->alpha_y) <= 1e-4f ||
-      !(NdotI > 0.0f && NdotO > 0.0f)) {
+  if ((cosNgO < 0.0f) || fmaxf(bsdf->alpha_x, bsdf->alpha_y) <= 1e-4f ||
+      !(NdotI > 0.0f && NdotO > 0.0f))
+  {
     *pdf = 0.0f;
     return zero_spectrum();
   }
 
   NdotI = fmaxf(NdotI, 1e-6f);
   NdotO = fmaxf(NdotO, 1e-6f);
-  float3 H = normalize(omega_in + I);
-  float HdotI = fmaxf(fabsf(dot(H, I)), 1e-6f);
+  float3 H = normalize(wi + wo);
+  float HdotI = fmaxf(fabsf(dot(H, wi)), 1e-6f);
   float HdotN = fmaxf(dot(H, N), 1e-6f);
 
   /* pump from original paper
    * (first derivative disc., but cancels the HdotI in the pdf nicely) */
-  float pump = 1.0f / fmaxf(1e-6f, (HdotI * fmaxf(NdotO, NdotI)));
-  /* pump from d-brdf paper */
-  /*float pump = 1.0f / fmaxf(1e-4f, ((NdotO + NdotI) * (NdotO*NdotI))); */
+  float pump = 1.0f / fmaxf(1e-6f, (HdotI * fmaxf(NdotI, NdotO)));
+  /* `pump` from D-BRDF paper. */
+  // float pump = 1.0f / fmaxf(1e-4f, ((NdotI + NdotO) * (NdotI * NdotO)));
 
   float n_x = bsdf_ashikhmin_shirley_roughness_to_exponent(bsdf->alpha_x);
   float n_y = bsdf_ashikhmin_shirley_roughness_to_exponent(bsdf->alpha_y);
@@ -109,26 +112,21 @@ ccl_device_forceinline Spectrum bsdf_ashikhmin_shirley_eval(ccl_private const Sh
   return make_spectrum(out);
 }
 
-ccl_device_inline void bsdf_ashikhmin_shirley_sample_first_quadrant(float n_x,
-                                                                    float n_y,
-                                                                    float randu,
-                                                                    float randv,
-                                                                    ccl_private float *phi,
-                                                                    ccl_private float *cos_theta)
+ccl_device_inline void bsdf_ashikhmin_shirley_sample_first_quadrant(
+    float n_x, float n_y, const float2 rand, ccl_private float *phi, ccl_private float *cos_theta)
 {
-  *phi = atanf(sqrtf((n_x + 1.0f) / (n_y + 1.0f)) * tanf(M_PI_2_F * randu));
+  *phi = atanf(sqrtf((n_x + 1.0f) / (n_y + 1.0f)) * tanf(M_PI_2_F * rand.x));
   float cos_phi = cosf(*phi);
   float sin_phi = sinf(*phi);
-  *cos_theta = powf(randv, 1.0f / (n_x * cos_phi * cos_phi + n_y * sin_phi * sin_phi + 1.0f));
+  *cos_theta = powf(rand.y, 1.0f / (n_x * cos_phi * cos_phi + n_y * sin_phi * sin_phi + 1.0f));
 }
 
 ccl_device int bsdf_ashikhmin_shirley_sample(ccl_private const ShaderClosure *sc,
                                              float3 Ng,
-                                             float3 I,
-                                             float randu,
-                                             float randv,
+                                             float3 wi,
+                                             float2 rand,
                                              ccl_private Spectrum *eval,
-                                             ccl_private float3 *omega_in,
+                                             ccl_private float3 *wo,
                                              ccl_private float *pdf,
                                              ccl_private float2 *sampled_roughness)
 {
@@ -137,7 +135,7 @@ ccl_device int bsdf_ashikhmin_shirley_sample(ccl_private const ShaderClosure *sc
   float3 N = bsdf->N;
   int label = LABEL_REFLECT | LABEL_GLOSSY;
 
-  float NdotI = dot(N, I);
+  float NdotI = dot(N, wi);
   if (!(NdotI > 0.0f)) {
     *pdf = 0.0f;
     *eval = zero_spectrum();
@@ -160,32 +158,28 @@ ccl_device int bsdf_ashikhmin_shirley_sample(ccl_private const ShaderClosure *sc
   float cos_theta;
   if (n_x == n_y) {
     /* isotropic sampling */
-    phi = M_2PI_F * randu;
-    cos_theta = powf(randv, 1.0f / (n_x + 1.0f));
+    phi = M_2PI_F * rand.x;
+    cos_theta = powf(rand.y, 1.0f / (n_x + 1.0f));
   }
   else {
     /* anisotropic sampling */
-    if (randu < 0.25f) { /* first quadrant */
-      float remapped_randu = 4.0f * randu;
-      bsdf_ashikhmin_shirley_sample_first_quadrant(
-          n_x, n_y, remapped_randu, randv, &phi, &cos_theta);
+    if (rand.x < 0.25f) { /* first quadrant */
+      rand.x *= 4.0f;
+      bsdf_ashikhmin_shirley_sample_first_quadrant(n_x, n_y, rand, &phi, &cos_theta);
     }
-    else if (randu < 0.5f) { /* second quadrant */
-      float remapped_randu = 4.0f * (.5f - randu);
-      bsdf_ashikhmin_shirley_sample_first_quadrant(
-          n_x, n_y, remapped_randu, randv, &phi, &cos_theta);
+    else if (rand.x < 0.5f) { /* second quadrant */
+      rand.x = 4.0f * (0.5f - rand.x);
+      bsdf_ashikhmin_shirley_sample_first_quadrant(n_x, n_y, rand, &phi, &cos_theta);
       phi = M_PI_F - phi;
     }
-    else if (randu < 0.75f) { /* third quadrant */
-      float remapped_randu = 4.0f * (randu - 0.5f);
-      bsdf_ashikhmin_shirley_sample_first_quadrant(
-          n_x, n_y, remapped_randu, randv, &phi, &cos_theta);
+    else if (rand.x < 0.75f) { /* third quadrant */
+      rand.x = 4.0f * (rand.x - 0.5f);
+      bsdf_ashikhmin_shirley_sample_first_quadrant(n_x, n_y, rand, &phi, &cos_theta);
       phi = M_PI_F + phi;
     }
     else { /* fourth quadrant */
-      float remapped_randu = 4.0f * (1.0f - randu);
-      bsdf_ashikhmin_shirley_sample_first_quadrant(
-          n_x, n_y, remapped_randu, randv, &phi, &cos_theta);
+      rand.x = 4.0f * (1.0f - rand.x);
+      bsdf_ashikhmin_shirley_sample_first_quadrant(n_x, n_y, rand, &phi, &cos_theta);
       phi = 2.0f * M_PI_F - phi;
     }
   }
@@ -198,12 +192,12 @@ ccl_device int bsdf_ashikhmin_shirley_sample(ccl_private const ShaderClosure *sc
 
   /* half vector to world space */
   float3 H = h.x * X + h.y * Y + h.z * N;
-  float HdotI = dot(H, I);
+  float HdotI = dot(H, wi);
   if (HdotI < 0.0f)
     H = -H;
 
-  /* reflect I on H to get omega_in */
-  *omega_in = -I + (2.0f * HdotI) * H;
+  /* reflect wi on H to get wo */
+  *wo = -wi + (2.0f * HdotI) * H;
 
   if (fmaxf(bsdf->alpha_x, bsdf->alpha_y) <= 1e-4f) {
     /* Some high number for MIS. */
@@ -213,7 +207,7 @@ ccl_device int bsdf_ashikhmin_shirley_sample(ccl_private const ShaderClosure *sc
   }
   else {
     /* leave the rest to eval */
-    *eval = bsdf_ashikhmin_shirley_eval(sc, N, I, *omega_in, pdf);
+    *eval = bsdf_ashikhmin_shirley_eval(sc, Ng, wi, *wo, pdf);
   }
 
   return label;
