@@ -2,6 +2,7 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 
+import bpy
 import re
 import os
 from typing import List
@@ -13,7 +14,11 @@ from ...io.com.gltf2_io_constants import ComponentType, DataType
 from ...io.exp import gltf2_io_binary_data, gltf2_io_buffer, gltf2_io_image_data
 from ...io.exp.gltf2_io_user_extensions import export_user_extensions
 from .gltf2_blender_gather_accessors import gather_accessor
+from .material.gltf2_blender_gather_image import get_gltf_image_from_blender_image
 
+class AdditionalData:
+    def __init__(self):
+        additional_textures = []
 
 class GlTF2Exporter:
     """
@@ -59,6 +64,9 @@ class GlTF2Exporter:
             textures=[]
         )
 
+
+        self.additional_data = AdditionalData()
+
         self.__buffer = gltf2_io_buffer.Buffer()
         self.__images = {}
 
@@ -117,7 +125,7 @@ class GlTF2Exporter:
                     f.write(self.__buffer.to_bytes())
                 uri = buffer_name
             else:
-                pass # This is no more possible, we don't export embedded buffers
+                uri = self.__buffer.to_embed_string()
 
             buffer = gltf2_io.Buffer(
                 byte_length=self.__buffer.byte_length,
@@ -268,57 +276,44 @@ class GlTF2Exporter:
 
             self.nodes_idx_to_remove.extend(insts)
 
-    def add_scene(self, scene: gltf2_io.Scene, active: bool = False, export_settings=None):
-        """
-        Add a scene to the glTF.
-
-        The scene should be built up with the generated glTF classes
-        :param scene: gltf2_io.Scene type. Root node of the scene graph
-        :param active: If true, sets the glTD.scene index to the added scene
-        :return: nothing
-        """
-        if self.__finalized:
-            raise RuntimeError("Tried to add scene to finalized glTF file")
-
-        scene_num = self.__traverse(scene)
-        if active:
-            self.__gltf.scene = scene_num
 
+    def manage_gpu_instancing_nodes(self, export_settings):
         if export_settings['gltf_gpu_instances'] is True:
-            # Modify the scene data in case of EXT_mesh_gpu_instancing export
+            for scene_num in range(len(self.__gltf.scenes)):
+                # Modify the scene data in case of EXT_mesh_gpu_instancing export
 
-            self.nodes_idx_to_remove = []
-            for node_idx in self.__gltf.scenes[scene_num].nodes:
-                node = self.__gltf.nodes[node_idx]
-                if node.mesh is None:
-                    self.manage_gpu_instancing(node)
-                else:
-                    self.manage_gpu_instancing(node, also_mesh=True)
-                for child_idx in node.children:
-                    child = self.__gltf.nodes[child_idx]
-                    self.manage_gpu_instancing(child, also_mesh=child.mesh is not None)
+                self.nodes_idx_to_remove = []
+                for node_idx in self.__gltf.scenes[scene_num].nodes:
+                    node = self.__gltf.nodes[node_idx]
+                    if node.mesh is None:
+                        self.manage_gpu_instancing(node)
+                    else:
+                        self.manage_gpu_instancing(node, also_mesh=True)
+                    for child_idx in node.children:
+                        child = self.__gltf.nodes[child_idx]
+                        self.manage_gpu_instancing(child, also_mesh=child.mesh is not None)
 
-            # Slides other nodes index
+                # Slides other nodes index
 
-            self.nodes_idx_to_remove.sort()
-            for node_idx in self.__gltf.scenes[scene_num].nodes:
-                self.recursive_slide_node_idx(node_idx)
+                self.nodes_idx_to_remove.sort()
+                for node_idx in self.__gltf.scenes[scene_num].nodes:
+                    self.recursive_slide_node_idx(node_idx)
 
-            new_node_list = []
-            for node_idx in self.__gltf.scenes[scene_num].nodes:
-                len_ = len([i for i in self.nodes_idx_to_remove if i < node_idx])
-                new_node_list.append(node_idx - len_)
-            self.__gltf.scenes[scene_num].nodes = new_node_list
-
-            for skin in self.__gltf.skins:
-                new_joint_list = []
-                for node_idx in skin.joints:
+                new_node_list = []
+                for node_idx in self.__gltf.scenes[scene_num].nodes:
                     len_ = len([i for i in self.nodes_idx_to_remove if i < node_idx])
-                    new_joint_list.append(node_idx - len_)
-                skin.joints = new_joint_list
-                if skin.skeleton is not None:
-                    len_ = len([i for i in self.nodes_idx_to_remove if i < skin.skeleton])
-                    skin.skeleton = skin.skeleton - len_
+                    new_node_list.append(node_idx - len_)
+                self.__gltf.scenes[scene_num].nodes = new_node_list
+
+                for skin in self.__gltf.skins:
+                    new_joint_list = []
+                    for node_idx in skin.joints:
+                        len_ = len([i for i in self.nodes_idx_to_remove if i < node_idx])
+                        new_joint_list.append(node_idx - len_)
+                    skin.joints = new_joint_list
+                    if skin.skeleton is not None:
+                        len_ = len([i for i in self.nodes_idx_to_remove if i < skin.skeleton])
+                        skin.skeleton = skin.skeleton - len_
 
             # Remove animation channels that was targeting a node that will be removed
             new_animation_list = []
@@ -337,6 +332,23 @@ class GlTF2Exporter:
             # And now really remove nodes
             self.__gltf.nodes = [node for idx, node in enumerate(self.__gltf.nodes) if idx not in self.nodes_idx_to_remove]
 
+
+    def add_scene(self, scene: gltf2_io.Scene, active: bool = False, export_settings=None):
+        """
+        Add a scene to the glTF.
+
+        The scene should be built up with the generated glTF classes
+        :param scene: gltf2_io.Scene type. Root node of the scene graph
+        :param active: If true, sets the glTD.scene index to the added scene
+        :return: nothing
+        """
+        if self.__finalized:
+            raise RuntimeError("Tried to add scene to finalized glTF file")
+
+        scene_num = self.__traverse(scene)
+        if active:
+            self.__gltf.scene = scene_num
+
     def recursive_slide_node_idx(self, node_idx):
         node = self.__gltf.nodes[node_idx]
 
@@ -355,6 +367,22 @@ class GlTF2Exporter:
         for s in skins:
             self.__traverse(s)
 
+    def traverse_additional_textures(self):
+        if self.export_settings['gltf_unused_textures'] is True:
+            tab = []
+            for tex in self.export_settings['additional_texture_export']:
+                res = self.__traverse(tex)
+                tab.append(res)
+
+            self.additional_data.additional_textures = tab
+
+    def traverse_additional_images(self):
+        if self.export_settings['gltf_unused_images']:
+            for img in [img for img in bpy.data.images if img.source != "VIEWER"]:
+                # TODO manage full / partial / custom via hook ...
+                if img.name not in self.export_settings['exported_images'].keys():
+                    self.__traverse(get_gltf_image_from_blender_image(img.name, self.export_settings))
+
     def add_animation(self, animation: gltf2_io.Animation):
         """
         Add an animation to the glTF.
@@ -507,3 +535,38 @@ class GlTF2Exporter:
 
         # do nothing for any type that does not match a glTF schema (primitives)
         return node
+
+def fix_json(obj):
+    # TODO: move to custom JSON encoder
+    fixed = obj
+    if isinstance(obj, dict):
+        fixed = {}
+        for key, value in obj.items():
+            if key == 'extras' and value is not None:
+                fixed[key] = value
+                continue
+            if not __should_include_json_value(key, value):
+                continue
+            fixed[key] = fix_json(value)
+    elif isinstance(obj, list):
+        fixed = []
+        for value in obj:
+            fixed.append(fix_json(value))
+    elif isinstance(obj, float):
+        # force floats to int, if they are integers (prevent INTEGER_WRITTEN_AS_FLOAT validator warnings)
+        if int(obj) == obj:
+            return int(obj)
+    return fixed
+
+def __should_include_json_value(key, value):
+    allowed_empty_collections = ["KHR_materials_unlit", "KHR_materials_specular"]
+
+    if value is None:
+        return False
+    elif __is_empty_collection(value) and key not in allowed_empty_collections:
+        return False
+    return True
+
+
+def __is_empty_collection(value):
+    return (isinstance(value, dict) or isinstance(value, list)) and len(value) == 0
