@@ -6,11 +6,12 @@
 # XXX: This script is meant to be used from inside Blender!
 #      You should not directly use this script, rather use update_msg.py!
 
-import datetime
+import time
 import os
 import re
 import sys
 import glob
+from pathlib import PurePath
 
 # XXX Relative import does not work here when used from Blender...
 from bl_i18n_utils import settings as settings_i18n, utils
@@ -573,7 +574,9 @@ def dump_py_messages_from_files(msgs, reports, files, settings):
         ("pgettext", ("_",)),
         ("pgettext_iface", ("iface_",)),
         ("pgettext_tip", ("tip_",)),
+        ("pgettext_rpt", ("rpt_",)),
         ("pgettext_data", ("data_",)),
+        ("poll_message_set", ()),
     )
     pgettext_variants_args = {"msgid": (0, {"msgctxt": 1})}
 
@@ -588,6 +591,7 @@ def dump_py_messages_from_files(msgs, reports, files, settings):
                   ),
         "message": (),
         "heading": (),
+        "placeholder": ((("text_ctxt",), _ctxt_to_ctxt),),
     }
 
     context_kw_set = {}
@@ -621,7 +625,8 @@ def dump_py_messages_from_files(msgs, reports, files, settings):
         for arg_pos, (arg_kw, arg) in enumerate(func.parameters.items()):
             if (not arg.is_output) and (arg.type == 'STRING'):
                 for msgid, msgctxts in context_kw_set.items():
-                    if arg_kw in msgctxts:
+                    # The msgid can be missing if it is used in only some UILayout functions but not all
+                    if arg_kw in msgctxts and msgid in func_translate_args[func_id]:
                         func_translate_args[func_id][msgid][1][arg_kw] = arg_pos
     # The report() func of operators.
     for func_id, func in bpy.types.Operator.bl_rna.functions.items():
@@ -661,6 +666,7 @@ def dump_py_messages_from_files(msgs, reports, files, settings):
             root_node = ast.parse(filedata.read(), fp, 'exec')
 
         fp_rel = make_rel(fp)
+        fp_rel = PurePath(fp_rel).as_posix()
 
         for node in ast.walk(root_node):
             if type(node) == ast.Call:
@@ -775,7 +781,7 @@ def dump_src_messages(msgs, reports, settings):
 
     def clean_str(s):
         # The encode/decode to/from 'raw_unicode_escape' allows to transform the C-type unicode hexadecimal escapes
-        # (like '\u2715' for the '×' symbol) back into a proper unicode character.
+        # (like '\u00d7' for the '×' symbol) back into a proper unicode character.
         return "".join(
             m.group("clean") for m in _clean_str(s)
         ).encode('raw_unicode_escape').decode('raw_unicode_escape')
@@ -939,6 +945,14 @@ def dump_asset_messages(msgs, reports, settings):
 
     # Parse the asset blend files
     asset_files = {}
+    # Store assets according to this structure:
+    # {"basename": [
+    #     {"name": "Name",
+    #      "description": "Description",
+    #      "sockets": [
+    #          ("Name", "Description"),
+    #      ]},
+    # ]}
 
     bfiles = glob.glob(assets_dir + "/**/*.blend", recursive=True)
     for bfile in bfiles:
@@ -951,11 +965,18 @@ def dump_asset_messages(msgs, reports, settings):
                 if asset.asset_data is None:  # Not an asset
                     continue
                 assets = asset_files.setdefault(basename, [])
-                assets.append((asset.name, asset.asset_data.description))
+                asset_data = {"name": asset.name,
+                              "description": asset.asset_data.description}
+                for interface in asset.interface.items_tree:
+                    if interface.name == "Geometry":  # Ignore common socket
+                        continue
+                    socket_data = asset_data.setdefault("sockets", [])
+                    socket_data.append((interface.name, interface.description))
+                assets.append(asset_data)
 
     for asset_file in sorted(asset_files):
-        for asset in sorted(asset_files[asset_file]):
-            name, description = asset
+        for asset in sorted(asset_files[asset_file], key=lambda a: a["name"]):
+            name, description = asset["name"], asset["description"]
             msgsrc = "Asset name from file " + asset_file
             process_msg(msgs, settings.DEFAULT_CONTEXT, name, msgsrc,
                         reports, None, settings)
@@ -963,6 +984,15 @@ def dump_asset_messages(msgs, reports, settings):
             process_msg(msgs, settings.DEFAULT_CONTEXT, description, msgsrc,
                         reports, None, settings)
 
+            if "sockets" in asset:
+                for socket_name, socket_description in asset["sockets"]:
+                    msgsrc = f"Socket name from node group {name}, file {asset_file}"
+                    process_msg(msgs, settings.DEFAULT_CONTEXT, socket_name, msgsrc,
+                                reports, None, settings)
+                    msgsrc = f"Socket description from node group {name}, file {asset_file}"
+                    process_msg(msgs, settings.DEFAULT_CONTEXT, socket_description, msgsrc,
+                                reports, None, settings)
+
 
 def dump_addon_bl_info(msgs, reports, module, settings):
     for prop in ('name', 'location', 'description', 'warning'):
@@ -984,10 +1014,9 @@ def dump_addon_bl_info(msgs, reports, module, settings):
 def dump_messages(do_messages, do_checks, settings):
     bl_ver = "Blender " + bpy.app.version_string
     bl_hash = bpy.app.build_hash
-    bl_date = datetime.datetime.strptime(bpy.app.build_date.decode() + "T" + bpy.app.build_time.decode(),
-                                         "%Y-%m-%dT%H:%M:%S")
-    pot = utils.I18nMessages.gen_empty_messages(settings.PARSER_TEMPLATE_ID, bl_ver, bl_hash, bl_date, bl_date.year,
-                                                settings=settings)
+    bl_time = time.strptime(f"{bpy.app.build_date.decode()} {bpy.app.build_time.decode()} UTC", "%Y-%m-%d %H:%M:%S %Z")
+    pot = utils.I18nMessages.gen_empty_messages(
+        settings.PARSER_TEMPLATE_ID, bl_ver, bl_hash, bl_time, settings=settings)
     msgs = pot.msgs
 
     # Enable all wanted addons.
@@ -1074,25 +1103,24 @@ def dump_messages(do_messages, do_checks, settings):
     return pot  # Not used currently, but may be useful later (and to be consistent with dump_addon_messages!).
 
 
-def dump_addon_messages(module_name, do_checks, settings):
+def dump_addon_messages(addon_module_name, do_checks, settings):
     import addon_utils
 
     # Get current addon state (loaded or not):
-    was_loaded = addon_utils.check(module_name)[1]
+    was_loaded = addon_utils.check(addon_module_name)[1]
 
     # Enable our addon.
-    addon = utils.enable_addons(addons={module_name})[0]
+    addon = utils.enable_addons(addons={addon_module_name})[0]
 
     addon_info = addon_utils.module_bl_info(addon)
     ver = addon_info["name"] + " " + ".".join(str(v) for v in addon_info["version"])
     rev = 0
-    date = datetime.datetime.now()
-    pot = utils.I18nMessages.gen_empty_messages(settings.PARSER_TEMPLATE_ID, ver, rev, date, date.year,
-                                                settings=settings)
+    curr_time = time.gmtime()
+    pot = utils.I18nMessages.gen_empty_messages(settings.PARSER_TEMPLATE_ID, ver, rev, curr_time, settings=settings)
     msgs = pot.msgs
 
-    minus_pot = utils.I18nMessages.gen_empty_messages(settings.PARSER_TEMPLATE_ID, ver, rev, date, date.year,
-                                                      settings=settings)
+    minus_pot = utils.I18nMessages.gen_empty_messages(
+        settings.PARSER_TEMPLATE_ID, ver, rev, curr_time, settings=settings)
     minus_msgs = minus_pot.msgs
 
     check_ctxt = _gen_check_ctxt(settings) if do_checks else None
@@ -1106,7 +1134,7 @@ def dump_addon_messages(module_name, do_checks, settings):
     print("C")
 
     # Now disable our addon, and re-scan RNA.
-    utils.enable_addons(addons={module_name}, disable=True)
+    utils.enable_addons(addons={addon_module_name}, disable=True)
     print("D")
     reports["check_ctxt"] = minus_check_ctxt
     print("E")
@@ -1115,7 +1143,7 @@ def dump_addon_messages(module_name, do_checks, settings):
 
     # Restore previous state if needed!
     if was_loaded:
-        utils.enable_addons(addons={module_name})
+        utils.enable_addons(addons={addon_module_name})
 
     # and make the diff!
     for key in minus_msgs:
