@@ -30,13 +30,13 @@
 #include "BLI_string.h"
 #include "BLI_utildefines.h"
 
-#include "BKE_appdir.h"
-#include "BKE_blender_undo.h" /* own include */
-#include "BKE_blendfile.h"
-#include "BKE_context.h"
+#include "BKE_appdir.hh"
+#include "BKE_blender_undo.hh" /* own include */
+#include "BKE_blendfile.hh"
+#include "BKE_context.hh"
 #include "BKE_global.h"
-#include "BKE_main.h"
-#include "BKE_undo_system.h"
+#include "BKE_main.hh"
+#include "BKE_undo_system.hh"
 
 #include "BLO_readfile.h"
 #include "BLO_undofile.hh"
