@@ -208,25 +208,23 @@ void calculate_auto_handles(const bool cyclic,
                           positions_right.last());
 }
 
-void evaluate_segment(const float3 &point_0,
-                      const float3 &point_1,
-                      const float3 &point_2,
-                      const float3 &point_3,
-                      MutableSpan<float3> result)
+template<typename T>
+void evaluate_segment_ex(
+    const T &point_0, const T &point_1, const T &point_2, const T &point_3, MutableSpan<T> result)
 {
   BLI_assert(result.size() > 0);
   const float inv_len = 1.0f / float(result.size());
   const float inv_len_squared = inv_len * inv_len;
   const float inv_len_cubed = inv_len_squared * inv_len;
 
-  const float3 rt1 = 3.0f * (point_1 - point_0) * inv_len;
-  const float3 rt2 = 3.0f * (point_0 - 2.0f * point_1 + point_2) * inv_len_squared;
-  const float3 rt3 = (point_3 - point_0 + 3.0f * (point_1 - point_2)) * inv_len_cubed;
+  const T rt1 = 3.0f * (point_1 - point_0) * inv_len;
+  const T rt2 = 3.0f * (point_0 - 2.0f * point_1 + point_2) * inv_len_squared;
+  const T rt3 = (point_3 - point_0 + 3.0f * (point_1 - point_2)) * inv_len_cubed;
 
-  float3 q0 = point_0;
-  float3 q1 = rt1 + rt2 + rt3;
-  float3 q2 = 2.0f * rt2 + 6.0f * rt3;
-  float3 q3 = 6.0f * rt3;
+  T q0 = point_0;
+  T q1 = rt1 + rt2 + rt3;
+  T q2 = 2.0f * rt2 + 6.0f * rt3;
+  T q3 = 6.0f * rt3;
   for (const int i : result.index_range()) {
     result[i] = q0;
     q0 += q1;
@@ -234,6 +232,24 @@ void evaluate_segment(const float3 &point_0,
     q2 += q3;
   }
 }
+template<>
+void evaluate_segment(const float3 &point_0,
+                      const float3 &point_1,
+                      const float3 &point_2,
+                      const float3 &point_3,
+                      MutableSpan<float3> result)
+{
+  evaluate_segment_ex<float3>(point_0, point_1, point_2, point_3, result);
+}
+template<>
+void evaluate_segment(const float2 &point_0,
+                      const float2 &point_1,
+                      const float2 &point_2,
+                      const float2 &point_3,
+                      MutableSpan<float2> result)
+{
+  evaluate_segment_ex<float2>(point_0, point_1, point_2, point_3, result);
+}
 
 void calculate_evaluated_positions(const Span<float3> positions,
                                    const Span<float3> handles_left,
