@@ -6,25 +6,22 @@
  * \ingroup bke
  */
 
-#include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
-#include "DNA_object_types.h"
-
 #include "BLI_map.hh"
 #include "BLI_ordered_edge.hh"
 #include "BLI_task.hh"
 #include "BLI_threads.h"
-#include "BLI_timeit.hh"
 
 #include "BKE_attribute.hh"
-#include "BKE_customdata.h"
+#include "BKE_customdata.hh"
 #include "BKE_mesh.hh"
 
-namespace blender::bke::calc_edges {
+namespace blender::bke {
+
+namespace calc_edges {
 
 /**
  * Return a hash value that is likely to be different in the low bits from the normal `hash()`
- * function. This is necessary to avoid collisions in #BKE_mesh_calc_edges.
+ * function. This is necessary to avoid collisions in #mesh_calc_edges.
  */
 static uint64_t edge_hash_2(const OrderedEdge &edge)
 {
@@ -38,21 +35,21 @@ union OrigEdgeOrIndex {
 };
 using EdgeMap = Map<OrderedEdge, OrigEdgeOrIndex>;
 
-static void reserve_hash_maps(const Mesh *mesh,
+static void reserve_hash_maps(const Mesh &mesh,
                               const bool keep_existing_edges,
                               MutableSpan<EdgeMap> edge_maps)
 {
-  const int totedge_guess = std::max(keep_existing_edges ? mesh->totedge : 0, mesh->faces_num * 2);
+  const int totedge_guess = std::max(keep_existing_edges ? mesh.edges_num : 0, mesh.faces_num * 2);
   threading::parallel_for_each(
       edge_maps, [&](EdgeMap &edge_map) { edge_map.reserve(totedge_guess / edge_maps.size()); });
 }
 
-static void add_existing_edges_to_hash_maps(Mesh *mesh,
+static void add_existing_edges_to_hash_maps(Mesh &mesh,
                                             MutableSpan<EdgeMap> edge_maps,
                                             uint32_t parallel_mask)
 {
   /* Assume existing edges are valid. */
-  const Span<int2> edges = mesh->edges();
+  const Span<int2> edges = mesh.edges();
   threading::parallel_for_each(edge_maps, [&](EdgeMap &edge_map) {
     const int task_index = &edge_map - edge_maps.data();
     for (const int2 &edge : edges) {
@@ -65,12 +62,12 @@ static void add_existing_edges_to_hash_maps(Mesh *mesh,
   });
 }
 
-static void add_face_edges_to_hash_maps(Mesh *mesh,
+static void add_face_edges_to_hash_maps(Mesh &mesh,
                                         MutableSpan<EdgeMap> edge_maps,
                                         uint32_t parallel_mask)
 {
-  const OffsetIndices faces = mesh->faces();
-  const Span<int> corner_verts = mesh->corner_verts();
+  const OffsetIndices faces = mesh.faces();
+  const Span<int> corner_verts = mesh.corner_verts();
   threading::parallel_for_each(edge_maps, [&](EdgeMap &edge_map) {
     const int task_index = &edge_map - edge_maps.data();
     for (const int i : faces.index_range()) {
@@ -159,38 +156,34 @@ static void update_edge_indices_in_face_loops(const OffsetIndices<int> faces,
   });
 }
 
-static int get_parallel_maps_count(const Mesh *mesh)
+static int get_parallel_maps_count(const Mesh &mesh)
 {
   /* Don't use parallelization when the mesh is small. */
-  if (mesh->faces_num < 1000) {
+  if (mesh.faces_num < 1000) {
     return 1;
   }
   /* Use at most 8 separate hash tables. Using more threads has diminishing returns. These threads
-   * can better do something more useful instead. */
+   * are better off doing something more useful instead. */
   const int system_thread_count = BLI_system_thread_count();
   return power_of_2_min_i(std::min(8, system_thread_count));
 }
 
 static void clear_hash_tables(MutableSpan<EdgeMap> edge_maps)
 {
-  threading::parallel_for_each(edge_maps, [](EdgeMap &edge_map) { edge_map.clear(); });
+  threading::parallel_for_each(edge_maps, [](EdgeMap &edge_map) { edge_map.clear_and_shrink(); });
 }
 
-}  // namespace blender::bke::calc_edges
+}  // namespace calc_edges
 
-void BKE_mesh_calc_edges(Mesh *mesh, bool keep_existing_edges, const bool select_new_edges)
+void mesh_calc_edges(Mesh &mesh, bool keep_existing_edges, const bool select_new_edges)
 {
-  using namespace blender;
-  using namespace blender::bke;
-  using namespace blender::bke::calc_edges;
-
   /* Parallelization is achieved by having multiple hash tables for different subsets of edges.
    * Each edge is assigned to one of the hash maps based on the lower bits of a hash value. */
-  const int parallel_maps = get_parallel_maps_count(mesh);
+  const int parallel_maps = calc_edges::get_parallel_maps_count(mesh);
   BLI_assert(is_power_of_2_i(parallel_maps));
   const uint32_t parallel_mask = uint32_t(parallel_maps) - 1;
-  Array<EdgeMap> edge_maps(parallel_maps);
-  reserve_hash_maps(mesh, keep_existing_edges, edge_maps);
+  Array<calc_edges::EdgeMap> edge_maps(parallel_maps);
+  calc_edges::reserve_hash_maps(mesh, keep_existing_edges, edge_maps);
 
   /* Add all edges. */
   if (keep_existing_edges) {
@@ -200,36 +193,33 @@ void BKE_mesh_calc_edges(Mesh *mesh, bool keep_existing_edges, const bool select
 
   /* Compute total number of edges. */
   int new_totedge = 0;
-  for (EdgeMap &edge_map : edge_maps) {
+  for (calc_edges::EdgeMap &edge_map : edge_maps) {
     new_totedge += edge_map.size();
   }
 
   /* Create new edges. */
-  MutableAttributeAccessor attributes = mesh->attributes_for_write();
-  attributes.add<int>(".corner_edge", ATTR_DOMAIN_CORNER, AttributeInitConstruct());
+  MutableAttributeAccessor attributes = mesh.attributes_for_write();
+  attributes.add<int>(".corner_edge", AttrDomain::Corner, AttributeInitConstruct());
   MutableSpan<int2> new_edges{
       static_cast<int2 *>(MEM_calloc_arrayN(new_totedge, sizeof(int2), __func__)), new_totedge};
   calc_edges::serialize_and_initialize_deduplicated_edges(edge_maps, new_edges);
-  calc_edges::update_edge_indices_in_face_loops(mesh->faces(),
-                                                mesh->corner_verts(),
-                                                edge_maps,
-                                                parallel_mask,
-                                                mesh->corner_edges_for_write());
+  calc_edges::update_edge_indices_in_face_loops(
+      mesh.faces(), mesh.corner_verts(), edge_maps, parallel_mask, mesh.corner_edges_for_write());
 
   /* Free old CustomData and assign new one. */
-  CustomData_free(&mesh->edge_data, mesh->totedge);
-  CustomData_reset(&mesh->edge_data);
-  mesh->totedge = new_totedge;
-  attributes.add<int2>(".edge_verts", ATTR_DOMAIN_EDGE, AttributeInitMoveArray(new_edges.data()));
+  CustomData_free(&mesh.edge_data, mesh.edges_num);
+  CustomData_reset(&mesh.edge_data);
+  mesh.edges_num = new_totedge;
+  attributes.add<int2>(".edge_verts", AttrDomain::Edge, AttributeInitMoveArray(new_edges.data()));
 
   if (select_new_edges) {
-    MutableAttributeAccessor attributes = mesh->attributes_for_write();
+    MutableAttributeAccessor attributes = mesh.attributes_for_write();
     SpanAttributeWriter<bool> select_edge = attributes.lookup_or_add_for_write_span<bool>(
-        ".select_edge", ATTR_DOMAIN_EDGE);
+        ".select_edge", AttrDomain::Edge);
     if (select_edge) {
       int new_edge_index = 0;
-      for (const EdgeMap &edge_map : edge_maps) {
-        for (EdgeMap::Item item : edge_map.items()) {
+      for (const calc_edges::EdgeMap &edge_map : edge_maps) {
+        for (calc_edges::EdgeMap::Item item : edge_map.items()) {
           if (item.value.original_edge == nullptr) {
             select_edge.span[new_edge_index] = true;
           }
@@ -242,9 +232,11 @@ void BKE_mesh_calc_edges(Mesh *mesh, bool keep_existing_edges, const bool select
 
   if (!keep_existing_edges) {
     /* All edges are rebuilt from the faces, so there are no loose edges. */
-    mesh->tag_loose_edges_none();
+    mesh.tag_loose_edges_none();
   }
 
   /* Explicitly clear edge maps, because that way it can be parallelized. */
   clear_hash_tables(edge_maps);
 }
+
+}  // namespace blender::bke
