@@ -128,13 +128,6 @@ MINLINE double sqrt3d(double d);
 
 MINLINE float sqrtf_signed(float f);
 
-MINLINE float saacosf(float f);
-MINLINE float saasinf(float f);
-MINLINE float sasqrtf(float f);
-MINLINE float saacos(float fac);
-MINLINE float saasin(float fac);
-MINLINE float sasqrt(float fac);
-
 /* Compute linear interpolation (lerp) between origin and target. */
 MINLINE float interpf(float target, float origin, float t);
 MINLINE double interpd(double target, double origin, double t);
@@ -296,18 +289,24 @@ MINLINE uint ceil_to_multiple_u(uint a, uint b);
 MINLINE uint64_t ceil_to_multiple_ul(uint64_t a, uint64_t b);
 
 /**
- * modulo that handles negative numbers, works the same as Python's.
+ * Floored modulo that is useful for wrapping numbers over \a n,
+ * including when \a i is negative.
+ *
+ * This is the same as Python % or GLSL mod(): `mod_i(-5, 3) = 1`.
+ *
+ * \return an integer in the interval [0, n), same sign as n.
  */
 MINLINE int mod_i(int i, int n);
 
 /**
- * Modulo that returns a positive result, regardless of the sign of \a f.
+ * Floored modulo that is useful for wrapping numbers over \a n,
+ * including when \a f is negative.
  *
- * For example, mod_f_positive(-0.1, 1.0) => 0.9.
+ * This is the same as Python % or GLSL mod(): `floored_fmod(-0.2, 1.0) = 0.8`.
  *
- * \returns a float in the interval [0, n).
+ * \return a float in the interval [0, n), same sign as n.
  */
-MINLINE float mod_f_positive(float f, float n);
+MINLINE float floored_fmod(float f, float n);
 
 /**
  * Round to closest even number, halfway cases are rounded away from zero.
