@@ -1,4 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later */
+/* SPDX-FileCopyrightText: 2023 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 /** \file
  * \ingroup bli
@@ -14,6 +16,7 @@
 
 #include "BLI_lazy_threading.hh"
 #include "BLI_task.h"
+#include "BLI_task.hh"
 #include "BLI_threads.h"
 
 #include "atomic_ops.h"
@@ -50,7 +53,7 @@ struct RangeTask {
   }
 
   /* Splitting constructor for parallel reduce. */
-  RangeTask(RangeTask &other, tbb::split /* unused */)
+  RangeTask(RangeTask &other, tbb::split /*unused*/)
       : func(other.func), userdata(other.userdata), settings(other.settings)
   {
     init_chunk(settings->userdata_chunk);
@@ -58,7 +61,7 @@ struct RangeTask {
 
   ~RangeTask()
   {
-    if (settings->func_free != nullptr && userdata_chunk != nullptr) {
+    if (settings->func_free != nullptr) {
       settings->func_free(userdata, userdata_chunk);
     }
     MEM_SAFE_FREE(userdata_chunk);
@@ -102,7 +105,7 @@ void BLI_task_parallel_range(const int start,
   /* Multithreading. */
   if (settings->use_threading && BLI_task_scheduler_num_threads() > 1) {
     RangeTask task(func, userdata, settings);
-    const size_t grainsize = MAX2(settings->min_iter_per_thread, 1);
+    const size_t grainsize = std::max(settings->min_iter_per_thread, 1);
     const tbb::blocked_range<int> range(start, stop, grainsize);
 
     blender::lazy_threading::send_hint();
@@ -127,7 +130,7 @@ void BLI_task_parallel_range(const int start,
   for (int i = start; i < stop; i++) {
     func(userdata, i, &tls);
   }
-  if (settings->func_free != nullptr && settings->userdata_chunk != nullptr) {
+  if (settings->func_free != nullptr) {
     settings->func_free(userdata, settings->userdata_chunk);
   }
 }
@@ -154,3 +157,28 @@ int BLI_task_parallel_thread_id(const TaskParallelTLS * /*tls*/)
   return 0;
 #endif
 }
+
+namespace blender::threading::detail {
+
+void parallel_for_impl(const IndexRange range,
+                       const int64_t grain_size,
+                       const FunctionRef<void(IndexRange)> function)
+{
+#ifdef WITH_TBB
+  /* Invoking tbb for small workloads has a large overhead. */
+  if (range.size() >= grain_size) {
+    lazy_threading::send_hint();
+    tbb::parallel_for(
+        tbb::blocked_range<int64_t>(range.first(), range.one_after_last(), grain_size),
+        [function](const tbb::blocked_range<int64_t> &subrange) {
+          function(IndexRange(subrange.begin(), subrange.size()));
+        });
+    return;
+  }
+#else
+  UNUSED_VARS(grain_size);
+#endif
+  function(range);
+}
+
+}  // namespace blender::threading::detail
