@@ -21,14 +21,15 @@
 #include "BKE_animsys.h"
 #include "BKE_idprop.h"
 #include "BKE_ipo.h"
-#include "BKE_lib_id.h"
+#include "BKE_lib_id.hh"
 #include "BKE_lib_override.hh"
-#include "BKE_main.h"
-#include "BKE_main_namemap.h"
+#include "BKE_main.hh"
+#include "BKE_main_namemap.hh"
+#include "BKE_mesh_legacy_convert.hh"
 #include "BKE_node.hh"
 #include "BKE_node_runtime.hh"
 
-#include "SEQ_sequencer.h"
+#include "SEQ_sequencer.hh"
 
 #include "MEM_guardedalloc.h"
 
@@ -100,10 +101,7 @@ ID *do_versions_rename_id(Main *bmain,
     }
   }
   if (id != nullptr) {
-    BKE_main_namemap_remove_name(bmain, id, id->name + 2);
-    BLI_strncpy(id->name + 2, name_dst, sizeof(id->name) - 2);
-    /* We know it's unique, this just sorts. */
-    BLI_libblock_ensure_unique_name(bmain, id->name);
+    BKE_libblock_rename(bmain, id, name_dst);
   }
   return id;
 }
@@ -283,20 +281,24 @@ void node_tree_relink_with_socket_id_map(bNodeTree &ntree,
   LISTBASE_FOREACH_MUTABLE (bNodeLink *, link, &ntree.links) {
     if (link->tonode == &old_node) {
       bNodeSocket *old_socket = link->tosock;
-      if (const std::string *new_identifier = map.lookup_ptr_as(old_socket->identifier)) {
-        bNodeSocket *new_socket = nodeFindSocket(&new_node, SOCK_IN, new_identifier->c_str());
-        link->tonode = &new_node;
-        link->tosock = new_socket;
-        old_socket->link = nullptr;
+      if (old_socket->is_available()) {
+        if (const std::string *new_identifier = map.lookup_ptr_as(old_socket->identifier)) {
+          bNodeSocket *new_socket = nodeFindSocket(&new_node, SOCK_IN, new_identifier->c_str());
+          link->tonode = &new_node;
+          link->tosock = new_socket;
+          old_socket->link = nullptr;
+        }
       }
     }
     if (link->fromnode == &old_node) {
       bNodeSocket *old_socket = link->fromsock;
-      if (const std::string *new_identifier = map.lookup_ptr_as(old_socket->identifier)) {
-        bNodeSocket *new_socket = nodeFindSocket(&new_node, SOCK_OUT, new_identifier->c_str());
-        link->fromnode = &new_node;
-        link->fromsock = new_socket;
-        old_socket->link = nullptr;
+      if (old_socket->is_available()) {
+        if (const std::string *new_identifier = map.lookup_ptr_as(old_socket->identifier)) {
+          bNodeSocket *new_socket = nodeFindSocket(&new_node, SOCK_OUT, new_identifier->c_str());
+          link->fromnode = &new_node;
+          link->fromsock = new_socket;
+          old_socket->link = nullptr;
+        }
       }
     }
   }
@@ -526,4 +528,11 @@ void do_versions_after_setup(Main *new_bmain, BlendFileReadReport *reports)
     /* Does not add any new IDs, but needs the full Main data-base. */
     BKE_lib_override_library_main_hierarchy_root_ensure(new_bmain);
   }
+
+  if (!blendfile_or_libraries_versions_atleast(new_bmain, 402, 22)) {
+    /* Initial auto smooth versioning started at (401, 2), but a bug caused the legacy flag to not
+     * be cleared, so it is re-run in a later version when the bug is fixed and the versioning has
+     * been made idempotent. */
+    BKE_main_mesh_legacy_convert_auto_smooth(*new_bmain);
+  }
 }
