@@ -21,7 +21,7 @@ void InpaintNode::convert_to_operations(NodeConverter &converter,
   /* if (editor_node->custom1 == CMP_NODE_INPAINT_SIMPLE) { */
   if (true) {
     InpaintSimpleOperation *operation = new InpaintSimpleOperation();
-    operation->set_iterations(editor_node->custom2);
+    operation->set_max_distance(editor_node->custom2);
     converter.add_operation(operation);
 
     converter.map_input_socket(get_input_socket(0), operation->get_input_socket(0));
