@@ -1,12 +1,13 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2012 Blender Foundation. */
+/* SPDX-FileCopyrightText: 2012 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 #pragma once
 
 #include "BLI_listbase.h"
 #include "COM_MultiThreadedOperation.h"
 #include "DNA_mask_types.h"
-#include "IMB_imbuf_types.h"
+#include "IMB_imbuf_types.hh"
 
 /* Forward declarations. */
 struct MaskRasterHandle;
@@ -82,7 +83,7 @@ class MaskOperation : public MultiThreadedOperation {
 
   void set_motion_blur_samples(int samples)
   {
-    raster_mask_handle_tot_ = MIN2(MAX2(1, samples), CMP_NODE_MASK_MBLUR_SAMPLES_MAX);
+    raster_mask_handle_tot_ = std::min(std::max(1, samples), CMP_NODE_MASK_MBLUR_SAMPLES_MAX);
   }
   void set_motion_blur_shutter(float shutter)
   {
