@@ -6,13 +6,16 @@
  * \ingroup draw_engine
  */
 
+#include "BKE_object_types.hh"
 #include "BKE_paint.hh"
+
 #include "BLI_math_color.h"
-#include "DRW_render.h"
+#include "BLI_time.h"
+
+#include "DRW_render.hh"
 
 #include "ED_view3d.hh"
 
-#include "PIL_time.h"
 #include "UI_resources.hh"
 
 #include "overlay_private.hh"
@@ -22,7 +25,7 @@ void OVERLAY_mode_transfer_cache_init(OVERLAY_Data *vedata)
   OVERLAY_PassList *psl = vedata->psl;
   OVERLAY_PrivateData *pd = vedata->stl->pd;
 
-  pd->mode_transfer.time = PIL_check_seconds_timer();
+  pd->mode_transfer.time = BLI_check_seconds_timer();
 
   for (int i = 0; i < 2; i++) {
     /* Non Meshes Pass (Camera, empties, lights ...) */
@@ -80,7 +83,7 @@ void OVERLAY_mode_transfer_cache_populate(OVERLAY_Data *vedata, Object *ob)
   }
 
   const float animation_time = pd->mode_transfer.time -
-                               ob->runtime.overlay_mode_transfer_start_time;
+                               ob->runtime->overlay_mode_transfer_start_time;
 
   if (!mode_transfer_is_animation_running(animation_time)) {
     return;
