@@ -6,9 +6,10 @@
  * \ingroup draw_engine
  */
 
-#include "DRW_render.h"
+#include "DRW_render.hh"
 
 #include "BLI_listbase.h"
+#include "BLI_math_matrix.hh"
 #include "BLI_string.h"
 
 #include "DNA_armature_types.h"
@@ -19,7 +20,7 @@
 
 #include "UI_resources.hh"
 
-#include "draw_manager_text.h"
+#include "draw_manager_text.hh"
 
 #include "overlay_private.hh"
 
@@ -92,7 +93,7 @@ static void motion_path_get_frame_range_to_draw(bAnimVizSettings *avs,
   }
   else {
     start = avs->path_sf;
-    end = avs->path_ef;
+    end = avs->path_ef + 1;
   }
 
   if (start > end) {
@@ -107,12 +108,22 @@ static void motion_path_get_frame_range_to_draw(bAnimVizSettings *avs,
   *r_step = max_ii(avs->path_step, 1);
 }
 
+static Object *get_camera_for_motion_path(const DRWContextState *draw_context,
+                                          const eMotionPath_BakeFlag bake_flag)
+{
+  if ((bake_flag & MOTIONPATH_BAKE_CAMERA_SPACE) == 0) {
+    return nullptr;
+  }
+  return draw_context->v3d->camera;
+}
+
 static void motion_path_cache(OVERLAY_Data *vedata,
                               Object *ob,
                               bPoseChannel *pchan,
                               bAnimVizSettings *avs,
                               bMotionPath *mpath)
 {
+  using namespace blender;
   OVERLAY_PrivateData *pd = vedata->stl->pd;
   const DRWContextState *draw_ctx = DRW_context_state_get();
   DRWTextStore *dt = DRW_text_cache_ensure();
@@ -133,8 +144,26 @@ static void motion_path_cache(OVERLAY_Data *vedata,
   if (len == 0) {
     return;
   }
+
+  /* Avoid 0 size allocations. Current code to calculate motion paths should
+   * sanitize this already [see animviz_verify_motionpaths()], we might however
+   * encounter an older file where this was still possible. */
+  if (mpath->length == 0) {
+    return;
+  }
+
   int start_index = sfra - mpath->start_frame;
 
+  float camera_matrix[4][4];
+  Object *motion_path_camera = get_camera_for_motion_path(
+      draw_ctx, eMotionPath_BakeFlag(avs->path_bakeflag));
+  if (motion_path_camera) {
+    copy_m4_m4(camera_matrix, motion_path_camera->object_to_world);
+  }
+  else {
+    unit_m4(camera_matrix);
+  }
+
   /* Draw curve-line of path. */
   if (show_lines) {
     const int motion_path_settings[4] = {cfra, sfra, efra, mpath->start_frame};
@@ -143,6 +172,7 @@ static void motion_path_cache(OVERLAY_Data *vedata,
     DRW_shgroup_uniform_int_copy(grp, "lineThickness", mpath->line_thickness);
     DRW_shgroup_uniform_bool_copy(grp, "selected", selected);
     DRW_shgroup_uniform_vec3_copy(grp, "customColor", color);
+    DRW_shgroup_uniform_mat4_copy(grp, "camera_space_matrix", camera_matrix);
     /* Only draw the required range. */
     DRW_shgroup_call_range(grp, nullptr, mpath_batch_line_get(mpath), start_index, len);
   }
@@ -155,6 +185,7 @@ static void motion_path_cache(OVERLAY_Data *vedata,
     DRW_shgroup_uniform_ivec4_copy(grp, "mpathPointSettings", motion_path_settings);
     DRW_shgroup_uniform_bool_copy(grp, "showKeyFrames", show_keyframes);
     DRW_shgroup_uniform_vec3_copy(grp, "customColor", color);
+    DRW_shgroup_uniform_mat4_copy(grp, "camera_space_matrix", camera_matrix);
     /* Only draw the required range. */
     DRW_shgroup_call_range(grp, nullptr, mpath_batch_points_get(mpath), start_index, len);
   }
@@ -168,17 +199,29 @@ static void motion_path_cache(OVERLAY_Data *vedata,
     UI_GetThemeColor3ubv(TH_VERTEX_SELECT, col_kf);
     col[3] = col_kf[3] = 255;
 
+    Object *cam_eval = nullptr;
+    if (motion_path_camera) {
+      cam_eval = DEG_get_evaluated_object(draw_ctx->depsgraph, motion_path_camera);
+    }
+
     bMotionPathVert *mpv = mpath->points + start_index;
     for (i = 0; i < len; i += stepsize, mpv += stepsize) {
       int frame = sfra + i;
       char numstr[32];
       size_t numstr_len;
       bool is_keyframe = (mpv->flag & MOTIONPATH_VERT_KEY) != 0;
+      float3 vert_coordinate;
+      copy_v3_v3(vert_coordinate, mpv->co);
+      if (cam_eval) {
+        /* Projecting the point into world space from the cameras pov. */
+        vert_coordinate = math::transform_point(float4x4(cam_eval->object_to_world),
+                                                vert_coordinate);
+      }
 
       if ((show_keyframes && show_keyframes_no && is_keyframe) || (show_frame_no && (i == 0))) {
         numstr_len = SNPRINTF_RLEN(numstr, " %d", frame);
         DRW_text_cache_add(
-            dt, mpv->co, numstr, numstr_len, 0, 0, txt_flag, (is_keyframe) ? col_kf : col);
+            dt, vert_coordinate, numstr, numstr_len, 0, 0, txt_flag, (is_keyframe) ? col_kf : col);
       }
       else if (show_frame_no) {
         bMotionPathVert *mpvP = (mpv - stepsize);
@@ -187,7 +230,7 @@ static void motion_path_cache(OVERLAY_Data *vedata,
          * don't occur on same point. */
         if ((equals_v3v3(mpv->co, mpvP->co) == 0) || (equals_v3v3(mpv->co, mpvN->co) == 0)) {
           numstr_len = SNPRINTF_RLEN(numstr, " %d", frame);
-          DRW_text_cache_add(dt, mpv->co, numstr, numstr_len, 0, 0, txt_flag, col);
+          DRW_text_cache_add(dt, vert_coordinate, numstr, numstr_len, 0, 0, txt_flag, col);
         }
       }
     }
