@@ -21,8 +21,9 @@
 #include "DNA_volume_types.h"
 
 #include "BKE_global.h"
-#include "BKE_volume.h"
-#include "BKE_volume_render.h"
+#include "BKE_volume.hh"
+#include "BKE_volume_grid_fwd.hh"
+#include "BKE_volume_render.hh"
 
 #include "GPU_batch.h"
 #include "GPU_capabilities.h"
@@ -30,11 +31,13 @@
 
 #include "DEG_depsgraph_query.hh"
 
-#include "DRW_render.h"
+#include "DRW_render.hh"
 
-#include "draw_cache.h"       /* own include */
+#include "draw_cache.hh"      /* own include */
 #include "draw_cache_impl.hh" /* own include */
 
+namespace blender::draw {
+
 static void volume_batch_cache_clear(Volume *volume);
 
 /* ---------------------------------------------------------------------- */
@@ -210,7 +213,7 @@ GPUBatch *DRW_volume_batch_cache_get_wireframes_face(Volume *volume)
   VolumeBatchCache *cache = volume_batch_cache_get(volume);
 
   if (cache->face_wire.batch == nullptr) {
-    const VolumeGrid *volume_grid = BKE_volume_grid_active_get_for_read(volume);
+    const bke::VolumeGridData *volume_grid = BKE_volume_grid_active_get_for_read(volume);
     if (volume_grid == nullptr) {
       return nullptr;
     }
@@ -259,7 +262,7 @@ GPUBatch *DRW_volume_batch_cache_get_selection_surface(Volume *volume)
 {
   VolumeBatchCache *cache = volume_batch_cache_get(volume);
   if (cache->selection_surface == nullptr) {
-    const VolumeGrid *volume_grid = BKE_volume_grid_active_get_for_read(volume);
+    const bke::VolumeGridData *volume_grid = BKE_volume_grid_active_get_for_read(volume);
     if (volume_grid == nullptr) {
       return nullptr;
     }
@@ -270,21 +273,21 @@ GPUBatch *DRW_volume_batch_cache_get_selection_surface(Volume *volume)
 }
 
 static DRWVolumeGrid *volume_grid_cache_get(const Volume *volume,
-                                            const VolumeGrid *grid,
+                                            const bke::VolumeGridData *grid,
                                             VolumeBatchCache *cache)
 {
-  const char *name = BKE_volume_grid_name(grid);
+  const std::string name = bke::volume_grid::get_name(*grid);
 
   /* Return cached grid. */
   LISTBASE_FOREACH (DRWVolumeGrid *, cache_grid, &cache->grids) {
-    if (STREQ(cache_grid->name, name)) {
+    if (cache_grid->name == name) {
       return cache_grid;
     }
   }
 
   /* Allocate new grid. */
   DRWVolumeGrid *cache_grid = MEM_cnew<DRWVolumeGrid>(__func__);
-  cache_grid->name = BLI_strdup(name);
+  cache_grid->name = BLI_strdup(name.c_str());
   BLI_addtail(&cache->grids, cache_grid);
 
   /* TODO: can we load this earlier, avoid accessing the global and take
@@ -292,14 +295,12 @@ static DRWVolumeGrid *volume_grid_cache_get(const Volume *volume,
   BKE_volume_load(volume, G.main);
 
   /* Test if we support textures with the number of channels. */
-  size_t channels = BKE_volume_grid_channels(grid);
+  size_t channels = bke::volume_grid::get_channels_num(bke::volume_grid::get_type(*grid));
   if (!ELEM(channels, 1, 3)) {
     return cache_grid;
   }
 
-  /* Remember if grid was loaded. If it was not, we want to unload it after the GPUTexture has been
-   * created. */
-  const bool was_loaded = BKE_volume_grid_is_loaded(grid);
+  const bool was_loaded = bke::volume_grid::is_loaded(*grid);
 
   DenseFloatVolumeGrid dense_grid;
   if (BKE_volume_grid_dense_floats(volume, grid, &dense_grid)) {
@@ -312,8 +313,7 @@ static DRWVolumeGrid *volume_grid_cache_get(const Volume *volume,
                                                 UNPACK3(dense_grid.resolution),
                                                 1,
                                                 format,
-                                                GPU_TEXTURE_USAGE_SHADER_READ |
-                                                    GPU_TEXTURE_USAGE_MIP_SWIZZLE_VIEW,
+                                                GPU_TEXTURE_USAGE_SHADER_READ,
                                                 dense_grid.voxels);
     /* The texture can be null if the resolution along one axis is larger than
      * GL_MAX_3D_TEXTURE_SIZE. */
@@ -330,20 +330,23 @@ static DRWVolumeGrid *volume_grid_cache_get(const Volume *volume,
 
   /* Free grid from memory if it wasn't previously loaded. */
   if (!was_loaded) {
-    BKE_volume_grid_unload(volume, grid);
+    bke::volume_grid::unload_tree_if_possible(*grid);
   }
 
   return cache_grid;
 }
 
-DRWVolumeGrid *DRW_volume_batch_cache_get_grid(Volume *volume, const VolumeGrid *volume_grid)
+DRWVolumeGrid *DRW_volume_batch_cache_get_grid(Volume *volume,
+                                               const bke::VolumeGridData *volume_grid)
 {
   VolumeBatchCache *cache = volume_batch_cache_get(volume);
   DRWVolumeGrid *grid = volume_grid_cache_get(volume, volume_grid, cache);
   return (grid->texture != nullptr) ? grid : nullptr;
 }
 
-int DRW_volume_material_count_get(Volume *volume)
+int DRW_volume_material_count_get(const Volume *volume)
 {
   return max_ii(1, volume->totcol);
 }
+
+}  // namespace blender::draw
