@@ -10,7 +10,7 @@
 
 #include "extract_mesh.hh"
 
-#include "draw_subdivision.h"
+#include "draw_subdivision.hh"
 
 namespace blender::draw {
 
@@ -87,20 +87,21 @@ static void extract_pos_nor_iter_face_mesh(const MeshRenderData &mr,
                                            void *_data)
 {
   MeshExtract_PosNor_Data *data = static_cast<MeshExtract_PosNor_Data *>(_data);
-  const bool poly_hidden = mr.hide_poly && mr.hide_poly[face_index];
+  const bool poly_hidden = !mr.hide_poly.is_empty() && mr.hide_poly[face_index];
 
-  for (const int ml_index : mr.faces[face_index]) {
-    const int vert_i = mr.corner_verts[ml_index];
-    PosNorLoop *vert = &data->vbo_data[ml_index];
-    const bool vert_hidden = mr.hide_vert && mr.hide_vert[vert_i];
+  for (const int corner : mr.faces[face_index]) {
+    const int vert_i = mr.corner_verts[corner];
+    PosNorLoop *vert = &data->vbo_data[corner];
+    const bool vert_hidden = !mr.hide_vert.is_empty() && mr.hide_vert[vert_i];
     copy_v3_v3(vert->pos, mr.vert_positions[vert_i]);
     vert->nor = data->normals[vert_i].low;
     /* Flag for paint mode overlay. */
     if (poly_hidden || vert_hidden ||
-        ((mr.v_origindex) && (mr.v_origindex[vert_i] == ORIGINDEX_NONE))) {
+        ((mr.v_origindex) && (mr.v_origindex[vert_i] == ORIGINDEX_NONE)))
+    {
       vert->nor.w = -1;
     }
-    else if (mr.select_vert && mr.select_vert[vert_i]) {
+    else if (!mr.select_vert.is_empty() && mr.select_vert[vert_i]) {
       vert->nor.w = 1;
     }
     else {
@@ -130,8 +131,8 @@ static void extract_pos_nor_iter_loose_edge_mesh(const MeshRenderData &mr,
                                                  void *_data)
 {
   MeshExtract_PosNor_Data *data = static_cast<MeshExtract_PosNor_Data *>(_data);
-  const int ml_index = mr.loop_len + loose_edge_i * 2;
-  PosNorLoop *vert = &data->vbo_data[ml_index];
+  const int index = mr.loop_len + loose_edge_i * 2;
+  PosNorLoop *vert = &data->vbo_data[index];
   copy_v3_v3(vert[0].pos, mr.vert_positions[edge[0]]);
   copy_v3_v3(vert[1].pos, mr.vert_positions[edge[1]]);
   vert[0].nor = data->normals[edge[0]].low;
@@ -159,9 +160,9 @@ static void extract_pos_nor_iter_loose_vert_mesh(const MeshRenderData &mr,
   MeshExtract_PosNor_Data *data = static_cast<MeshExtract_PosNor_Data *>(_data);
   const int offset = mr.loop_len + (mr.edge_loose_len * 2);
 
-  const int ml_index = offset + loose_vert_i;
+  const int index = offset + loose_vert_i;
   const int v_index = mr.loose_verts[loose_vert_i];
-  PosNorLoop *vert = &data->vbo_data[ml_index];
+  PosNorLoop *vert = &data->vbo_data[index];
   copy_v3_v3(vert->pos, mr.vert_positions[v_index]);
   vert->nor = data->normals[v_index].low;
 }
@@ -199,12 +200,12 @@ static void extract_vertex_flags(const MeshRenderData &mr, char *flags)
 {
   for (int i = 0; i < mr.vert_len; i++) {
     char *flag = &flags[i];
-    const bool vert_hidden = mr.hide_vert && mr.hide_vert[i];
+    const bool vert_hidden = !mr.hide_vert.is_empty() && mr.hide_vert[i];
     /* Flag for paint mode overlay. */
     if (vert_hidden || ((mr.v_origindex) && (mr.v_origindex[i] == ORIGINDEX_NONE))) {
       *flag = -1;
     }
-    else if (mr.select_vert && mr.select_vert[i]) {
+    else if (!mr.select_vert.is_empty() && mr.select_vert[i]) {
       *flag = 1;
     }
     else {
@@ -258,18 +259,16 @@ static void extract_pos_nor_init_subdiv(const DRWSubdivCache &subdiv_cache,
   draw_subdiv_extract_pos_nor(subdiv_cache, flags_buffer, vbo, orco_vbo);
 
   if (subdiv_cache.use_custom_loop_normals) {
-    Mesh *coarse_mesh = subdiv_cache.mesh;
-    const float(*loop_normals)[3] = static_cast<const float(*)[3]>(
-        CustomData_get_layer(&coarse_mesh->loop_data, CD_NORMAL));
-    BLI_assert(loop_normals != nullptr);
+    const Mesh *coarse_mesh = subdiv_cache.mesh;
+    const Span<float3> corner_normals = coarse_mesh->corner_normals();
 
     GPUVertBuf *src_custom_normals = GPU_vertbuf_calloc();
     GPU_vertbuf_init_with_format(src_custom_normals, get_custom_normals_format());
-    GPU_vertbuf_data_alloc(src_custom_normals, coarse_mesh->totloop);
+    GPU_vertbuf_data_alloc(src_custom_normals, coarse_mesh->corners_num);
 
     memcpy(GPU_vertbuf_get_data(src_custom_normals),
-           loop_normals,
-           sizeof(float[3]) * coarse_mesh->totloop);
+           corner_normals.data(),
+           corner_normals.size_in_bytes());
 
     GPUVertBuf *dst_custom_normals = GPU_vertbuf_calloc();
     GPU_vertbuf_init_build_on_device(
@@ -331,7 +330,7 @@ static void extract_pos_nor_loose_geom_subdiv(const DRWSubdivCache &subdiv_cache
   /* Make sure buffer is active for sending loose data. */
   GPU_vertbuf_use(vbo);
 
-  blender::Span<DRWSubdivLooseEdge> loose_edges = draw_subdiv_cache_get_loose_edges(subdiv_cache);
+  Span<DRWSubdivLooseEdge> loose_edges = draw_subdiv_cache_get_loose_edges(subdiv_cache);
 
   SubdivPosNorLoop edge_data[2];
   memset(edge_data, 0, sizeof(SubdivPosNorLoop) * 2);
@@ -350,8 +349,7 @@ static void extract_pos_nor_loose_geom_subdiv(const DRWSubdivCache &subdiv_cache
 
   SubdivPosNorLoop vert_data;
   memset(&vert_data, 0, sizeof(SubdivPosNorLoop));
-  blender::Span<DRWSubdivLooseVertex> loose_verts = draw_subdiv_cache_get_loose_verts(
-      subdiv_cache);
+  Span<DRWSubdivLooseVertex> loose_verts = draw_subdiv_cache_get_loose_verts(subdiv_cache);
 
   for (const DRWSubdivLooseVertex &loose_vert : loose_verts) {
     copy_v3_v3(vert_data.pos, loose_vert.co);
@@ -459,22 +457,23 @@ static void extract_pos_nor_hq_iter_face_mesh(const MeshRenderData &mr,
                                               void *_data)
 {
   MeshExtract_PosNorHQ_Data *data = static_cast<MeshExtract_PosNorHQ_Data *>(_data);
-  const bool poly_hidden = mr.hide_poly && mr.hide_poly[face_index];
+  const bool poly_hidden = !mr.hide_poly.is_empty() && mr.hide_poly[face_index];
 
-  for (const int ml_index : mr.faces[face_index]) {
-    const int vert_i = mr.corner_verts[ml_index];
+  for (const int corner : mr.faces[face_index]) {
+    const int vert_i = mr.corner_verts[corner];
 
-    const bool vert_hidden = mr.hide_vert && mr.hide_vert[vert_i];
-    PosNorHQLoop *vert = &data->vbo_data[ml_index];
+    const bool vert_hidden = !mr.hide_vert.is_empty() && mr.hide_vert[vert_i];
+    PosNorHQLoop *vert = &data->vbo_data[corner];
     copy_v3_v3(vert->pos, mr.vert_positions[vert_i]);
     copy_v3_v3_short(vert->nor, data->normals[vert_i].high);
 
     /* Flag for paint mode overlay. */
     if (poly_hidden || vert_hidden ||
-        ((mr.v_origindex) && (mr.v_origindex[vert_i] == ORIGINDEX_NONE))) {
+        ((mr.v_origindex) && (mr.v_origindex[vert_i] == ORIGINDEX_NONE)))
+    {
       vert->nor[3] = -1;
     }
-    else if (mr.select_vert && mr.select_vert[vert_i]) {
+    else if (!mr.select_vert.is_empty() && mr.select_vert[vert_i]) {
       vert->nor[3] = 1;
     }
     else {
@@ -505,8 +504,8 @@ static void extract_pos_nor_hq_iter_loose_edge_mesh(const MeshRenderData &mr,
                                                     void *_data)
 {
   MeshExtract_PosNorHQ_Data *data = static_cast<MeshExtract_PosNorHQ_Data *>(_data);
-  const int ml_index = mr.loop_len + loose_edge_i * 2;
-  PosNorHQLoop *vert = &data->vbo_data[ml_index];
+  const int index = mr.loop_len + loose_edge_i * 2;
+  PosNorHQLoop *vert = &data->vbo_data[index];
   copy_v3_v3(vert[0].pos, mr.vert_positions[edge[0]]);
   copy_v3_v3(vert[1].pos, mr.vert_positions[edge[1]]);
   copy_v3_v3_short(vert[0].nor, data->normals[edge[0]].high);
@@ -537,9 +536,9 @@ static void extract_pos_nor_hq_iter_loose_vert_mesh(const MeshRenderData &mr,
   MeshExtract_PosNorHQ_Data *data = static_cast<MeshExtract_PosNorHQ_Data *>(_data);
   const int offset = mr.loop_len + (mr.edge_loose_len * 2);
 
-  const int ml_index = offset + loose_vert_i;
+  const int index = offset + loose_vert_i;
   const int v_index = mr.loose_verts[loose_vert_i];
-  PosNorHQLoop *vert = &data->vbo_data[ml_index];
+  PosNorHQLoop *vert = &data->vbo_data[index];
   copy_v3_v3(vert->pos, mr.vert_positions[v_index]);
   copy_v3_v3_short(vert->nor, data->normals[v_index].high);
   vert->nor[3] = 0;
@@ -575,7 +574,7 @@ constexpr MeshExtract create_extractor_pos_nor_hq()
 
 /** \} */
 
-}  // namespace blender::draw
+const MeshExtract extract_pos_nor = create_extractor_pos_nor();
+const MeshExtract extract_pos_nor_hq = create_extractor_pos_nor_hq();
 
-const MeshExtract extract_pos_nor = blender::draw::create_extractor_pos_nor();
-const MeshExtract extract_pos_nor_hq = blender::draw::create_extractor_pos_nor_hq();
+}  // namespace blender::draw
