@@ -22,8 +22,8 @@
 
 #include "BLT_translation.h"
 
-#include "BKE_context.h"
-#include "BKE_idtype.h"
+#include "BKE_context.hh"
+#include "BKE_idtype.hh"
 #include "BKE_report.h"
 #include "BKE_screen.hh"
 
