@@ -22,11 +22,11 @@
 #include "BLI_utildefines.h"
 
 #include "BKE_animsys.h"
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_fcurve.h"
 #include "BKE_fcurve_driver.h"
 #include "BKE_global.h"
-#include "BKE_main.h"
+#include "BKE_main.hh"
 #include "BKE_nla.h"
 
 #include "DEG_depsgraph.hh"
@@ -34,6 +34,8 @@
 
 #include "ED_keyframing.hh"
 
+#include "ANIM_keyframing.hh"
+
 #include "UI_interface.hh"
 
 #include "RNA_access.hh"
@@ -63,12 +65,7 @@ static FCurve *ui_but_get_fcurve(
 
 void ui_but_anim_flag(uiBut *but, const AnimationEvalContext *anim_eval_context)
 {
-  AnimData *adt;
-  bAction *act;
-  FCurve *fcu;
-  bool driven;
-  bool special;
-
+  /* Clear the flags that this function might set. */
   but->flag &= ~(UI_BUT_ANIMATED | UI_BUT_ANIMATED_KEY | UI_BUT_DRIVEN);
   but->drawflag &= ~UI_BUT_ANIMATED_CHANGED;
 
@@ -76,41 +73,46 @@ void ui_but_anim_flag(uiBut *but, const AnimationEvalContext *anim_eval_context)
    *        itself (which are used to animate properties of the animation data).
    *        We count those as "animated" too for now
    */
-  fcu = ui_but_get_fcurve(but, &adt, &act, &driven, &special);
+  AnimData *adt;
+  bAction *act;
+  bool driven;
+  bool special;
+  FCurve *fcu = ui_but_get_fcurve(but, &adt, &act, &driven, &special);
 
-  if (fcu) {
-    if (!driven) {
-      /* Empty curves are ignored by the animation evaluation system. */
-      if (BKE_fcurve_is_empty(fcu)) {
-        return;
-      }
+  if (!fcu) {
+    return;
+  }
+  if (driven) {
+    but->flag |= UI_BUT_DRIVEN;
+    return;
+  }
 
-      but->flag |= UI_BUT_ANIMATED;
+  /* Empty curves are ignored by the animation evaluation system. */
+  if (BKE_fcurve_is_empty(fcu)) {
+    return;
+  }
 
-      /* #41525 - When the active action is a NLA strip being edited,
-       * we need to correct the frame number to "look inside" the
-       * remapped action
-       */
-      float cfra = anim_eval_context->eval_time;
-      if (adt) {
-        cfra = BKE_nla_tweakedit_remap(adt, cfra, NLATIME_CONVERT_UNMAP);
-      }
+  but->flag |= UI_BUT_ANIMATED;
 
-      if (fcurve_frame_has_keyframe(fcu, cfra)) {
-        but->flag |= UI_BUT_ANIMATED_KEY;
-      }
+  /* #41525 - When the active action is a NLA strip being edited,
+   * we need to correct the frame number to "look inside" the
+   * remapped action
+   */
+  float cfra = anim_eval_context->eval_time;
+  if (adt) {
+    cfra = BKE_nla_tweakedit_remap(adt, cfra, NLATIME_CONVERT_UNMAP);
+  }
 
-      /* XXX: this feature is totally broken and useless with NLA */
-      if (adt == nullptr || adt->nla_tracks.first == nullptr) {
-        const AnimationEvalContext remapped_context = BKE_animsys_eval_context_construct_at(
-            anim_eval_context, cfra);
-        if (fcurve_is_changed(but->rnapoin, but->rnaprop, fcu, &remapped_context)) {
-          but->drawflag |= UI_BUT_ANIMATED_CHANGED;
-        }
-      }
-    }
-    else {
-      but->flag |= UI_BUT_DRIVEN;
+  if (fcurve_frame_has_keyframe(fcu, cfra)) {
+    but->flag |= UI_BUT_ANIMATED_KEY;
+  }
+
+  /* XXX: this feature is totally broken and useless with NLA */
+  if (adt == nullptr || adt->nla_tracks.first == nullptr) {
+    const AnimationEvalContext remapped_context = BKE_animsys_eval_context_construct_at(
+        anim_eval_context, cfra);
+    if (fcurve_is_changed(but->rnapoin, but->rnaprop, fcu, &remapped_context)) {
+      but->drawflag |= UI_BUT_ANIMATED_CHANGED;
     }
   }
 }
@@ -235,7 +237,6 @@ bool ui_but_anim_expression_create(uiBut *but, const char *str)
   bContext *C = static_cast<bContext *>(but->block->evil_C);
   ID *id;
   FCurve *fcu;
-  char *path;
   bool ok = false;
 
   /* button must have RNA-pointer to a numeric-capable property */
@@ -267,13 +268,14 @@ bool ui_but_anim_expression_create(uiBut *but, const char *str)
   }
 
   /* get path */
-  path = RNA_path_from_ID_to_property(&but->rnapoin, but->rnaprop);
-  if (path == nullptr) {
+  const std::optional<std::string> path = RNA_path_from_ID_to_property(&but->rnapoin,
+                                                                       but->rnaprop);
+  if (!path) {
     return false;
   }
 
   /* create driver */
-  fcu = verify_driver_fcurve(id, path, but->rnaindex, DRIVER_FCURVE_KEYFRAMES);
+  fcu = verify_driver_fcurve(id, path->c_str(), but->rnaindex, DRIVER_FCURVE_KEYFRAMES);
   if (fcu) {
     ChannelDriver *driver = fcu->driver;
 
@@ -293,14 +295,13 @@ bool ui_but_anim_expression_create(uiBut *but, const char *str)
     }
   }
 
-  MEM_freeN(path);
-
   return ok;
 }
 
 void ui_but_anim_autokey(bContext *C, uiBut *but, Scene *scene, float cfra)
 {
-  ED_autokeyframe_property(C, scene, &but->rnapoin, but->rnaprop, but->rnaindex, cfra, true);
+  blender::animrig::autokeyframe_property(
+      C, scene, &but->rnapoin, but->rnaprop, but->rnaindex, cfra, true);
 }
 
 void ui_but_anim_copy_driver(bContext *C)
