@@ -14,7 +14,7 @@
 #include "BLI_utildefines.h"
 #include "MEM_guardedalloc.h"
 
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_screen.hh"
 
 #include "WM_api.hh"
