@@ -12,7 +12,6 @@
 
 #include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
@@ -20,17 +19,20 @@
 #include "DNA_world_types.h"
 
 #include "BLI_blenlib.h"
+#include "BLI_time.h"
 #include "BLI_utildefines.h"
 
-#include "BKE_DerivedMesh.h"
+#include "BKE_DerivedMesh.hh"
+#include "BKE_attribute.hh"
 #include "BKE_blender.h"
 #include "BKE_cdderivedmesh.h"
-#include "BKE_context.h"
+#include "BKE_context.hh"
+#include "BKE_customdata.hh"
 #include "BKE_global.h"
 #include "BKE_image.h"
 #include "BKE_material.h"
 #include "BKE_mesh.hh"
-#include "BKE_modifier.h"
+#include "BKE_modifier.hh"
 #include "BKE_multires.hh"
 #include "BKE_report.h"
 #include "BKE_scene.h"
@@ -41,10 +43,8 @@
 #include "RE_pipeline.h"
 #include "RE_texture.h"
 
-#include "PIL_time.h"
-
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
 
 #include "WM_api.hh"
 #include "WM_types.hh"
@@ -114,9 +114,10 @@ struct MultiresBakeJob {
 
 static bool multiresbake_check(bContext *C, wmOperator *op)
 {
+  using namespace blender;
   Scene *scene = CTX_data_scene(C);
   Object *ob;
-  Mesh *me;
+  Mesh *mesh;
   MultiresModifierData *mmd;
   bool ok = true;
   int a;
@@ -132,7 +133,7 @@ static bool multiresbake_check(bContext *C, wmOperator *op)
       break;
     }
 
-    me = (Mesh *)ob->data;
+    mesh = (Mesh *)ob->data;
     mmd = get_multires_modifier(scene, ob, false);
 
     /* Multi-resolution should be and be last in the stack */
@@ -157,16 +158,19 @@ static bool multiresbake_check(bContext *C, wmOperator *op)
       break;
     }
 
-    if (!CustomData_has_layer(&me->loop_data, CD_PROP_FLOAT2)) {
+    if (!CustomData_has_layer(&mesh->corner_data, CD_PROP_FLOAT2)) {
       BKE_report(op->reports, RPT_ERROR, "Mesh should be unwrapped before multires data baking");
 
       ok = false;
     }
     else {
-      const int *material_indices = BKE_mesh_material_indices(me);
-      a = me->faces_num;
+      const bke::AttributeAccessor attributes = mesh->attributes();
+      const VArraySpan material_indices = *attributes.lookup<int>("material_index",
+                                                                  bke::AttrDomain::Face);
+      a = mesh->faces_num;
       while (ok && a--) {
-        Image *ima = bake_object_image_get(ob, material_indices ? material_indices[a] : 0);
+        Image *ima = bake_object_image_get(ob,
+                                           material_indices.is_empty() ? 0 : material_indices[a]);
 
         if (!ima) {
           BKE_report(
@@ -221,22 +225,22 @@ static DerivedMesh *multiresbake_create_loresdm(Scene *scene, Object *ob, int *l
 {
   DerivedMesh *dm;
   MultiresModifierData *mmd = get_multires_modifier(scene, ob, false);
-  Mesh *me = (Mesh *)ob->data;
+  Mesh *mesh = (Mesh *)ob->data;
   MultiresModifierData tmp_mmd = blender::dna::shallow_copy(*mmd);
 
   *lvl = mmd->lvl;
 
   if (mmd->lvl == 0) {
-    DerivedMesh *cddm = CDDM_from_mesh(me);
+    DerivedMesh *cddm = CDDM_from_mesh(mesh);
     DM_set_only_copy(cddm, &CD_MASK_BAREMESH);
     return cddm;
   }
 
-  DerivedMesh *cddm = CDDM_from_mesh(me);
+  DerivedMesh *cddm = CDDM_from_mesh(mesh);
   DM_set_only_copy(cddm, &CD_MASK_BAREMESH);
   tmp_mmd.lvl = mmd->lvl;
   tmp_mmd.sculptlvl = mmd->lvl;
-  dm = multires_make_derived_from_derived(cddm, &tmp_mmd, scene, ob, MultiresFlags(0));
+  dm = multires_make_derived_from_derived(cddm, &tmp_mmd, scene, ob, MULTIRES_IGNORE_SIMPLIFY);
 
   cddm->release(cddm);
 
@@ -245,10 +249,10 @@ static DerivedMesh *multiresbake_create_loresdm(Scene *scene, Object *ob, int *l
 
 static DerivedMesh *multiresbake_create_hiresdm(Scene *scene, Object *ob, int *lvl)
 {
-  Mesh *me = (Mesh *)ob->data;
+  Mesh *mesh = (Mesh *)ob->data;
   MultiresModifierData *mmd = get_multires_modifier(scene, ob, false);
   MultiresModifierData tmp_mmd = blender::dna::shallow_copy(*mmd);
-  DerivedMesh *cddm = CDDM_from_mesh(me);
+  DerivedMesh *cddm = CDDM_from_mesh(mesh);
   DerivedMesh *dm;
 
   DM_set_only_copy(cddm, &CD_MASK_BAREMESH);
@@ -263,7 +267,7 @@ static DerivedMesh *multiresbake_create_hiresdm(Scene *scene, Object *ob, int *l
 
   tmp_mmd.lvl = mmd->totlvl;
   tmp_mmd.sculptlvl = mmd->totlvl;
-  dm = multires_make_derived_from_derived(cddm, &tmp_mmd, scene, ob, MultiresFlags(0));
+  dm = multires_make_derived_from_derived(cddm, &tmp_mmd, scene, ob, MULTIRES_IGNORE_SIMPLIFY);
   cddm->release(cddm);
 
   return dm;
@@ -347,7 +351,7 @@ static int multiresbake_image_exec_locked(bContext *C, wmOperator *op)
       ClearFlag clear_flag = ClearFlag(0);
 
       ob = base->object;
-      // me = (Mesh *)ob->data;
+      // mesh = (Mesh *)ob->data;
 
       if (scene->r.bake_mode == RE_BAKE_NORMALS) {
         clear_flag = CLEAR_TANGENT_NORMAL;
@@ -464,7 +468,7 @@ static void init_multiresbake_job(bContext *C, MultiresBakeJob *bkj)
   CTX_DATA_END;
 }
 
-static void multiresbake_startjob(void *bkv, bool *stop, bool *do_update, float *progress)
+static void multiresbake_startjob(void *bkv, wmJobWorkerStatus *worker_status)
 {
   MultiresBakeJob *bkj = static_cast<MultiresBakeJob *>(bkv);
   int baked_objects = 0, tot_obj;
@@ -510,9 +514,9 @@ static void multiresbake_startjob(void *bkv, bool *stop, bool *do_update, float
     bkr.tot_obj = tot_obj;
     bkr.baked_objects = baked_objects;
 
-    bkr.stop = stop;
-    bkr.do_update = do_update;
-    bkr.progress = progress;
+    bkr.stop = &worker_status->stop;
+    bkr.do_update = &worker_status->do_update;
+    bkr.progress = &worker_status->progress;
 
     bkr.bias = bkj->bias;
     bkr.number_of_rays = bkj->number_of_rays;
