@@ -1,4 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later */
+/* SPDX-FileCopyrightText: 2023 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 /** \file
  * \ingroup edsculpt
@@ -7,7 +9,8 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_bitmap.h"
-#include "BLI_math.h"
+#include "BLI_math_geom.h"
+#include "BLI_math_vector.h"
 
 #include "DNA_brush_types.h"
 #include "DNA_mesh_types.h"
@@ -15,33 +18,37 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 
-#include "RNA_access.h"
-#include "RNA_define.h"
-#include "RNA_enum_types.h"
+#include "RNA_access.hh"
+#include "RNA_define.hh"
+#include "RNA_enum_types.hh"
 
 #include "BKE_attribute.hh"
-#include "BKE_brush.h"
-#include "BKE_colortools.h"
-#include "BKE_context.h"
-#include "BKE_deform.h"
+#include "BKE_brush.hh"
+#include "BKE_colortools.hh"
+#include "BKE_context.hh"
+#include "BKE_deform.hh"
 #include "BKE_mesh.hh"
-#include "BKE_mesh_iterators.h"
-#include "BKE_mesh_runtime.h"
-#include "BKE_modifier.h"
+#include "BKE_mesh_iterators.hh"
+#include "BKE_mesh_runtime.hh"
+#include "BKE_modifier.hh"
+#include "BKE_object.hh"
 #include "BKE_object_deform.h"
-#include "BKE_paint.h"
+#include "BKE_paint.hh"
 #include "BKE_report.h"
 
-#include "DEG_depsgraph.h"
-#include "DEG_depsgraph_query.h"
+#include "DEG_depsgraph.hh"
+#include "DEG_depsgraph_query.hh"
 
-#include "WM_api.h"
-#include "WM_types.h"
+#include "WM_api.hh"
+#include "WM_types.hh"
 
-#include "ED_armature.h"
-#include "ED_mesh.h"
-#include "ED_screen.h"
-#include "ED_view3d.h"
+#include "ED_armature.hh"
+#include "ED_mesh.hh"
+#include "ED_screen.hh"
+#include "ED_view3d.hh"
+
+#include "UI_interface.hh"
+#include "UI_resources.hh"
 
 #include "paint_intern.hh" /* own include */
 
@@ -104,15 +111,15 @@ static int weight_from_bones_exec(bContext *C, wmOperator *op)
   Scene *scene = CTX_data_scene(C);
   Object *ob = CTX_data_active_object(C);
   Object *armob = BKE_modifiers_is_deformed_by_armature(ob);
-  Mesh *me = static_cast<Mesh *>(ob->data);
+  Mesh *mesh = static_cast<Mesh *>(ob->data);
   int type = RNA_enum_get(op->ptr, "type");
 
   ED_object_vgroup_calc_from_armature(
-      op->reports, depsgraph, scene, ob, armob, type, (me->symmetry & ME_SYMMETRY_X));
+      op->reports, depsgraph, scene, ob, armob, type, (mesh->symmetry & ME_SYMMETRY_X));
 
-  DEG_id_tag_update(&me->id, 0);
+  DEG_id_tag_update(&mesh->id, 0);
   DEG_relations_tag_update(CTX_data_main(C));
-  WM_event_add_notifier(C, NC_GEOM | ND_DATA, me);
+  WM_event_add_notifier(C, NC_GEOM | ND_DATA, mesh);
 
   return OPERATOR_FINISHED;
 }
@@ -163,16 +170,15 @@ void PAINT_OT_weight_from_bones(wmOperatorType *ot)
 static int weight_sample_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
-  ViewContext vc;
-  Mesh *me;
+  Mesh *mesh;
   bool changed = false;
 
-  ED_view3d_viewcontext_init(C, &vc, depsgraph);
-  me = BKE_mesh_from_object(vc.obact);
-  const MDeformVert *dvert = BKE_mesh_deform_verts(me);
+  ViewContext vc = ED_view3d_viewcontext_init(C, depsgraph);
+  mesh = BKE_mesh_from_object(vc.obact);
+  const MDeformVert *dvert = mesh->deform_verts().data();
 
-  if (me && dvert && vc.v3d && vc.rv3d && (me->vertex_group_active_index != 0)) {
-    const bool use_vert_sel = (me->editflag & ME_EDIT_PAINT_VERT_SEL) != 0;
+  if (mesh && dvert && vc.v3d && vc.rv3d && (mesh->vertex_group_active_index != 0)) {
+    const bool use_vert_sel = (mesh->editflag & ME_EDIT_PAINT_VERT_SEL) != 0;
     int v_idx_best = -1;
     uint index;
 
@@ -181,7 +187,8 @@ static int weight_sample_invoke(bContext *C, wmOperator *op, const wmEvent *even
 
     if (use_vert_sel) {
       if (ED_mesh_pick_vert(
-              C, vc.obact, event->mval, ED_MESH_PICK_DEFAULT_VERT_DIST, true, &index)) {
+              C, vc.obact, event->mval, ED_MESH_PICK_DEFAULT_VERT_DIST, true, &index))
+      {
         v_idx_best = index;
       }
     }
@@ -201,9 +208,9 @@ static int weight_sample_invoke(bContext *C, wmOperator *op, const wmEvent *even
     if (v_idx_best != -1) { /* should always be valid */
       ToolSettings *ts = vc.scene->toolsettings;
       Brush *brush = BKE_paint_brush(&ts->wpaint->paint);
-      const int vgroup_active = me->vertex_group_active_index - 1;
+      const int vgroup_active = mesh->vertex_group_active_index - 1;
       float vgroup_weight = BKE_defvert_find_weight(&dvert[v_idx_best], vgroup_active);
-      const int defbase_tot = BLI_listbase_count(&me->vertex_group_names);
+      const int defbase_tot = BLI_listbase_count(&mesh->vertex_group_names);
       bool use_lock_relative = ts->wpaint_lock_relative;
       bool *defbase_locked = nullptr, *defbase_unlocked = nullptr;
 
@@ -223,7 +230,7 @@ static int weight_sample_invoke(bContext *C, wmOperator *op, const wmEvent *even
             vc.obact, defbase_tot, &defbase_tot_sel);
 
         if (defbase_tot_sel > 1) {
-          if (ME_USING_MIRROR_X_VERTEX_GROUPS(me)) {
+          if (ME_USING_MIRROR_X_VERTEX_GROUPS(mesh)) {
             BKE_object_defgroup_mirror_selection(
                 vc.obact, defbase_tot, defbase_sel, defbase_sel, &defbase_tot_sel);
           }
@@ -278,7 +285,7 @@ void PAINT_OT_weight_sample(wmOperatorType *ot)
   ot->poll = weight_paint_mode_poll;
 
   /* flags */
-  ot->flag = OPTYPE_UNDO;
+  ot->flag = OPTYPE_UNDO | OPTYPE_DEPENDS_ON_CURSOR;
 }
 
 /** \} */
@@ -287,147 +294,107 @@ void PAINT_OT_weight_sample(wmOperatorType *ot)
 /** \name Weight Paint Sample Group Operator
  * \{ */
 
-/* samples cursor location, and gives menu with vertex groups to activate */
-static bool weight_paint_sample_enum_itemf__helper(const MDeformVert *dvert,
-                                                   const int defbase_tot,
-                                                   int *groups)
+/**
+ * Samples cursor location, and gives menu with vertex groups to activate.
+ * This function fills in used vertex-groups.
+ */
+static bool weight_paint_sample_mark_groups(const MDeformVert *dvert,
+                                            blender::MutableSpan<bool> groups)
 {
-  /* this func fills in used vgroup's */
   bool found = false;
   int i = dvert->totweight;
   MDeformWeight *dw;
   for (dw = dvert->dw; i > 0; dw++, i--) {
-    if (dw->def_nr < defbase_tot) {
-      groups[dw->def_nr] = true;
-      found = true;
+    if (UNLIKELY(dw->def_nr >= groups.size())) {
+      continue;
     }
+    groups[dw->def_nr] = true;
+    found = true;
   }
   return found;
 }
-static const EnumPropertyItem *weight_paint_sample_enum_itemf(bContext *C,
-                                                              PointerRNA * /*ptr*/,
-                                                              PropertyRNA * /*prop*/,
-                                                              bool *r_free)
+
+static int weight_sample_group_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-  if (C) {
-    wmWindow *win = CTX_wm_window(C);
-    if (win && win->eventstate) {
-      Depsgraph *depsgraph = CTX_data_depsgraph_pointer(C);
-      ViewContext vc;
-      Mesh *me;
+  Depsgraph *depsgraph = CTX_data_depsgraph_pointer(C);
+  ViewContext vc = ED_view3d_viewcontext_init(C, depsgraph);
+  BLI_assert(vc.v3d && vc.rv3d); /* Ensured by poll. */
 
-      ED_view3d_viewcontext_init(C, &vc, depsgraph);
-      me = BKE_mesh_from_object(vc.obact);
-      const blender::OffsetIndices polys = me->polys();
-      const blender::Span<int> corner_verts = me->corner_verts();
-      const MDeformVert *dverts = BKE_mesh_deform_verts(me);
+  Mesh *mesh = BKE_mesh_from_object(vc.obact);
+  const MDeformVert *dverts = mesh->deform_verts().data();
+  if (BLI_listbase_is_empty(&mesh->vertex_group_names) || (dverts == nullptr)) {
+    BKE_report(op->reports, RPT_WARNING, "No vertex group data");
+    return OPERATOR_CANCELLED;
+  }
 
-      if (me && dverts && vc.v3d && vc.rv3d && me->vertex_group_names.first) {
-        const int defbase_tot = BLI_listbase_count(&me->vertex_group_names);
-        const bool use_vert_sel = (me->editflag & ME_EDIT_PAINT_VERT_SEL) != 0;
-        int *groups = static_cast<int *>(MEM_callocN(defbase_tot * sizeof(int), "groups"));
-        bool found = false;
-        uint index;
+  const bool use_vert_sel = (mesh->editflag & ME_EDIT_PAINT_VERT_SEL) != 0;
+  blender::Array<bool> groups(BLI_listbase_count(&mesh->vertex_group_names), false);
 
-        const int mval[2] = {
-            win->eventstate->xy[0] - vc.region->winrct.xmin,
-            win->eventstate->xy[1] - vc.region->winrct.ymin,
-        };
+  bool found = false;
 
-        view3d_operator_needs_opengl(C);
-        ED_view3d_init_mats_rv3d(vc.obact, vc.rv3d);
+  view3d_operator_needs_opengl(C);
+  ED_view3d_init_mats_rv3d(vc.obact, vc.rv3d);
 
-        if (use_vert_sel) {
-          if (ED_mesh_pick_vert(C, vc.obact, mval, ED_MESH_PICK_DEFAULT_VERT_DIST, true, &index)) {
-            const MDeformVert *dvert = &dverts[index];
-            found |= weight_paint_sample_enum_itemf__helper(dvert, defbase_tot, groups);
-          }
-        }
-        else {
-          if (ED_mesh_pick_face(C, vc.obact, mval, ED_MESH_PICK_DEFAULT_FACE_DIST, &index)) {
-            const blender::IndexRange poly = polys[index];
-            int fidx = poly.size() - 1;
-
-            do {
-              const MDeformVert *dvert = &dverts[corner_verts[poly[fidx]]];
-              found |= weight_paint_sample_enum_itemf__helper(dvert, defbase_tot, groups);
-            } while (fidx--);
-          }
-        }
-
-        if (found == false) {
-          MEM_freeN(groups);
-        }
-        else {
-          EnumPropertyItem *item = nullptr, item_tmp = {0};
-          int totitem = 0;
-          int i = 0;
-          bDeformGroup *dg;
-          for (dg = static_cast<bDeformGroup *>(me->vertex_group_names.first);
-               dg && i < defbase_tot;
-               i++, dg = dg->next)
-          {
-            if (groups[i]) {
-              item_tmp.identifier = item_tmp.name = dg->name;
-              item_tmp.value = i;
-              RNA_enum_item_add(&item, &totitem, &item_tmp);
-            }
-          }
-
-          RNA_enum_item_end(&item, &totitem);
-          *r_free = true;
-
-          MEM_freeN(groups);
-          return item;
-        }
+  if (use_vert_sel) {
+    /* Extract from the vertex. */
+    uint index;
+    if (ED_mesh_pick_vert(C, vc.obact, event->mval, ED_MESH_PICK_DEFAULT_VERT_DIST, true, &index))
+    {
+      const MDeformVert *dvert = &dverts[index];
+      found |= weight_paint_sample_mark_groups(dvert, groups);
+    }
+  }
+  else {
+    /* Extract from the face. */
+    const blender::OffsetIndices faces = mesh->faces();
+    const blender::Span<int> corner_verts = mesh->corner_verts();
+    uint index;
+    if (ED_mesh_pick_face(C, vc.obact, event->mval, ED_MESH_PICK_DEFAULT_FACE_DIST, &index)) {
+      for (const int vert : corner_verts.slice(faces[index])) {
+        found |= weight_paint_sample_mark_groups(&dverts[vert], groups);
       }
     }
   }
 
-  return DummyRNA_NULL_items;
-}
+  if (found == false) {
+    BKE_report(op->reports, RPT_WARNING, "No vertex groups found");
+    return OPERATOR_CANCELLED;
+  }
 
-static int weight_sample_group_exec(bContext *C, wmOperator *op)
-{
-  int type = RNA_enum_get(op->ptr, "group");
-  Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
-  ViewContext vc;
-  ED_view3d_viewcontext_init(C, &vc, depsgraph);
+  uiPopupMenu *pup = UI_popup_menu_begin(
+      C, WM_operatortype_name(op->type, op->ptr).c_str(), ICON_NONE);
+  uiLayout *layout = UI_popup_menu_layout(pup);
+  wmOperatorType *ot = WM_operatortype_find("OBJECT_OT_vertex_group_set_active", false);
+  wmOperatorCallContext opcontext = WM_OP_EXEC_DEFAULT;
+  uiLayoutSetOperatorContext(layout, opcontext);
+  int i = 0;
+  LISTBASE_FOREACH_INDEX (bDeformGroup *, dg, &mesh->vertex_group_names, i) {
+    if (groups[i] == false) {
+      continue;
+    }
+    PointerRNA op_ptr;
+    uiItemFullO_ptr(
+        layout, ot, dg->name, ICON_NONE, nullptr, WM_OP_EXEC_DEFAULT, UI_ITEM_NONE, &op_ptr);
+    RNA_property_enum_set(&op_ptr, ot->prop, i);
+  }
+  UI_popup_menu_end(C, pup);
 
-  BLI_assert(type + 1 >= 0);
-  BKE_object_defgroup_active_index_set(vc.obact, type + 1);
-
-  DEG_id_tag_update(&vc.obact->id, ID_RECALC_GEOMETRY);
-  WM_event_add_notifier(C, NC_OBJECT | ND_DRAW, vc.obact);
-  return OPERATOR_FINISHED;
+  return OPERATOR_INTERFACE;
 }
 
 void PAINT_OT_weight_sample_group(wmOperatorType *ot)
 {
-  /* TODO: we could make this a menu into #OBJECT_OT_vertex_group_set_active
-   * rather than its own operator */
-
-  PropertyRNA *prop = nullptr;
-
   /* identifiers */
   ot->name = "Weight Paint Sample Group";
   ot->idname = "PAINT_OT_weight_sample_group";
   ot->description = "Select one of the vertex groups available under current mouse position";
 
   /* api callbacks */
-  ot->exec = weight_sample_group_exec;
-  ot->invoke = WM_menu_invoke;
-  ot->poll = weight_paint_mode_poll;
+  ot->invoke = weight_sample_group_invoke;
+  ot->poll = weight_paint_mode_region_view3d_poll;
 
   /* flags */
-  ot->flag = OPTYPE_UNDO;
-
-  /* Group to use (dynamic enum). */
-  prop = RNA_def_enum(
-      ot->srna, "group", DummyRNA_NULL_items, 0, "Group", "Vertex group to set as active");
-  RNA_def_enum_funcs(prop, weight_paint_sample_enum_itemf);
-  RNA_def_property_flag(prop, PROP_ENUM_NO_TRANSLATE);
-  ot->prop = prop;
+  ot->flag = OPTYPE_UNDO | OPTYPE_DEPENDS_ON_CURSOR;
 }
 
 /** \} */
@@ -439,62 +406,59 @@ void PAINT_OT_weight_sample_group(wmOperatorType *ot)
 /* fills in the selected faces with the current weight and vertex group */
 static bool weight_paint_set(Object *ob, float paintweight)
 {
-  Mesh *me = static_cast<Mesh *>(ob->data);
+  using namespace blender;
+  Mesh *mesh = static_cast<Mesh *>(ob->data);
   MDeformWeight *dw, *dw_prev;
   int vgroup_active, vgroup_mirror = -1;
-  const bool topology = (me->editflag & ME_EDIT_MIRROR_TOPO) != 0;
+  const bool topology = (mesh->editflag & ME_EDIT_MIRROR_TOPO) != 0;
 
   /* mutually exclusive, could be made into a */
-  const short paint_selmode = ME_EDIT_PAINT_SEL_MODE(me);
+  const short paint_selmode = ME_EDIT_PAINT_SEL_MODE(mesh);
 
-  const blender::OffsetIndices polys = me->polys();
-  const blender::Span<int> corner_verts = me->corner_verts();
-  MDeformVert *dvert = BKE_mesh_deform_verts_for_write(me);
+  const blender::OffsetIndices faces = mesh->faces();
+  const blender::Span<int> corner_verts = mesh->corner_verts();
+  MDeformVert *dvert = mesh->deform_verts_for_write().data();
 
-  if (me->totpoly == 0 || dvert == nullptr) {
+  if (mesh->faces_num == 0 || dvert == nullptr) {
     return false;
   }
 
   vgroup_active = BKE_object_defgroup_active_index_get(ob) - 1;
 
   /* if mirror painting, find the other group */
-  if (ME_USING_MIRROR_X_VERTEX_GROUPS(me)) {
+  if (ME_USING_MIRROR_X_VERTEX_GROUPS(mesh)) {
     vgroup_mirror = ED_wpaint_mirror_vgroup_ensure(ob, vgroup_active);
   }
 
   WPaintPrev wpp;
-  wpaint_prev_create(&wpp, dvert, me->totvert);
+  wpaint_prev_create(&wpp, dvert, mesh->verts_num);
 
-  const bool *select_vert = (const bool *)CustomData_get_layer_named(
-      &me->vdata, CD_PROP_BOOL, ".select_vert");
-  const bool *select_poly = (const bool *)CustomData_get_layer_named(
-      &me->pdata, CD_PROP_BOOL, ".select_poly");
+  const bke::AttributeAccessor attributes = mesh->attributes();
+  const VArraySpan select_vert = *attributes.lookup<bool>(".select_vert", bke::AttrDomain::Point);
+  const VArraySpan select_poly = *attributes.lookup<bool>(".select_poly", bke::AttrDomain::Face);
 
-  for (const int i : polys.index_range()) {
-    const blender::IndexRange poly = polys[i];
-    uint fidx = poly.size() - 1;
-
-    if ((paint_selmode == SCE_SELECT_FACE) && !(select_poly && select_poly[i])) {
+  for (const int i : faces.index_range()) {
+    if ((paint_selmode == SCE_SELECT_FACE) && !(!select_poly.is_empty() && select_poly[i])) {
       continue;
     }
 
-    do {
-      const int vidx = corner_verts[poly[fidx]];
-
-      if (!dvert[vidx].flag) {
-        if ((paint_selmode == SCE_SELECT_VERTEX) && !(select_vert && select_vert[vidx])) {
+    for (const int vert : corner_verts.slice(faces[i])) {
+      if (!dvert[vert].flag) {
+        if ((paint_selmode == SCE_SELECT_VERTEX) &&
+            !(!select_vert.is_empty() && select_vert[vert]))
+        {
           continue;
         }
 
-        dw = BKE_defvert_ensure_index(&dvert[vidx], vgroup_active);
+        dw = BKE_defvert_ensure_index(&dvert[vert], vgroup_active);
         if (dw) {
-          dw_prev = BKE_defvert_ensure_index(wpp.wpaint_prev + vidx, vgroup_active);
+          dw_prev = BKE_defvert_ensure_index(wpp.wpaint_prev + vert, vgroup_active);
           dw_prev->weight = dw->weight; /* set the undo weight */
           dw->weight = paintweight;
 
-          if (me->symmetry & ME_SYMMETRY_X) {
+          if (mesh->symmetry & ME_SYMMETRY_X) {
             /* x mirror painting */
-            int j = mesh_get_x_mirror_vert(ob, nullptr, vidx, topology);
+            int j = mesh_get_x_mirror_vert(ob, nullptr, vert, topology);
             if (j >= 0) {
               /* copy, not paint again */
               if (vgroup_mirror != -1) {
@@ -510,22 +474,21 @@ static bool weight_paint_set(Object *ob, float paintweight)
             }
           }
         }
-        dvert[vidx].flag = 1;
+        dvert[vert].flag = 1;
       }
-
-    } while (fidx--);
+    }
   }
 
   {
     MDeformVert *dv = dvert;
-    for (int index = me->totvert; index != 0; index--, dv++) {
+    for (int index = mesh->verts_num; index != 0; index--, dv++) {
       dv->flag = 0;
     }
   }
 
   wpaint_prev_destroy(&wpp);
 
-  DEG_id_tag_update(&me->id, 0);
+  DEG_id_tag_update(&mesh->id, 0);
 
   return true;
 }
@@ -558,7 +521,7 @@ void PAINT_OT_weight_set(wmOperatorType *ot)
 
   /* api callbacks */
   ot->exec = weight_paint_set_exec;
-  ot->poll = mask_paint_poll;
+  ot->poll = weight_paint_mode_poll;
 
   /* flags */
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -591,9 +554,9 @@ struct WPGradient_vertStoreBase {
 struct WPGradient_userData {
   ARegion *region;
   Scene *scene;
-  Mesh *me;
+  Mesh *mesh;
   MDeformVert *dvert;
-  const bool *select_vert;
+  blender::VArraySpan<bool> select_vert;
   blender::VArray<bool> hide_vert;
   Brush *brush;
   const float *sco_start; /* [2] */
@@ -670,12 +633,12 @@ static void gradientVert_update(WPGradient_userData *grad_data, int index)
   }
 }
 
-static void gradientVertUpdate__mapFunc(void *userData,
+static void gradientVertUpdate__mapFunc(void *user_data,
                                         int index,
                                         const float /*co*/[3],
                                         const float /*no*/[3])
 {
-  WPGradient_userData *grad_data = static_cast<WPGradient_userData *>(userData);
+  WPGradient_userData *grad_data = static_cast<WPGradient_userData *>(user_data);
   WPGradient_vertStore *vs = &grad_data->vert_cache->elem[index];
 
   if (vs->sco[0] == FLT_MAX) {
@@ -685,16 +648,17 @@ static void gradientVertUpdate__mapFunc(void *userData,
   gradientVert_update(grad_data, index);
 }
 
-static void gradientVertInit__mapFunc(void *userData,
+static void gradientVertInit__mapFunc(void *user_data,
                                       int index,
                                       const float co[3],
                                       const float /*no*/[3])
 {
-  WPGradient_userData *grad_data = static_cast<WPGradient_userData *>(userData);
+  WPGradient_userData *grad_data = static_cast<WPGradient_userData *>(user_data);
   WPGradient_vertStore *vs = &grad_data->vert_cache->elem[index];
 
   if (grad_data->hide_vert[index] ||
-      (grad_data->use_select && (grad_data->select_vert && !grad_data->select_vert[index])))
+      (grad_data->use_select &&
+       (!grad_data->select_vert.is_empty() && !grad_data->select_vert[index])))
   {
     copy_v2_fl(vs->sco, FLT_MAX);
     return;
@@ -758,11 +722,11 @@ static int paint_weight_gradient_modal(bContext *C, wmOperator *op, const wmEven
 
   if (ret & OPERATOR_CANCELLED) {
     if (vert_cache != nullptr) {
-      Mesh *me = static_cast<Mesh *>(ob->data);
+      Mesh *mesh = static_cast<Mesh *>(ob->data);
       if (vert_cache->wpp.wpaint_prev) {
-        MDeformVert *dvert = BKE_mesh_deform_verts_for_write(me);
-        BKE_defvert_array_free_elems(dvert, me->totvert);
-        BKE_defvert_array_copy(dvert, vert_cache->wpp.wpaint_prev, me->totvert);
+        MDeformVert *dvert = mesh->deform_verts_for_write().data();
+        BKE_defvert_array_free_elems(dvert, mesh->verts_num);
+        BKE_defvert_array_copy(dvert, vert_cache->wpp.wpaint_prev, mesh->verts_num);
         wpaint_prev_destroy(&vert_cache->wpp);
       }
       MEM_freeN(vert_cache);
@@ -781,13 +745,14 @@ static int paint_weight_gradient_modal(bContext *C, wmOperator *op, const wmEven
 
 static int paint_weight_gradient_exec(bContext *C, wmOperator *op)
 {
+  using namespace blender;
   wmGesture *gesture = static_cast<wmGesture *>(op->customdata);
   WPGradient_vertStoreBase *vert_cache;
   ARegion *region = CTX_wm_region(C);
   Scene *scene = CTX_data_scene(C);
   Object *ob = CTX_data_active_object(C);
-  Mesh *me = static_cast<Mesh *>(ob->data);
-  MDeformVert *dverts = BKE_mesh_deform_verts_for_write(me);
+  Mesh *mesh = static_cast<Mesh *>(ob->data);
+  MDeformVert *dverts = mesh->deform_verts_for_write().data();
   int x_start = RNA_int_get(op->ptr, "xstart");
   int y_start = RNA_int_get(op->ptr, "ystart");
   int x_end = RNA_int_get(op->ptr, "xend");
@@ -803,17 +768,17 @@ static int paint_weight_gradient_exec(bContext *C, wmOperator *op)
   if (is_interactive) {
     if (gesture->user_data.data == nullptr) {
       gesture->user_data.data = MEM_mallocN(sizeof(WPGradient_vertStoreBase) +
-                                                (sizeof(WPGradient_vertStore) * me->totvert),
+                                                (sizeof(WPGradient_vertStore) * mesh->verts_num),
                                             __func__);
       gesture->user_data.use_free = false;
       data.is_init = true;
 
       wpaint_prev_create(
-          &((WPGradient_vertStoreBase *)gesture->user_data.data)->wpp, dverts, me->totvert);
+          &((WPGradient_vertStoreBase *)gesture->user_data.data)->wpp, dverts, mesh->verts_num);
 
       /* On initialization only, convert face -> vert sel. */
-      if (me->editflag & ME_EDIT_PAINT_FACE_SEL) {
-        BKE_mesh_flush_select_from_polys(me);
+      if (mesh->editflag & ME_EDIT_PAINT_FACE_SEL) {
+        bke::mesh_select_face_flush(*mesh);
       }
     }
 
@@ -826,24 +791,24 @@ static int paint_weight_gradient_exec(bContext *C, wmOperator *op)
 
     data.is_init = true;
     vert_cache = static_cast<WPGradient_vertStoreBase *>(MEM_mallocN(
-        sizeof(WPGradient_vertStoreBase) + (sizeof(WPGradient_vertStore) * me->totvert),
+        sizeof(WPGradient_vertStoreBase) + (sizeof(WPGradient_vertStore) * mesh->verts_num),
         __func__));
   }
 
-  const blender::bke::AttributeAccessor attributes = me->attributes();
+  const blender::bke::AttributeAccessor attributes = mesh->attributes();
 
   data.region = region;
   data.scene = scene;
-  data.me = me;
+  data.mesh = mesh;
   data.dvert = dverts;
-  data.select_vert = (const bool *)CustomData_get_layer_named(
-      &me->vdata, CD_PROP_BOOL, ".select_vert");
-  data.hide_vert = *attributes.lookup_or_default<bool>(".hide_vert", ATTR_DOMAIN_POINT, false);
+  data.select_vert = *attributes.lookup<bool>(".select_vert", bke::AttrDomain::Point);
+  data.hide_vert = *attributes.lookup_or_default<bool>(
+      ".hide_vert", bke::AttrDomain::Point, false);
   data.sco_start = sco_start;
   data.sco_end = sco_end;
   data.sco_line_div = 1.0f / len_v2v2(sco_start, sco_end);
   data.def_nr = BKE_object_defgroup_active_index_get(ob) - 1;
-  data.use_select = (me->editflag & (ME_EDIT_PAINT_FACE_SEL | ME_EDIT_PAINT_VERT_SEL)) != 0;
+  data.use_select = (mesh->editflag & (ME_EDIT_PAINT_FACE_SEL | ME_EDIT_PAINT_VERT_SEL)) != 0;
   data.vert_cache = vert_cache;
   data.vert_visit = nullptr;
   data.type = RNA_enum_get(op->ptr, "type");
@@ -862,16 +827,10 @@ static int paint_weight_gradient_exec(bContext *C, wmOperator *op)
 
   ED_view3d_init_mats_rv3d(ob, static_cast<RegionView3D *>(region->regiondata));
 
-  Scene *scene_eval = DEG_get_evaluated_scene(depsgraph);
-  Object *ob_eval = DEG_get_evaluated_object(depsgraph, ob);
-
-  CustomData_MeshMasks cddata_masks = scene->customdata_mask;
-  cddata_masks.vmask |= CD_MASK_ORIGINDEX;
-  cddata_masks.emask |= CD_MASK_ORIGINDEX;
-  cddata_masks.pmask |= CD_MASK_ORIGINDEX;
-  Mesh *me_eval = mesh_get_eval_final(depsgraph, scene_eval, ob_eval, &cddata_masks);
+  const Object *ob_eval = DEG_get_evaluated_object(depsgraph, ob);
+  const Mesh *me_eval = BKE_object_get_evaluated_mesh(ob_eval);
   if (data.is_init) {
-    data.vert_visit = BLI_BITMAP_NEW(me->totvert, __func__);
+    data.vert_visit = BLI_BITMAP_NEW(mesh->verts_num, __func__);
 
     BKE_mesh_foreach_mapped_vert(me_eval, gradientVertInit__mapFunc, &data, MESH_FOREACH_NOP);
 
@@ -890,12 +849,12 @@ static int paint_weight_gradient_exec(bContext *C, wmOperator *op)
   }
 
   if (scene->toolsettings->auto_normalize) {
-    const int vgroup_num = BLI_listbase_count(&me->vertex_group_names);
+    const int vgroup_num = BLI_listbase_count(&mesh->vertex_group_names);
     bool *lock_flags = BKE_object_defgroup_lock_flags_get(ob, vgroup_num);
     bool *vgroup_validmap = BKE_object_defgroup_validmap_get(ob, vgroup_num);
     if (vgroup_validmap != nullptr) {
       MDeformVert *dvert = dverts;
-      for (int i = 0; i < me->totvert; i++) {
+      for (int i = 0; i < mesh->verts_num; i++) {
         if ((data.vert_cache->elem[i].flag & WPGradient_vertStore::VGRAD_STORE_IS_MODIFIED) != 0) {
           if (lock_flags != nullptr) {
             BKE_defvert_normalize_lock_map(
@@ -959,7 +918,7 @@ void PAINT_OT_weight_gradient(wmOperatorType *ot)
   ot->cancel = WM_gesture_straightline_cancel;
 
   /* flags */
-  ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+  ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_DEPENDS_ON_CURSOR;
 
   prop = RNA_def_enum(ot->srna, "type", gradient_types, 0, "Type", "");
   RNA_def_property_flag(prop, PROP_SKIP_SAVE);
