@@ -1,5 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2020 Blender Foundation */
+/* SPDX-FileCopyrightText: 2020 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 /** \file
  * \ingroup edsculpt
@@ -7,41 +8,49 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "BLI_math.h"
-#include "BLI_task.h"
+#include "BLI_math_matrix.h"
+#include "BLI_math_vector.h"
+#include "BLI_math_vector_types.hh"
+#include "BLI_span.hh"
 
-#include "DNA_meshdata_types.h"
-
-#include "BKE_brush.h"
-#include "BKE_context.h"
+#include "BKE_brush.hh"
+#include "BKE_context.hh"
 #include "BKE_kelvinlet.h"
-#include "BKE_paint.h"
-#include "BKE_pbvh.h"
+#include "BKE_layer.hh"
+#include "BKE_paint.hh"
+#include "BKE_pbvh_api.hh"
 
-#include "DEG_depsgraph.h"
+#include "DEG_depsgraph.hh"
 
-#include "WM_api.h"
-#include "WM_types.h"
+#include "WM_api.hh"
+#include "WM_types.hh"
 
-#include "ED_screen.h"
-#include "ED_sculpt.h"
-#include "ED_view3d.h"
+#include "ED_screen.hh"
+#include "ED_sculpt.hh"
+#include "ED_view3d.hh"
 #include "paint_intern.hh"
 #include "sculpt_intern.hh"
 
-#include "RNA_access.h"
-#include "RNA_define.h"
+#include "RNA_access.hh"
+#include "RNA_define.hh"
 
-#include "bmesh.h"
+#include "bmesh.hh"
 
 #include <cmath>
 #include <cstdlib>
 
-void ED_sculpt_init_transform(bContext *C, Object *ob, const int mval[2], const char *undo_name)
+using blender::float3;
+using blender::MutableSpan;
+
+void ED_sculpt_init_transform(bContext *C,
+                              Object *ob,
+                              const float mval_fl[2],
+                              const char *undo_name)
 {
+  using namespace blender::ed::sculpt_paint;
   Sculpt *sd = CTX_data_tool_settings(C)->sculpt;
   SculptSession *ss = ob->sculpt;
-  Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
+  Depsgraph *depsgraph = CTX_data_depsgraph_pointer(C);
 
   copy_v3_v3(ss->init_pivot_pos, ss->pivot_pos);
   copy_v4_v4(ss->init_pivot_rot, ss->pivot_rot);
@@ -51,14 +60,14 @@ void ED_sculpt_init_transform(bContext *C, Object *ob, const int mval[2], const
   copy_v4_v4(ss->prev_pivot_rot, ss->pivot_rot);
   copy_v3_v3(ss->prev_pivot_scale, ss->pivot_scale);
 
-  BKE_sculpt_update_object_for_edit(depsgraph, ob, false, false, false);
-  SCULPT_undo_push_begin_ex(ob, undo_name);
+  undo::push_begin_ex(ob, undo_name);
+  BKE_sculpt_update_object_for_edit(depsgraph, ob, false);
 
   ss->pivot_rot[3] = 1.0f;
 
   SCULPT_vertex_random_access_ensure(ss);
 
-  SCULPT_filter_cache_init(C, ob, sd, SCULPT_UNDO_COORDS, mval, 5.0, 1.0f);
+  filter::cache_init(C, ob, sd, undo::Type::Position, mval_fl, 5.0, 1.0f);
 
   if (sd->transform_mode == SCULPT_TRANSFORM_MODE_RADIUS_ELASTIC) {
     ss->filter_cache->transform_displacement_mode = SCULPT_TRANSFORM_DISPLACEMENT_INCREMENTAL;
@@ -132,26 +141,22 @@ static void sculpt_transform_matrices_init(SculptSession *ss,
   }
 }
 
-static void sculpt_transform_task_cb(void *__restrict userdata,
-                                     const int i,
-                                     const TaskParallelTLS *__restrict /*tls*/)
+static void sculpt_transform_task(Object *ob, const float transform_mats[8][4][4], PBVHNode *node)
 {
-
-  SculptThreadedTaskData *data = static_cast<SculptThreadedTaskData *>(userdata);
-  SculptSession *ss = data->ob->sculpt;
-  PBVHNode *node = data->nodes[i];
+  using namespace blender::ed::sculpt_paint;
+  SculptSession *ss = ob->sculpt;
 
   SculptOrigVertData orig_data;
-  SCULPT_orig_vert_data_init(&orig_data, data->ob, data->nodes[i], SCULPT_UNDO_COORDS);
+  SCULPT_orig_vert_data_init(&orig_data, ob, node, undo::Type::Position);
 
   PBVHVertexIter vd;
 
-  SCULPT_undo_push_node(data->ob, node, SCULPT_UNDO_COORDS);
+  undo::push_node(ob, node, undo::Type::Position);
   BKE_pbvh_vertex_iter_begin (ss->pbvh, node, vd, PBVH_ITER_UNIQUE) {
     SCULPT_orig_vert_data_update(&orig_data, &vd);
     float *start_co;
     float transformed_co[3], orig_co[3], disp[3];
-    float fade = vd.mask ? *vd.mask : 0.0f;
+    float fade = vd.mask;
     copy_v3_v3(orig_co, orig_data.co);
     char symm_area = SCULPT_get_vertex_symm_area(orig_co);
 
@@ -165,54 +170,48 @@ static void sculpt_transform_task_cb(void *__restrict userdata,
     }
 
     copy_v3_v3(transformed_co, start_co);
-    mul_m4_v3(data->transform_mats[int(symm_area)], transformed_co);
+    mul_m4_v3(transform_mats[int(symm_area)], transformed_co);
     sub_v3_v3v3(disp, transformed_co, start_co);
     mul_v3_fl(disp, 1.0f - fade);
     add_v3_v3v3(vd.co, start_co, disp);
-
-    if (vd.is_mesh) {
-      BKE_pbvh_vert_tag_update_normal(ss->pbvh, vd.vertex);
-    }
   }
   BKE_pbvh_vertex_iter_end;
 
   BKE_pbvh_node_mark_update(node);
 }
 
-static void sculpt_transform_all_vertices(Sculpt *sd, Object *ob)
+static void sculpt_transform_all_vertices(Object *ob)
 {
+  using namespace blender;
   SculptSession *ss = ob->sculpt;
   const ePaintSymmetryFlags symm = SCULPT_mesh_symmetry_xyz_get(ob);
 
-  SculptThreadedTaskData data{};
-  data.sd = sd;
-  data.ob = ob;
-  data.nodes = ss->filter_cache->nodes;
-
+  float transform_mats[8][4][4];
   sculpt_transform_matrices_init(
-      ss, symm, ss->filter_cache->transform_displacement_mode, data.transform_mats);
+      ss, symm, ss->filter_cache->transform_displacement_mode, transform_mats);
 
   /* Regular transform applies all symmetry passes at once as it is split by symmetry areas
    * (each vertex can only be transformed once by the transform matrix of its area). */
-  TaskParallelSettings settings;
-  BKE_pbvh_parallel_range_settings(&settings, true, ss->filter_cache->nodes.size());
-  BLI_task_parallel_range(
-      0, ss->filter_cache->nodes.size(), &data, sculpt_transform_task_cb, &settings);
+  threading::parallel_for(ss->filter_cache->nodes.index_range(), 1, [&](const IndexRange range) {
+    for (const int i : range) {
+      sculpt_transform_task(ob, transform_mats, ss->filter_cache->nodes[i]);
+    }
+  });
 }
 
-static void sculpt_elastic_transform_task_cb(void *__restrict userdata,
-                                             const int i,
-                                             const TaskParallelTLS *__restrict /*tls*/)
+static void sculpt_elastic_transform_task(Object *ob,
+                                          const float transform_radius,
+                                          const float elastic_transform_mat[4][4],
+                                          const float elastic_transform_pivot[3],
+                                          PBVHNode *node)
 {
+  using namespace blender::ed::sculpt_paint;
+  SculptSession *ss = ob->sculpt;
 
-  SculptThreadedTaskData *data = static_cast<SculptThreadedTaskData *>(userdata);
-  SculptSession *ss = data->ob->sculpt;
-  PBVHNode *node = data->nodes[i];
-
-  float(*proxy)[3] = BKE_pbvh_node_add_proxy(ss->pbvh, data->nodes[i])->co;
+  const MutableSpan<float3> proxy = BKE_pbvh_node_add_proxy(*ss->pbvh, *node).co;
 
   SculptOrigVertData orig_data;
-  SCULPT_orig_vert_data_init(&orig_data, data->ob, data->nodes[i], SCULPT_UNDO_COORDS);
+  SCULPT_orig_vert_data_init(&orig_data, ob, node, undo::Type::Position);
 
   KelvinletParams params;
   /* TODO(pablodp606): These parameters can be exposed if needed as transform strength and volume
@@ -221,31 +220,26 @@ static void sculpt_elastic_transform_task_cb(void *__restrict userdata,
   const float force = 1.0f;
   const float shear_modulus = 1.0f;
   const float poisson_ratio = 0.4f;
-  BKE_kelvinlet_init_params(
-      &params, data->elastic_transform_radius, force, shear_modulus, poisson_ratio);
+  BKE_kelvinlet_init_params(&params, transform_radius, force, shear_modulus, poisson_ratio);
 
-  SCULPT_undo_push_node(data->ob, node, SCULPT_UNDO_COORDS);
+  undo::push_node(ob, node, undo::Type::Position);
 
   PBVHVertexIter vd;
   BKE_pbvh_vertex_iter_begin (ss->pbvh, node, vd, PBVH_ITER_UNIQUE) {
     SCULPT_orig_vert_data_update(&orig_data, &vd);
     float transformed_co[3], orig_co[3], disp[3];
-    const float fade = vd.mask ? *vd.mask : 0.0f;
+    const float fade = vd.mask;
     copy_v3_v3(orig_co, orig_data.co);
 
     copy_v3_v3(transformed_co, vd.co);
-    mul_m4_v3(data->elastic_transform_mat, transformed_co);
+    mul_m4_v3(elastic_transform_mat, transformed_co);
     sub_v3_v3v3(disp, transformed_co, vd.co);
 
     float final_disp[3];
-    BKE_kelvinlet_grab_triscale(final_disp, &params, vd.co, data->elastic_transform_pivot, disp);
+    BKE_kelvinlet_grab_triscale(final_disp, &params, vd.co, elastic_transform_pivot, disp);
     mul_v3_fl(final_disp, 20.0f * (1.0f - fade));
 
     copy_v3_v3(proxy[vd.i], final_disp);
-
-    if (vd.is_mesh) {
-      BKE_pbvh_vert_tag_update_normal(ss->pbvh, vd.vertex);
-    }
   }
   BKE_pbvh_vertex_iter_end;
 
@@ -254,35 +248,39 @@ static void sculpt_elastic_transform_task_cb(void *__restrict userdata,
 
 static void sculpt_transform_radius_elastic(Sculpt *sd, Object *ob, const float transform_radius)
 {
+  using namespace blender;
   SculptSession *ss = ob->sculpt;
   BLI_assert(ss->filter_cache->transform_displacement_mode ==
              SCULPT_TRANSFORM_DISPLACEMENT_INCREMENTAL);
 
   const ePaintSymmetryFlags symm = SCULPT_mesh_symmetry_xyz_get(ob);
 
-  SculptThreadedTaskData data{};
-  data.sd = sd;
-  data.ob = ob;
-  data.nodes = ss->filter_cache->nodes;
-  data.elastic_transform_radius = transform_radius;
-
+  float transform_mats[8][4][4];
   sculpt_transform_matrices_init(
-      ss, symm, ss->filter_cache->transform_displacement_mode, data.transform_mats);
-
-  TaskParallelSettings settings;
-  BKE_pbvh_parallel_range_settings(&settings, true, ss->filter_cache->nodes.size());
+      ss, symm, ss->filter_cache->transform_displacement_mode, transform_mats);
 
   /* Elastic transform needs to apply all transform matrices to all vertices and then combine the
    * displacement proxies as all vertices are modified by all symmetry passes. */
   for (ePaintSymmetryFlags symmpass = PAINT_SYMM_NONE; symmpass <= symm; symmpass++) {
     if (SCULPT_is_symmetry_iteration_valid(symmpass, symm)) {
-      flip_v3_v3(data.elastic_transform_pivot, ss->pivot_pos, symmpass);
-      flip_v3_v3(data.elastic_transform_pivot_init, ss->init_pivot_pos, symmpass);
+      float elastic_transform_pivot[3];
+      flip_v3_v3(elastic_transform_pivot, ss->pivot_pos, symmpass);
+      float elastic_transform_pivot_init[3];
+      flip_v3_v3(elastic_transform_pivot_init, ss->init_pivot_pos, symmpass);
 
-      const int symm_area = SCULPT_get_vertex_symm_area(data.elastic_transform_pivot);
-      copy_m4_m4(data.elastic_transform_mat, data.transform_mats[symm_area]);
-      BLI_task_parallel_range(
-          0, ss->filter_cache->nodes.size(), &data, sculpt_elastic_transform_task_cb, &settings);
+      const int symm_area = SCULPT_get_vertex_symm_area(elastic_transform_pivot);
+      float elastic_transform_mat[4][4];
+      copy_m4_m4(elastic_transform_mat, transform_mats[symm_area]);
+      threading::parallel_for(
+          ss->filter_cache->nodes.index_range(), 1, [&](const IndexRange range) {
+            for (const int i : range) {
+              sculpt_elastic_transform_task(ob,
+                                            transform_radius,
+                                            elastic_transform_mat,
+                                            elastic_transform_pivot,
+                                            ss->filter_cache->nodes[i]);
+            }
+          });
     }
   }
   SCULPT_combine_transform_proxies(sd, ob);
@@ -295,11 +293,11 @@ void ED_sculpt_update_modal_transform(bContext *C, Object *ob)
   Depsgraph *depsgraph = CTX_data_depsgraph_pointer(C);
 
   SCULPT_vertex_random_access_ensure(ss);
-  BKE_sculpt_update_object_for_edit(depsgraph, ob, false, false, false);
+  BKE_sculpt_update_object_for_edit(depsgraph, ob, false);
 
   switch (sd->transform_mode) {
     case SCULPT_TRANSFORM_MODE_ALL_VERTICES: {
-      sculpt_transform_all_vertices(sd, ob);
+      sculpt_transform_all_vertices(ob);
       break;
     }
     case SCULPT_TRANSFORM_MODE_RADIUS_ELASTIC: {
@@ -311,9 +309,7 @@ void ED_sculpt_update_modal_transform(bContext *C, Object *ob)
         transform_radius = BKE_brush_unprojected_radius_get(scene, brush);
       }
       else {
-        ViewContext vc;
-
-        ED_view3d_viewcontext_init(C, &vc, depsgraph);
+        ViewContext vc = ED_view3d_viewcontext_init(C, depsgraph);
 
         transform_radius = paint_calc_object_space_radius(
             &vc, ss->init_pivot_pos, BKE_brush_size_get(scene, brush));
@@ -337,9 +333,10 @@ void ED_sculpt_update_modal_transform(bContext *C, Object *ob)
 
 void ED_sculpt_end_transform(bContext *C, Object *ob)
 {
+  using namespace blender::ed::sculpt_paint;
   SculptSession *ss = ob->sculpt;
   if (ss->filter_cache) {
-    SCULPT_filter_cache_free(ss);
+    filter::cache_free(ss);
   }
   SCULPT_flush_update_done(C, ob, SCULPT_UPDATE_COORDS);
 }
@@ -391,7 +388,13 @@ static int sculpt_set_pivot_position_exec(bContext *C, wmOperator *op)
 
   int mode = RNA_enum_get(op->ptr, "mode");
 
-  BKE_sculpt_update_object_for_edit(depsgraph, ob, false, true, false);
+  const View3D *v3d = CTX_wm_view3d(C);
+  const Base *base = CTX_data_active_base(C);
+  if (!BKE_base_is_visible(v3d, base)) {
+    return OPERATOR_CANCELLED;
+  }
+
+  BKE_sculpt_update_object_for_edit(depsgraph, ob, false);
 
   /* Pivot to center. */
   if (mode == SCULPT_PIVOT_POSITION_ORIGIN) {
@@ -413,7 +416,7 @@ static int sculpt_set_pivot_position_exec(bContext *C, wmOperator *op)
     }
   }
   else {
-    Vector<PBVHNode *> nodes = blender::bke::pbvh::search_gather(ss->pbvh, nullptr, nullptr);
+    blender::Vector<PBVHNode *> nodes = blender::bke::pbvh::search_gather(ss->pbvh, {});
 
     float avg[3];
     int total = 0;
@@ -424,7 +427,7 @@ static int sculpt_set_pivot_position_exec(bContext *C, wmOperator *op)
       for (PBVHNode *node : nodes) {
         PBVHVertexIter vd;
         BKE_pbvh_vertex_iter_begin (ss->pbvh, node, vd, PBVH_ITER_UNIQUE) {
-          const float mask = (vd.mask) ? *vd.mask : 0.0f;
+          const float mask = vd.mask;
           if (mask < 1.0f) {
             if (SCULPT_check_vertex_pivot_symmetry(vd.co, ss->pivot_pos, symm)) {
               add_v3_v3(avg, vd.co);
@@ -442,7 +445,7 @@ static int sculpt_set_pivot_position_exec(bContext *C, wmOperator *op)
       for (PBVHNode *node : nodes) {
         PBVHVertexIter vd;
         BKE_pbvh_vertex_iter_begin (ss->pbvh, node, vd, PBVH_ITER_UNIQUE) {
-          const float mask = (vd.mask) ? *vd.mask : 0.0f;
+          const float mask = vd.mask;
           if (mask < (0.5f + threshold) && mask > (0.5f - threshold)) {
             if (SCULPT_check_vertex_pivot_symmetry(vd.co, ss->pivot_pos, symm)) {
               add_v3_v3(avg, vd.co);
