@@ -28,13 +28,13 @@ StringRefNull TreeElementIDLibrary::get_warning() const
   Library &library = reinterpret_cast<Library &>(id_);
 
   if (library.tag & LIBRARY_TAG_RESYNC_REQUIRED) {
-    return TIP_(
+    return RPT_(
         "Contains linked library overrides that need to be resynced, updating the library is "
         "recommended");
   }
 
   if (library.id.tag & LIB_TAG_MISSING) {
-    return TIP_("Missing library");
+    return RPT_("Missing library");
   }
 
   return {};
