@@ -14,18 +14,18 @@
 #include "DNA_anim_types.h"
 #include "DNA_sequence_types.h"
 
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_fcurve.h"
 #include "BKE_scene.h"
 
-#include "BLF_api.h"
+#include "BLF_api.hh"
 
 #include "GPU_batch.h"
 #include "GPU_batch_utils.h"
 #include "GPU_immediate.h"
 #include "GPU_immediate_util.h"
 #include "GPU_matrix.h"
-#include "GPU_select.h"
+#include "GPU_select.hh"
 #include "GPU_state.h"
 
 #include "RNA_access.hh"
@@ -46,13 +46,14 @@
 #include "UI_resources.hh"
 #include "UI_view2d.hh"
 
-#include "SEQ_iterator.h"
+#include "SEQ_iterator.hh"
 #include "SEQ_retiming.hh"
-#include "SEQ_sequencer.h"
-#include "SEQ_time.h"
+#include "SEQ_sequencer.hh"
+#include "SEQ_time.hh"
 
 /* Own include. */
 #include "sequencer_intern.hh"
+#include "sequencer_quads_batch.hh"
 
 #define KEY_SIZE (10 * U.pixelsize)
 #define KEY_CENTER (UI_view2d_view_to_region_y(v2d, strip_y_rescale(seq, 0.0f)) + 4 + KEY_SIZE / 2)
@@ -154,8 +155,6 @@ static rctf keys_box_get(const bContext *C, const Sequence *seq)
   rctf rect = strip_box_get(C, seq);
   rect.ymax = KEY_CENTER + KEY_SIZE / 2;
   rect.ymin = KEY_CENTER - KEY_SIZE / 2;
-  rect.xmax += RETIME_KEY_MOUSEOVER_THRESHOLD;
-  rect.xmin -= RETIME_KEY_MOUSEOVER_THRESHOLD;
   return rect;
 }
 
@@ -169,7 +168,7 @@ int left_fake_key_frame_get(const bContext *C, const Sequence *seq)
 int right_fake_key_frame_get(const bContext *C, const Sequence *seq)
 {
   const Scene *scene = CTX_data_scene(C);
-  const int content_end = SEQ_time_content_end_frame_get(scene, seq);
+  const int content_end = SEQ_time_content_end_frame_get(scene, seq) - 1;
   return min_ii(content_end, SEQ_time_right_handle_frame_get(scene, seq));
 }
 
@@ -200,7 +199,14 @@ SeqRetimingKey *try_to_realize_virtual_key(const bContext *C, Sequence *seq, con
     int frame = SEQ_time_left_handle_frame_get(scene, seq);
     key = SEQ_retiming_add_key(scene, seq, frame);
   }
-  if (retiming_fake_key_is_clicked(C, seq, right_fake_key_frame_get(C, seq), mval)) {
+
+  int right_key_frame = right_fake_key_frame_get(C, seq);
+  /* `key_x_get()` compensates 1 frame offset of last key, however this can not
+   * be conveyed via `fake_key` alone. Therefore the same offset must be emulated. */
+  if (SEQ_time_right_handle_frame_get(scene, seq) >= SEQ_time_content_end_frame_get(scene, seq)) {
+    right_key_frame += 1;
+  }
+  if (retiming_fake_key_is_clicked(C, seq, right_key_frame, mval)) {
     SEQ_retiming_data_ensure(seq);
     const int frame = SEQ_time_right_handle_frame_get(scene, seq);
     key = SEQ_retiming_add_key(scene, seq, frame);
@@ -223,7 +229,14 @@ static SeqRetimingKey *mouse_over_key_get_from_strip(const bContext *C,
     int distance = round_fl_to_int(
         fabsf(UI_view2d_view_to_region_x(v2d, key_x_get(scene, seq, &key)) - mval[0]));
 
-    if (distance < RETIME_KEY_MOUSEOVER_THRESHOLD && distance < best_distance) {
+    int threshold = RETIME_KEY_MOUSEOVER_THRESHOLD;
+    if (key_x_get(scene, seq, &key) == SEQ_time_left_handle_frame_get(scene, seq) ||
+        key_x_get(scene, seq, &key) == SEQ_time_right_handle_frame_get(scene, seq))
+    {
+      threshold *= 2; /* Make first and last key easier to select. */
+    }
+
+    if (distance < threshold && distance < best_distance) {
       best_distance = distance;
       best_key = &key;
     }
@@ -235,7 +248,6 @@ static SeqRetimingKey *mouse_over_key_get_from_strip(const bContext *C,
 SeqRetimingKey *retiming_mousover_key_get(const bContext *C, const int mval[2], Sequence **r_seq)
 {
   for (Sequence *seq : sequencer_visible_strips_get(C)) {
-
     rctf box = keys_box_get(C, seq);
     if (!BLI_rctf_isect_pt(&box, mval[0], mval[1])) {
       continue;
@@ -261,7 +273,11 @@ SeqRetimingKey *retiming_mousover_key_get(const bContext *C, const int mval[2],
 /** \name Retiming Key
  * \{ */
 
-static void retime_key_draw(const bContext *C, const Sequence *seq, const SeqRetimingKey *key)
+static void retime_key_draw(const bContext *C,
+                            const Sequence *seq,
+                            const SeqRetimingKey *key,
+                            const KeyframeShaderBindings &sh_bindings,
+                            const blender::Map<SeqRetimingKey *, Sequence *> &selection)
 {
   const Scene *scene = CTX_data_scene(C);
   const float key_x = key_x_get(scene, seq, key);
@@ -272,24 +288,6 @@ static void retime_key_draw(const bContext *C, const Sequence *seq, const SeqRet
     return; /* Key out of the strip bounds. */
   }
 
-  GPUVertFormat *format = immVertexFormat();
-  KeyframeShaderBindings sh_bindings;
-
-  sh_bindings.pos_id = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
-  sh_bindings.size_id = GPU_vertformat_attr_add(format, "size", GPU_COMP_F32, 1, GPU_FETCH_FLOAT);
-  sh_bindings.color_id = GPU_vertformat_attr_add(
-      format, "color", GPU_COMP_U8, 4, GPU_FETCH_INT_TO_FLOAT_UNIT);
-  sh_bindings.outline_color_id = GPU_vertformat_attr_add(
-      format, "outlineColor", GPU_COMP_U8, 4, GPU_FETCH_INT_TO_FLOAT_UNIT);
-  sh_bindings.flags_id = GPU_vertformat_attr_add(format, "flags", GPU_COMP_U32, 1, GPU_FETCH_INT);
-
-  GPU_blend(GPU_BLEND_ALPHA);
-  GPU_program_point_size(true);
-  immBindBuiltinProgram(GPU_SHADER_KEYFRAME_SHAPE);
-  immUniform1f("outline_scale", 1.0f);
-  immUniform2f("ViewportSize", BLI_rcti_size_x(&v2d->mask) + 1, BLI_rcti_size_y(&v2d->mask) + 1);
-  immBegin(GPU_PRIM_POINTS, 1);
-
   eBezTriple_KeyframeType key_type = BEZT_KEYTYPE_KEYFRAME;
   if (SEQ_retiming_key_is_freeze_frame(key)) {
     key_type = BEZT_KEYTYPE_BREAKDOWN;
@@ -298,7 +296,7 @@ static void retime_key_draw(const bContext *C, const Sequence *seq, const SeqRet
     key_type = BEZT_KEYTYPE_MOVEHOLD;
   }
 
-  const bool is_selected = SEQ_retiming_selection_contains(SEQ_editing_get(scene), key);
+  const bool is_selected = selection.contains(const_cast<SeqRetimingKey *>(key));
   const int size = KEY_SIZE;
   const float bottom = KEY_CENTER;
 
@@ -323,17 +321,16 @@ static void retime_key_draw(const bContext *C, const Sequence *seq, const SeqRet
                       &sh_bindings,
                       0,
                       0);
-  immEnd();
-  GPU_program_point_size(false);
-  immUnbindProgram();
-  GPU_blend(GPU_BLEND_NONE);
 }
 
-static void draw_continuity(const bContext *C, const Sequence *seq, const SeqRetimingKey *key)
+static void draw_continuity(const bContext *C,
+                            const Sequence *seq,
+                            const SeqRetimingKey *key,
+                            const blender::Map<SeqRetimingKey *, Sequence *> &selection,
+                            SeqQuadsBatch *quads)
 {
   const View2D *v2d = UI_view2d_fromcontext(C);
   const Scene *scene = CTX_data_scene(C);
-  const Editing *ed = SEQ_editing_get(scene);
 
   if (key_x_get(scene, seq, key) == SEQ_time_left_handle_frame_get(scene, seq) ||
       key->strip_frame_index == 0)
@@ -358,51 +355,66 @@ static void draw_continuity(const bContext *C, const Sequence *seq, const SeqRet
   const float bottom = y_center - size * width_fac;
   const float top = y_center + size * width_fac;
 
-  GPU_blend(GPU_BLEND_ALPHA);
-  uint pos = GPU_vertformat_attr_add(immVertexFormat(), "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
-  immBindBuiltinProgram(GPU_SHADER_3D_UNIFORM_COLOR);
-
+  uchar color[4];
   if (SEQ_retiming_data_is_editable(seq) &&
-      (SEQ_retiming_selection_contains(ed, key) || SEQ_retiming_selection_contains(ed, key - 1)))
+      (selection.contains(const_cast<SeqRetimingKey *>(key)) ||
+       selection.contains(const_cast<SeqRetimingKey *>(key - 1))))
   {
-    immUniform4f("color", 0.65f, 0.5f, 0.2f, 1.0f);
+    color[0] = 166;
+    color[1] = 127;
+    color[2] = 51;
+    color[3] = 255;
   }
   else {
-    immUniform4f("color", 0.0f, 0.0f, 0.0f, 0.1f);
+    color[0] = 0;
+    color[1] = 0;
+    color[2] = 0;
+    color[3] = 25;
   }
-  immRectf(pos, prev_key_position, bottom, key_position, top);
-  immUnbindProgram();
-  GPU_blend(GPU_BLEND_NONE);
+  quads->add_quad(prev_key_position, bottom, key_position, top, color);
 }
 
 /* If there are no keys, draw fake keys and create real key when they are selected. */
 /* TODO: would be nice to draw continuity between fake keys. */
-static void fake_keys_draw(const bContext *C, Sequence *seq)
+static bool fake_keys_draw(const bContext *C,
+                           Sequence *seq,
+                           const KeyframeShaderBindings &sh_bindings,
+                           const blender::Map<SeqRetimingKey *, Sequence *> &selection)
 {
   if (!SEQ_retiming_is_active(seq) && !SEQ_retiming_data_is_editable(seq)) {
-    return;
+    return false;
   }
 
   const Scene *scene = CTX_data_scene(C);
   const int left_key_frame = left_fake_key_frame_get(C, seq);
-  const int right_key_frame = right_fake_key_frame_get(C, seq);
+  int right_key_frame = right_fake_key_frame_get(C, seq);
 
   if (SEQ_retiming_key_get_by_timeline_frame(scene, seq, left_key_frame) == nullptr) {
     SeqRetimingKey fake_key;
-    fake_key.strip_frame_index = left_key_frame - SEQ_time_start_frame_get(seq);
+    fake_key.strip_frame_index = (left_key_frame - SEQ_time_start_frame_get(seq)) *
+                                 SEQ_time_media_playback_rate_factor_get(scene, seq);
     fake_key.flag = 0;
-    retime_key_draw(C, seq, &fake_key);
+    retime_key_draw(C, seq, &fake_key, sh_bindings, selection);
   }
 
   if (SEQ_retiming_key_get_by_timeline_frame(scene, seq, right_key_frame) == nullptr) {
+    /* `key_x_get()` compensates 1 frame offset of last key, however this can not
+     * be conveyed via `fake_key` alone. Therefore the same offset must be emulated. */
+    if (SEQ_time_right_handle_frame_get(scene, seq) >= SEQ_time_content_end_frame_get(scene, seq))
+    {
+      right_key_frame += 1;
+    }
+
     SeqRetimingKey fake_key;
-    fake_key.strip_frame_index = right_key_frame - SEQ_time_start_frame_get(seq);
+    fake_key.strip_frame_index = (right_key_frame - SEQ_time_start_frame_get(seq)) *
+                                 SEQ_time_media_playback_rate_factor_get(scene, seq);
     fake_key.flag = 0;
-    retime_key_draw(C, seq, &fake_key);
+    retime_key_draw(C, seq, &fake_key, sh_bindings, selection);
   }
+  return true;
 }
 
-static void retime_keys_draw(const bContext *C)
+static void retime_keys_draw(const bContext *C, SeqQuadsBatch *quads)
 {
   const Scene *scene = CTX_data_scene(C);
   if (scene->ed == nullptr) {
@@ -413,22 +425,75 @@ static void retime_keys_draw(const bContext *C)
     return;
   }
 
+  /* Get the selection here once, for faster "is key selected?" lookups later. */
+  blender::Map<SeqRetimingKey *, Sequence *> selection = SEQ_retiming_selection_get(scene->ed);
+
   wmOrtho2_region_pixelspace(CTX_wm_region(C));
 
-  for (Sequence *seq : sequencer_visible_strips_get(C)) {
+  blender::Vector<Sequence *> strips = sequencer_visible_strips_get(C);
+
+  /* Draw all continuity sections. */
+  GPU_blend(GPU_BLEND_ALPHA);
+  for (Sequence *seq : strips) {
     if (!SEQ_retiming_is_allowed(seq)) {
       continue;
     }
 
     for (const SeqRetimingKey &key : SEQ_retiming_keys_get(seq)) {
-      draw_continuity(C, seq, &key);
-    }
-
-    fake_keys_draw(C, seq);
-    for (const SeqRetimingKey &key : SEQ_retiming_keys_get(seq)) {
-      retime_key_draw(C, seq, &key);
+      draw_continuity(C, seq, &key, selection, quads);
     }
   }
+  quads->draw();
+
+  /* Draw all keys. */
+  const View2D *v2d = UI_view2d_fromcontext(C);
+
+  GPUVertFormat *format = immVertexFormat();
+  KeyframeShaderBindings sh_bindings;
+
+  sh_bindings.pos_id = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
+  sh_bindings.size_id = GPU_vertformat_attr_add(format, "size", GPU_COMP_F32, 1, GPU_FETCH_FLOAT);
+  sh_bindings.color_id = GPU_vertformat_attr_add(
+      format, "color", GPU_COMP_U8, 4, GPU_FETCH_INT_TO_FLOAT_UNIT);
+  sh_bindings.outline_color_id = GPU_vertformat_attr_add(
+      format, "outlineColor", GPU_COMP_U8, 4, GPU_FETCH_INT_TO_FLOAT_UNIT);
+  sh_bindings.flags_id = GPU_vertformat_attr_add(format, "flags", GPU_COMP_U32, 1, GPU_FETCH_INT);
+
+  GPU_program_point_size(true);
+  immBindBuiltinProgram(GPU_SHADER_KEYFRAME_SHAPE);
+  immUniform1f("outline_scale", 1.0f);
+  immUniform2f("ViewportSize", BLI_rcti_size_x(&v2d->mask) + 1, BLI_rcti_size_y(&v2d->mask) + 1);
+
+  constexpr int MAX_KEYS_IN_BATCH = 1024;
+  int point_counter = 0;
+  immBeginAtMost(GPU_PRIM_POINTS, MAX_KEYS_IN_BATCH);
+
+  for (Sequence *seq : strips) {
+    if (!SEQ_retiming_is_allowed(seq)) {
+      continue;
+    }
+
+    if (fake_keys_draw(C, seq, sh_bindings, selection)) {
+      point_counter += 2;
+    }
+
+    for (const SeqRetimingKey &key : SEQ_retiming_keys_get(seq)) {
+      retime_key_draw(C, seq, &key, sh_bindings, selection);
+      point_counter++;
+
+      /* Next key plus possible two fake keys for next sequence would need at
+       * most 3 points, so restart the batch if we're close to that. */
+      if (point_counter + 3 >= MAX_KEYS_IN_BATCH) {
+        immEnd();
+        immBeginAtMost(GPU_PRIM_POINTS, MAX_KEYS_IN_BATCH);
+        point_counter = 0;
+      }
+    }
+  }
+  immEnd();
+  GPU_program_point_size(false);
+  immUnbindProgram();
+  GPU_blend(GPU_BLEND_NONE);
 }
 
 /** \} */
@@ -550,8 +615,8 @@ static void retime_speed_draw(const bContext *C)
 
 /** \} */
 
-void sequencer_draw_retiming(const bContext *C)
+void sequencer_draw_retiming(const bContext *C, SeqQuadsBatch *quads)
 {
-  retime_keys_draw(C);
+  retime_keys_draw(C, quads);
   retime_speed_draw(C);
 }
