@@ -1,4 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later */
+/* SPDX-FileCopyrightText: 2023 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "DNA_space_types.h"
 
@@ -7,6 +9,7 @@
 #include "BLI_color.hh"
 #include "BLI_cpp_type.hh"
 #include "BLI_hash.hh"
+#include "BLI_math_quaternion_types.hh"
 #include "BLI_math_vector_types.hh"
 #include "BLI_string.h"
 #include "BLI_string_ref.hh"
@@ -54,6 +57,9 @@ eSpreadsheetColumnValueType cpp_type_to_column_type(const CPPType &type)
   if (type.is<ColorGeometry4b>()) {
     return SPREADSHEET_VALUE_TYPE_BYTE_COLOR;
   }
+  if (type.is<math::Quaternion>()) {
+    return SPREADSHEET_VALUE_TYPE_QUATERNION;
+  }
 
   return SPREADSHEET_VALUE_TYPE_UNKNOWN;
 }
