@@ -21,7 +21,7 @@
 #include "BLI_string.h"
 
 #include "BKE_constraint.h"
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_nla.h"
 
 #include "RNA_access.hh"
@@ -534,12 +534,12 @@ void headerRotation(TransInfo *t, char *str, const int str_size, float final)
     outputNumInput(&(t->num), c, &t->scene->unit);
 
     ofs += BLI_snprintf_rlen(
-        str + ofs, str_size - ofs, TIP_("Rotation: %s %s %s"), &c[0], t->con.text, t->proptext);
+        str + ofs, str_size - ofs, IFACE_("Rotation: %s %s %s"), &c[0], t->con.text, t->proptext);
   }
   else {
     ofs += BLI_snprintf_rlen(str + ofs,
                              str_size - ofs,
-                             TIP_("Rotation: %.2f%s %s"),
+                             IFACE_("Rotation: %.2f%s %s"),
                              RAD2DEGF(final),
                              t->con.text,
                              t->proptext);
@@ -547,7 +547,7 @@ void headerRotation(TransInfo *t, char *str, const int str_size, float final)
 
   if (t->flag & T_PROP_EDIT_ALL) {
     ofs += BLI_snprintf_rlen(
-        str + ofs, str_size - ofs, TIP_(" Proportional size: %.2f"), t->prop_size);
+        str + ofs, str_size - ofs, IFACE_(" Proportional size: %.2f"), t->prop_size);
   }
 }
 
@@ -566,11 +566,17 @@ void ElementRotation_ex(const TransInfo *t,
 
     /* Apply gpencil falloff. */
     if (t->options & CTX_GPENCIL_STROKES) {
-      bGPDstroke *gps = (bGPDstroke *)td->extra;
-      if (gps->runtime.multi_frame_falloff != 1.0f) {
-        float ident_mat[3][3];
-        unit_m3(ident_mat);
-        interp_m3_m3m3(smat, ident_mat, smat, gps->runtime.multi_frame_falloff);
+      if (t->obedit_type == OB_GPENCIL_LEGACY) {
+
+        bGPDstroke *gps = (bGPDstroke *)td->extra;
+        if (gps->runtime.multi_frame_falloff != 1.0f) {
+          float ident_mat[3][3];
+          unit_m3(ident_mat);
+          interp_m3_m3m3(smat, ident_mat, smat, gps->runtime.multi_frame_falloff);
+        }
+      }
+      else if (t->obedit_type == OB_GREASE_PENCIL) {
+        /* pass */
       }
     }
 
@@ -841,13 +847,17 @@ void headerResize(TransInfo *t, const float vec[3], char *str, const int str_siz
   if (t->con.mode & CON_APPLY) {
     switch (t->num.idx_max) {
       case 0:
-        ofs += BLI_snprintf_rlen(
-            str + ofs, str_size - ofs, TIP_("Scale: %s%s %s"), &tvec[0], t->con.text, t->proptext);
+        ofs += BLI_snprintf_rlen(str + ofs,
+                                 str_size - ofs,
+                                 IFACE_("Scale: %s%s %s"),
+                                 &tvec[0],
+                                 t->con.text,
+                                 t->proptext);
         break;
       case 1:
         ofs += BLI_snprintf_rlen(str + ofs,
                                  str_size - ofs,
-                                 TIP_("Scale: %s : %s%s %s"),
+                                 IFACE_("Scale: %s : %s%s %s"),
                                  &tvec[0],
                                  &tvec[NUM_STR_REP_LEN],
                                  t->con.text,
@@ -856,7 +866,7 @@ void headerResize(TransInfo *t, const float vec[3], char *str, const int str_siz
       case 2:
         ofs += BLI_snprintf_rlen(str + ofs,
                                  str_size - ofs,
-                                 TIP_("Scale: %s : %s : %s%s %s"),
+                                 IFACE_("Scale: %s : %s : %s%s %s"),
                                  &tvec[0],
                                  &tvec[NUM_STR_REP_LEN],
                                  &tvec[NUM_STR_REP_LEN * 2],
@@ -869,7 +879,7 @@ void headerResize(TransInfo *t, const float vec[3], char *str, const int str_siz
     if (t->flag & T_2D_EDIT) {
       ofs += BLI_snprintf_rlen(str + ofs,
                                str_size - ofs,
-                               TIP_("Scale X: %s   Y: %s%s %s"),
+                               IFACE_("Scale X: %s   Y: %s%s %s"),
                                &tvec[0],
                                &tvec[NUM_STR_REP_LEN],
                                t->con.text,
@@ -878,7 +888,7 @@ void headerResize(TransInfo *t, const float vec[3], char *str, const int str_siz
     else {
       ofs += BLI_snprintf_rlen(str + ofs,
                                str_size - ofs,
-                               TIP_("Scale X: %s   Y: %s  Z: %s%s %s"),
+                               IFACE_("Scale X: %s   Y: %s  Z: %s%s %s"),
                                &tvec[0],
                                &tvec[NUM_STR_REP_LEN],
                                &tvec[NUM_STR_REP_LEN * 2],
@@ -889,7 +899,7 @@ void headerResize(TransInfo *t, const float vec[3], char *str, const int str_siz
 
   if (t->flag & T_PROP_EDIT_ALL) {
     ofs += BLI_snprintf_rlen(
-        str + ofs, str_size - ofs, TIP_(" Proportional size: %.2f"), t->prop_size);
+        str + ofs, str_size - ofs, IFACE_(" Proportional size: %.2f"), t->prop_size);
   }
 }
 
@@ -1030,21 +1040,26 @@ void ElementResize(const TransInfo *t,
    *   Operating on copies as a temporary solution.
    */
   if (t->options & CTX_GPENCIL_STROKES) {
-    bGPDstroke *gps = (bGPDstroke *)td->extra;
-    mul_v3_fl(vec, td->factor * gps->runtime.multi_frame_falloff);
+    if (t->obedit_type == OB_GPENCIL_LEGACY) {
+      bGPDstroke *gps = (bGPDstroke *)td->extra;
+      mul_v3_fl(vec, td->factor * gps->runtime.multi_frame_falloff);
 
-    /* Scale stroke thickness. */
-    if (td->val) {
-      NumInput num_evil = t->num;
-      float values_final_evil[4];
-      copy_v4_v4(values_final_evil, t->values_final);
-      transform_snap_increment(t, values_final_evil);
-      applyNumInput(&num_evil, values_final_evil);
+      /* Scale stroke thickness. */
+      if (td->val) {
+        NumInput num_evil = t->num;
+        float values_final_evil[4];
+        copy_v4_v4(values_final_evil, t->values_final);
+        transform_snap_increment(t, values_final_evil);
+        applyNumInput(&num_evil, values_final_evil);
 
-      float ratio = values_final_evil[0];
-      float transformed_value = td->ival * fabs(ratio);
-      *td->val = max_ff(interpf(transformed_value, td->ival, gps->runtime.multi_frame_falloff),
-                        0.001f);
+        float ratio = values_final_evil[0];
+        float transformed_value = td->ival * fabs(ratio);
+        *td->val = max_ff(interpf(transformed_value, td->ival, gps->runtime.multi_frame_falloff),
+                          0.001f);
+      }
+    }
+    else if (t->obedit_type == OB_GREASE_PENCIL) {
+      mul_v3_fl(vec, td->factor);
     }
   }
   else {
