@@ -23,16 +23,15 @@
 
 #include "DNA_image_types.h"
 #include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_node_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_space_types.h"
 
-#include "BKE_context.h"
-#include "BKE_customdata.h"
-#include "BKE_editmesh.h"
-#include "BKE_layer.h"
+#include "BKE_context.hh"
+#include "BKE_customdata.hh"
+#include "BKE_editmesh.hh"
+#include "BKE_layer.hh"
 #include "BKE_mesh.hh"
 #include "BKE_report.h"
 
@@ -52,10 +51,12 @@
 
 #include "UI_view2d.hh"
 
-#include "intern/bmesh_marking.h"
-#include "uvedit_intern.h"
+#include "intern/bmesh_marking.hh"
+#include "uvedit_intern.hh"
 
-#include "bmesh_tools.h"
+#include "bmesh_tools.hh"
+
+using blender::Vector;
 
 /* -------------------------------------------------------------------- */
 /** \name Path Select Struct & Properties
@@ -562,9 +563,8 @@ static int uv_shortest_path_pick_invoke(bContext *C, wmOperator *op, const wmEve
 
   Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
   ViewLayer *view_layer = CTX_data_view_layer(C);
-  uint objects_len = 0;
-  Object **objects = BKE_view_layer_array_from_objects_in_edit_mode_unique_data_with_uvs(
-      scene, view_layer, nullptr, &objects_len);
+  Vector<Object *> objects = BKE_view_layer_array_from_objects_in_edit_mode_unique_data_with_uvs(
+      scene, view_layer, nullptr);
 
   float co[2];
 
@@ -578,17 +578,17 @@ static int uv_shortest_path_pick_invoke(bContext *C, wmOperator *op, const wmEve
   UvNearestHit hit = uv_nearest_hit_init_max(&region->v2d);
   bool hit_found = false;
   if (uv_selectmode == UV_SELECT_FACE) {
-    if (uv_find_nearest_face_multi(scene, objects, objects_len, co, &hit)) {
+    if (uv_find_nearest_face_multi(scene, objects, co, &hit)) {
       hit_found = true;
     }
   }
   else if (uv_selectmode & UV_SELECT_EDGE) {
-    if (uv_find_nearest_edge_multi(scene, objects, objects_len, co, 0.0f, &hit)) {
+    if (uv_find_nearest_edge_multi(scene, objects, co, 0.0f, &hit)) {
       hit_found = true;
     }
   }
   else {
-    if (uv_find_nearest_vert_multi(scene, objects, objects_len, co, 0.0f, &hit)) {
+    if (uv_find_nearest_vert_multi(scene, objects, co, 0.0f, &hit)) {
       hit_found = true;
     }
   }
@@ -682,8 +682,6 @@ static int uv_shortest_path_pick_invoke(bContext *C, wmOperator *op, const wmEve
     }
   }
 
-  MEM_freeN(objects);
-
   return changed ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
 }
 
@@ -803,11 +801,9 @@ static int uv_shortest_path_select_exec(bContext *C, wmOperator *op)
   const float aspect_y = ED_uvedit_get_aspect_y(CTX_data_edit_object(C));
 
   ViewLayer *view_layer = CTX_data_view_layer(C);
-  uint objects_len = 0;
-  Object **objects = BKE_view_layer_array_from_objects_in_edit_mode_unique_data_with_uvs(
-      scene, view_layer, nullptr, &objects_len);
-  for (uint ob_index = 0; ob_index < objects_len; ob_index++) {
-    Object *obedit = objects[ob_index];
+  Vector<Object *> objects = BKE_view_layer_array_from_objects_in_edit_mode_unique_data_with_uvs(
+      scene, view_layer, nullptr);
+  for (Object *obedit : objects) {
     BMEditMesh *em = BKE_editmesh_from_object(obedit);
     BMesh *bm = em->bm;
 
@@ -846,7 +842,6 @@ static int uv_shortest_path_select_exec(bContext *C, wmOperator *op)
       found_valid_elements = true;
     }
   }
-  MEM_freeN(objects);
 
   if (!found_valid_elements) {
     BKE_report(
