@@ -37,7 +37,9 @@ int BinaryPredicate0D_Init(PyObject *module)
 
 //------------------------INSTANCE METHODS ----------------------------------
 
-static char BinaryPredicate0D___doc__[] =
+PyDoc_STRVAR(
+    /* Wrap. */
+    BinaryPredicate0D___doc__,
     "Base class for binary predicates working on :class:`Interface0D`\n"
     "objects. A BinaryPredicate0D is typically an ordering relation\n"
     "between two Interface0D objects. The predicate evaluates a relation\n"
@@ -58,7 +60,7 @@ static char BinaryPredicate0D___doc__[] =
     "   :arg inter2: The second Interface0D object.\n"
     "   :type inter2: :class:`Interface0D`\n"
     "   :return: True or false.\n"
-    "   :rtype: bool\n";
+    "   :rtype: bool\n");
 
 static int BinaryPredicate0D___init__(BPy_BinaryPredicate0D *self, PyObject *args, PyObject *kwds)
 {
@@ -112,10 +114,12 @@ static PyObject *BinaryPredicate0D___call__(BPy_BinaryPredicate0D *self,
 
 /*----------------------BinaryPredicate0D get/setters ----------------------------*/
 
-PyDoc_STRVAR(BinaryPredicate0D_name_doc,
-             "The name of the binary 0D predicate.\n"
-             "\n"
-             ":type: str");
+PyDoc_STRVAR(
+    /* Wrap. */
+    BinaryPredicate0D_name_doc,
+    "The name of the binary 0D predicate.\n"
+    "\n"
+    ":type: str");
 
 static PyObject *BinaryPredicate0D_name_get(BPy_BinaryPredicate0D *self, void * /*closure*/)
 {
