@@ -6,37 +6,34 @@
 
 #include "DNA_collection_types.h"
 #include "DNA_layer_types.h"
-#include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_pointcloud_types.h"
 
 #include "BLI_listbase.h"
 #include "BLI_math_matrix.hh"
+#include "BLI_math_rotation.hh"
 #include "BLI_noise.hh"
 #include "BLI_task.hh"
 
 #include "BKE_collection.h"
 #include "BKE_curves.hh"
-#include "BKE_deform.h"
+#include "BKE_customdata.hh"
 #include "BKE_geometry_set_instances.hh"
 #include "BKE_instances.hh"
 #include "BKE_material.h"
 #include "BKE_mesh.hh"
-#include "BKE_pointcloud.h"
+#include "BKE_pointcloud.hh"
 #include "BKE_type_conversions.hh"
 
 namespace blender::geometry {
 
+using blender::bke::AttrDomain;
 using blender::bke::AttributeIDRef;
 using blender::bke::AttributeKind;
 using blender::bke::AttributeMetaData;
-using blender::bke::custom_data_type_to_cpp_type;
-using blender::bke::CustomDataAttributes;
 using blender::bke::GSpanAttributeWriter;
 using blender::bke::InstanceReference;
 using blender::bke::Instances;
-using blender::bke::object_get_evaluated_geometry_set;
 using blender::bke::SpanAttributeWriter;
 
 /**
@@ -162,6 +159,9 @@ struct RealizeCurveInfo {
    * curves.
    */
   Span<float> nurbs_weight;
+
+  /** Custom normals are rotated based on each instance's transformation. */
+  Span<float3> custom_normal;
 };
 
 /** Start indices in the final output curves data-block. */
@@ -207,6 +207,7 @@ struct AllMeshesInfo {
   /** True if we know that there are no loose edges in any of the input meshes. */
   bool no_loose_edges_hint = false;
   bool no_loose_verts_hint = false;
+  bool no_overlapping_hint = false;
 };
 
 struct AllCurvesInfo {
@@ -221,6 +222,7 @@ struct AllCurvesInfo {
   bool create_radius_attribute = false;
   bool create_resolution_attribute = false;
   bool create_nurbs_weight_attribute = false;
+  bool create_custom_normal_attribute = false;
 };
 
 /** Collects all tasks that need to be executed to realize all instances. */
@@ -295,6 +297,23 @@ static void copy_transformed_positions(const Span<float3> src,
   });
 }
 
+static void copy_transformed_normals(const Span<float3> src,
+                                     const float4x4 &transform,
+                                     MutableSpan<float3> dst)
+{
+  const float3x3 normal_transform = math::transpose(math::invert(float3x3(transform)));
+  if (math::is_equal(normal_transform, float3x3::identity(), 1e-6f)) {
+    dst.copy_from(src);
+  }
+  else {
+    threading::parallel_for(src.index_range(), 1024, [&](const IndexRange range) {
+      for (const int i : range) {
+        dst[i] = normal_transform * src[i];
+      }
+    });
+  }
+}
+
 static void threaded_copy(const GSpan src, GMutableSpan dst)
 {
   BLI_assert(src.size() == dst.size());
@@ -316,13 +335,13 @@ static void copy_generic_attributes_to_result(
     const Span<std::optional<GVArraySpan>> src_attributes,
     const AttributeFallbacksArray &attribute_fallbacks,
     const OrderedAttributes &ordered_attributes,
-    const FunctionRef<IndexRange(eAttrDomain)> &range_fn,
+    const FunctionRef<IndexRange(bke::AttrDomain)> &range_fn,
     MutableSpan<GSpanAttributeWriter> dst_attribute_writers)
 {
   threading::parallel_for(
       dst_attribute_writers.index_range(), 10, [&](const IndexRange attribute_range) {
         for (const int attribute_index : attribute_range) {
-          const eAttrDomain domain = ordered_attributes.kinds[attribute_index].domain;
+          const bke::AttrDomain domain = ordered_attributes.kinds[attribute_index].domain;
           const IndexRange element_slice = range_fn(domain);
 
           GMutableSpan dst_span = dst_attribute_writers[attribute_index].span.slice(element_slice);
@@ -392,35 +411,37 @@ static Vector<std::pair<int, GSpan>> prepare_attribute_fallbacks(
     const OrderedAttributes &ordered_attributes)
 {
   Vector<std::pair<int, GSpan>> attributes_to_override;
-  const CustomDataAttributes &attributes = instances.custom_data_attributes();
-  attributes.foreach_attribute(
-      [&](const AttributeIDRef &attribute_id, const AttributeMetaData &meta_data) {
-        const int attribute_index = ordered_attributes.ids.index_of_try(attribute_id);
-        if (attribute_index == -1) {
-          /* The attribute is not propagated to the final geometry. */
-          return true;
-        }
-        GSpan span = *attributes.get_for_read(attribute_id);
-        const eCustomDataType expected_type = ordered_attributes.kinds[attribute_index].data_type;
-        if (meta_data.data_type != expected_type) {
-          const CPPType &from_type = span.type();
-          const CPPType &to_type = *custom_data_type_to_cpp_type(expected_type);
-          const bke::DataTypeConversions &conversions = bke::get_implicit_type_conversions();
-          if (!conversions.is_convertible(from_type, to_type)) {
-            /* Ignore the attribute because it can not be converted to the desired type. */
-            return true;
-          }
-          /* Convert the attribute on the instances component to the expected attribute type. */
-          std::unique_ptr<GArray<>> temporary_array = std::make_unique<GArray<>>(
-              to_type, instances.instances_num());
-          conversions.convert_to_initialized_n(span, temporary_array->as_mutable_span());
-          span = temporary_array->as_span();
-          gather_info.r_temporary_arrays.append(std::move(temporary_array));
-        }
-        attributes_to_override.append({attribute_index, span});
+  const bke::AttributeAccessor attributes = instances.attributes();
+  attributes.for_all([&](const AttributeIDRef &attribute_id, const AttributeMetaData &meta_data) {
+    const int attribute_index = ordered_attributes.ids.index_of_try(attribute_id);
+    if (attribute_index == -1) {
+      /* The attribute is not propagated to the final geometry. */
+      return true;
+    }
+    const bke::GAttributeReader attribute = attributes.lookup(attribute_id);
+    if (!attribute || !attribute.varray.is_span()) {
+      return true;
+    }
+    GSpan span = attribute.varray.get_internal_span();
+    const eCustomDataType expected_type = ordered_attributes.kinds[attribute_index].data_type;
+    if (meta_data.data_type != expected_type) {
+      const CPPType &from_type = span.type();
+      const CPPType &to_type = *bke::custom_data_type_to_cpp_type(expected_type);
+      const bke::DataTypeConversions &conversions = bke::get_implicit_type_conversions();
+      if (!conversions.is_convertible(from_type, to_type)) {
+        /* Ignore the attribute because it can not be converted to the desired type. */
         return true;
-      },
-      ATTR_DOMAIN_INSTANCE);
+      }
+      /* Convert the attribute on the instances component to the expected attribute type. */
+      std::unique_ptr<GArray<>> temporary_array = std::make_unique<GArray<>>(
+          to_type, instances.instances_num());
+      conversions.convert_to_initialized_n(span, temporary_array->as_mutable_span());
+      span = temporary_array->as_span();
+      gather_info.r_temporary_arrays.append(std::move(temporary_array));
+    }
+    attributes_to_override.append({attribute_index, span});
+    return true;
+  });
   return attributes_to_override;
 }
 
@@ -438,8 +459,8 @@ static void foreach_geometry_in_reference(
   switch (reference.type()) {
     case InstanceReference::Type::Object: {
       const Object &object = reference.object();
-      const bke::GeometrySet object_geometry_set = object_get_evaluated_geometry_set(object);
-      fn(object_geometry_set, base_transform, id);
+      const bke::GeometrySet object_geometry = bke::object_get_evaluated_geometry_set(object);
+      fn(object_geometry, base_transform, id);
       break;
     }
     case InstanceReference::Type::Collection: {
@@ -448,11 +469,11 @@ static void foreach_geometry_in_reference(
       offset_matrix.location() -= collection.instance_offset;
       int index = 0;
       FOREACH_COLLECTION_OBJECT_RECURSIVE_BEGIN (&collection, object) {
-        const bke::GeometrySet object_geometry_set = object_get_evaluated_geometry_set(*object);
+        const bke::GeometrySet object_geometry = bke::object_get_evaluated_geometry_set(*object);
         const float4x4 matrix = base_transform * offset_matrix *
                                 float4x4_view(object->object_to_world);
         const int sub_id = noise::hash(id, index);
-        fn(object_geometry_set, matrix, sub_id);
+        fn(object_geometry, matrix, sub_id);
         index++;
       }
       FOREACH_COLLECTION_OBJECT_RECURSIVE_END;
@@ -480,9 +501,9 @@ static void gather_realize_tasks_for_instances(GatherTasksInfo &gather_info,
 
   Span<int> stored_instance_ids;
   if (gather_info.create_id_attribute_on_any_component) {
-    std::optional<GSpan> ids = instances.custom_data_attributes().get_for_read("id");
-    if (ids.has_value()) {
-      stored_instance_ids = ids->typed<int>();
+    bke::AttributeReader ids = instances.attributes().lookup<int>("id");
+    if (ids) {
+      stored_instance_ids = ids.varray.get_internal_span();
     }
   }
 
@@ -554,7 +575,7 @@ static void gather_realize_tasks_recursive(GatherTasksInfo &gather_info,
         const bke::MeshComponent &mesh_component = *static_cast<const bke::MeshComponent *>(
             component);
         const Mesh *mesh = mesh_component.get();
-        if (mesh != nullptr && mesh->totvert > 0) {
+        if (mesh != nullptr && mesh->verts_num > 0) {
           const int mesh_index = gather_info.meshes.order.index_of(mesh);
           const MeshRealizeInfo &mesh_info = gather_info.meshes.realize_info[mesh_index];
           gather_info.r_tasks.mesh_tasks.append({gather_info.r_offsets.mesh_offsets,
@@ -562,9 +583,9 @@ static void gather_realize_tasks_recursive(GatherTasksInfo &gather_info,
                                                  base_transform,
                                                  base_instance_context.meshes,
                                                  base_instance_context.id});
-          gather_info.r_offsets.mesh_offsets.vertex += mesh->totvert;
-          gather_info.r_offsets.mesh_offsets.edge += mesh->totedge;
-          gather_info.r_offsets.mesh_offsets.loop += mesh->totloop;
+          gather_info.r_offsets.mesh_offsets.vertex += mesh->verts_num;
+          gather_info.r_offsets.mesh_offsets.edge += mesh->edges_num;
+          gather_info.r_offsets.mesh_offsets.loop += mesh->corners_num;
           gather_info.r_offsets.mesh_offsets.face += mesh->faces_num;
         }
         break;
@@ -615,7 +636,8 @@ static void gather_realize_tasks_recursive(GatherTasksInfo &gather_info,
         const auto *volume_component = static_cast<const bke::VolumeComponent *>(component);
         if (!gather_info.r_tasks.first_volume) {
           volume_component->add_user();
-          gather_info.r_tasks.first_volume = volume_component;
+          gather_info.r_tasks.first_volume = ImplicitSharingPtr<const bke::VolumeComponent>(
+              volume_component);
         }
         break;
       }
@@ -624,7 +646,8 @@ static void gather_realize_tasks_recursive(GatherTasksInfo &gather_info,
             component);
         if (!gather_info.r_tasks.first_edit_data) {
           edit_component->add_user();
-          gather_info.r_tasks.first_edit_data = edit_component;
+          gather_info.r_tasks.first_edit_data =
+              ImplicitSharingPtr<const bke::GeometryComponentEditData>(edit_component);
         }
         break;
       }
@@ -706,7 +729,7 @@ static AllPointCloudsInfo preprocess_pointclouds(const bke::GeometrySet &geometr
     for (const int attribute_index : info.attributes.index_range()) {
       const AttributeIDRef &attribute_id = info.attributes.ids[attribute_index];
       const eCustomDataType data_type = info.attributes.kinds[attribute_index].data_type;
-      const eAttrDomain domain = info.attributes.kinds[attribute_index].domain;
+      const bke::AttrDomain domain = info.attributes.kinds[attribute_index].domain;
       if (attributes.contains(attribute_id)) {
         GVArray attribute = *attributes.lookup_or_default(attribute_id, domain, data_type);
         pointcloud_info.attributes[attribute_index].emplace(std::move(attribute));
@@ -719,10 +742,11 @@ static AllPointCloudsInfo preprocess_pointclouds(const bke::GeometrySet &geometr
       }
     }
     if (info.create_radius_attribute) {
-      pointcloud_info.radii = *attributes.lookup_or_default("radius", ATTR_DOMAIN_POINT, 0.01f);
+      pointcloud_info.radii = *attributes.lookup_or_default(
+          "radius", bke::AttrDomain::Point, 0.01f);
     }
     const VArray<float3> position_attribute = *attributes.lookup_or_default<float3>(
-        "position", ATTR_DOMAIN_POINT, float3(0));
+        "position", bke::AttrDomain::Point, float3(0));
     pointcloud_info.positions = position_attribute.get_internal_span();
   }
   return info;
@@ -757,8 +781,8 @@ static void execute_realize_pointcloud_task(
       pointcloud_info.attributes,
       task.attribute_fallbacks,
       ordered_attributes,
-      [&](const eAttrDomain domain) {
-        BLI_assert(domain == ATTR_DOMAIN_POINT);
+      [&](const bke::AttrDomain domain) {
+        BLI_assert(domain == bke::AttrDomain::Point);
         UNUSED_VARS_NDEBUG(domain);
         return point_slice;
       },
@@ -790,17 +814,18 @@ static void execute_realize_pointcloud_tasks(const RealizeInstancesOptions &opti
   dst_pointcloud->totcol = first_pointcloud.totcol;
 
   SpanAttributeWriter<float3> positions = dst_attributes.lookup_or_add_for_write_only_span<float3>(
-      "position", ATTR_DOMAIN_POINT);
+      "position", bke::AttrDomain::Point);
 
   /* Prepare id attribute. */
   SpanAttributeWriter<int> point_ids;
   if (all_pointclouds_info.create_id_attribute) {
-    point_ids = dst_attributes.lookup_or_add_for_write_only_span<int>("id", ATTR_DOMAIN_POINT);
+    point_ids = dst_attributes.lookup_or_add_for_write_only_span<int>("id",
+                                                                      bke::AttrDomain::Point);
   }
   SpanAttributeWriter<float> point_radii;
   if (all_pointclouds_info.create_radius_attribute) {
     point_radii = dst_attributes.lookup_or_add_for_write_only_span<float>("radius",
-                                                                          ATTR_DOMAIN_POINT);
+                                                                          bke::AttrDomain::Point);
   }
 
   /* Prepare generic output attributes. */
@@ -809,7 +834,7 @@ static void execute_realize_pointcloud_tasks(const RealizeInstancesOptions &opti
     const AttributeIDRef &attribute_id = ordered_attributes.ids[attribute_index];
     const eCustomDataType data_type = ordered_attributes.kinds[attribute_index].data_type;
     dst_attribute_writers.append(dst_attributes.lookup_or_add_for_write_only_span(
-        attribute_id, ATTR_DOMAIN_POINT, data_type));
+        attribute_id, bke::AttrDomain::Point, data_type));
   }
 
   /* Actually execute all tasks. */
@@ -877,7 +902,7 @@ static void gather_meshes_to_realize(const bke::GeometrySet &geometry_set,
                                      VectorSet<const Mesh *> &r_meshes)
 {
   if (const Mesh *mesh = geometry_set.get_mesh()) {
-    if (mesh->totvert > 0) {
+    if (mesh->verts_num > 0) {
       r_meshes.add(mesh);
     }
   }
@@ -939,7 +964,7 @@ static AllMeshesInfo preprocess_meshes(const bke::GeometrySet &geometry_set,
     for (const int attribute_index : info.attributes.index_range()) {
       const AttributeIDRef &attribute_id = info.attributes.ids[attribute_index];
       const eCustomDataType data_type = info.attributes.kinds[attribute_index].data_type;
-      const eAttrDomain domain = info.attributes.kinds[attribute_index].domain;
+      const bke::AttrDomain domain = info.attributes.kinds[attribute_index].domain;
       if (attributes.contains(attribute_id)) {
         GVArray attribute = *attributes.lookup_or_default(attribute_id, domain, data_type);
         mesh_info.attributes[attribute_index].emplace(std::move(attribute));
@@ -952,7 +977,7 @@ static AllMeshesInfo preprocess_meshes(const bke::GeometrySet &geometry_set,
       }
     }
     mesh_info.material_indices = *attributes.lookup_or_default<int>(
-        "material_index", ATTR_DOMAIN_FACE, 0);
+        "material_index", bke::AttrDomain::Face, 0);
   }
 
   info.no_loose_edges_hint = std::all_of(
@@ -963,6 +988,10 @@ static AllMeshesInfo preprocess_meshes(const bke::GeometrySet &geometry_set,
       info.order.begin(), info.order.end(), [](const Mesh *mesh) {
         return mesh->runtime->loose_verts_cache.is_cached() && mesh->loose_verts().count == 0;
       });
+  info.no_overlapping_hint = std::all_of(
+      info.order.begin(), info.order.end(), [](const Mesh *mesh) {
+        return mesh->no_overlapping_topology();
+      });
 
   return info;
 }
@@ -1054,22 +1083,22 @@ static void execute_realize_mesh_task(const RealizeInstancesOptions &options,
     create_result_ids(options,
                       mesh_info.stored_vertex_ids,
                       task.id,
-                      all_dst_vertex_ids.slice(task.start_indices.vertex, mesh.totvert));
+                      all_dst_vertex_ids.slice(task.start_indices.vertex, mesh.verts_num));
   }
 
   copy_generic_attributes_to_result(
       mesh_info.attributes,
       task.attribute_fallbacks,
       ordered_attributes,
-      [&](const eAttrDomain domain) {
+      [&](const bke::AttrDomain domain) {
         switch (domain) {
-          case ATTR_DOMAIN_POINT:
+          case bke::AttrDomain::Point:
             return dst_vert_range;
-          case ATTR_DOMAIN_EDGE:
+          case bke::AttrDomain::Edge:
             return dst_edge_range;
-          case ATTR_DOMAIN_FACE:
+          case bke::AttrDomain::Face:
             return dst_face_range;
-          case ATTR_DOMAIN_CORNER:
+          case bke::AttrDomain::Corner:
             return dst_loop_range;
           default:
             BLI_assert_unreachable();
@@ -1092,9 +1121,9 @@ static void execute_realize_mesh_tasks(const RealizeInstancesOptions &options,
 
   const RealizeMeshTask &last_task = tasks.last();
   const Mesh &last_mesh = *last_task.mesh_info->mesh;
-  const int tot_vertices = last_task.start_indices.vertex + last_mesh.totvert;
-  const int tot_edges = last_task.start_indices.edge + last_mesh.totedge;
-  const int tot_loops = last_task.start_indices.loop + last_mesh.totloop;
+  const int tot_vertices = last_task.start_indices.vertex + last_mesh.verts_num;
+  const int tot_edges = last_task.start_indices.edge + last_mesh.edges_num;
+  const int tot_loops = last_task.start_indices.loop + last_mesh.corners_num;
   const int tot_faces = last_task.start_indices.face + last_mesh.faces_num;
 
   Mesh *dst_mesh = BKE_mesh_new_nomain(tot_vertices, tot_edges, tot_faces, tot_loops);
@@ -1123,25 +1152,41 @@ static void execute_realize_mesh_tasks(const RealizeInstancesOptions &options,
   /* Prepare id attribute. */
   SpanAttributeWriter<int> vertex_ids;
   if (all_meshes_info.create_id_attribute) {
-    vertex_ids = dst_attributes.lookup_or_add_for_write_only_span<int>("id", ATTR_DOMAIN_POINT);
+    vertex_ids = dst_attributes.lookup_or_add_for_write_only_span<int>("id",
+                                                                       bke::AttrDomain::Point);
   }
   /* Prepare material indices. */
   SpanAttributeWriter<int> material_indices;
   if (all_meshes_info.create_material_index_attribute) {
-    material_indices = dst_attributes.lookup_or_add_for_write_only_span<int>("material_index",
-                                                                             ATTR_DOMAIN_FACE);
+    material_indices = dst_attributes.lookup_or_add_for_write_only_span<int>(
+        "material_index", bke::AttrDomain::Face);
   }
 
   /* Prepare generic output attributes. */
   Vector<GSpanAttributeWriter> dst_attribute_writers;
   for (const int attribute_index : ordered_attributes.index_range()) {
     const AttributeIDRef &attribute_id = ordered_attributes.ids[attribute_index];
-    const eAttrDomain domain = ordered_attributes.kinds[attribute_index].domain;
+    const bke::AttrDomain domain = ordered_attributes.kinds[attribute_index].domain;
     const eCustomDataType data_type = ordered_attributes.kinds[attribute_index].data_type;
     dst_attribute_writers.append(
         dst_attributes.lookup_or_add_for_write_only_span(attribute_id, domain, data_type));
   }
-
+  const char *active_layer = CustomData_get_active_layer_name(&first_mesh.corner_data,
+                                                              CD_PROP_FLOAT2);
+  if (active_layer != nullptr) {
+    int id = CustomData_get_named_layer(&dst_mesh->corner_data, CD_PROP_FLOAT2, active_layer);
+    if (id >= 0) {
+      CustomData_set_layer_active(&dst_mesh->corner_data, CD_PROP_FLOAT2, id);
+    }
+  }
+  const char *render_layer = CustomData_get_render_layer_name(&first_mesh.corner_data,
+                                                              CD_PROP_FLOAT2);
+  if (render_layer != nullptr) {
+    int id = CustomData_get_named_layer(&dst_mesh->corner_data, CD_PROP_FLOAT2, render_layer);
+    if (id >= 0) {
+      CustomData_set_layer_render(&dst_mesh->corner_data, CD_PROP_FLOAT2, id);
+    }
+  }
   /* Actually execute all tasks. */
   threading::parallel_for(tasks.index_range(), 100, [&](const IndexRange task_range) {
     for (const int task_index : task_range) {
@@ -1173,6 +1218,9 @@ static void execute_realize_mesh_tasks(const RealizeInstancesOptions &options,
   if (all_meshes_info.no_loose_verts_hint) {
     dst_mesh->tag_loose_verts_none();
   }
+  if (all_meshes_info.no_overlapping_hint) {
+    dst_mesh->tag_overlapping_none();
+  }
 }
 
 /** \} */
@@ -1204,6 +1252,7 @@ static OrderedAttributes gather_generic_curve_attributes_to_propagate(
   attributes_to_propagate.remove("resolution");
   attributes_to_propagate.remove("handle_right");
   attributes_to_propagate.remove("handle_left");
+  attributes_to_propagate.remove("custom_normal");
   r_create_id = attributes_to_propagate.pop_try("id").has_value();
   OrderedAttributes ordered_attributes;
   for (const auto item : attributes_to_propagate.items()) {
@@ -1247,7 +1296,7 @@ static AllCurvesInfo preprocess_curves(const bke::GeometrySet &geometry_set,
     bke::AttributeAccessor attributes = curves.attributes();
     curve_info.attributes.reinitialize(info.attributes.size());
     for (const int attribute_index : info.attributes.index_range()) {
-      const eAttrDomain domain = info.attributes.kinds[attribute_index].domain;
+      const bke::AttrDomain domain = info.attributes.kinds[attribute_index].domain;
       const AttributeIDRef &attribute_id = info.attributes.ids[attribute_index];
       const eCustomDataType data_type = info.attributes.kinds[attribute_index].data_type;
       if (attributes.contains(attribute_id)) {
@@ -1264,12 +1313,12 @@ static AllCurvesInfo preprocess_curves(const bke::GeometrySet &geometry_set,
 
     if (attributes.contains("radius")) {
       curve_info.radius =
-          attributes.lookup<float>("radius", ATTR_DOMAIN_POINT).varray.get_internal_span();
+          attributes.lookup<float>("radius", bke::AttrDomain::Point).varray.get_internal_span();
       info.create_radius_attribute = true;
     }
     if (attributes.contains("nurbs_weight")) {
-      curve_info.nurbs_weight =
-          attributes.lookup<float>("nurbs_weight", ATTR_DOMAIN_POINT).varray.get_internal_span();
+      curve_info.nurbs_weight = attributes.lookup<float>("nurbs_weight", bke::AttrDomain::Point)
+                                    .varray.get_internal_span();
       info.create_nurbs_weight_attribute = true;
     }
     curve_info.resolution = curves.resolution();
@@ -1277,12 +1326,17 @@ static AllCurvesInfo preprocess_curves(const bke::GeometrySet &geometry_set,
       info.create_resolution_attribute = true;
     }
     if (attributes.contains("handle_right")) {
-      curve_info.handle_left =
-          attributes.lookup<float3>("handle_left", ATTR_DOMAIN_POINT).varray.get_internal_span();
-      curve_info.handle_right =
-          attributes.lookup<float3>("handle_right", ATTR_DOMAIN_POINT).varray.get_internal_span();
+      curve_info.handle_left = attributes.lookup<float3>("handle_left", bke::AttrDomain::Point)
+                                   .varray.get_internal_span();
+      curve_info.handle_right = attributes.lookup<float3>("handle_right", bke::AttrDomain::Point)
+                                    .varray.get_internal_span();
       info.create_handle_postion_attributes = true;
     }
+    if (attributes.contains("custom_normal")) {
+      curve_info.custom_normal = attributes.lookup<float3>("custom_normal", bke::AttrDomain::Point)
+                                     .varray.get_internal_span();
+      info.create_custom_normal_attribute = true;
+    }
   }
   return info;
 }
@@ -1298,7 +1352,8 @@ static void execute_realize_curve_task(const RealizeInstancesOptions &options,
                                        MutableSpan<float3> all_handle_right,
                                        MutableSpan<float> all_radii,
                                        MutableSpan<float> all_nurbs_weights,
-                                       MutableSpan<int> all_resolutions)
+                                       MutableSpan<int> all_resolutions,
+                                       MutableSpan<float3> all_custom_normals)
 {
   const RealizeCurveInfo &curves_info = *task.curve_info;
   const Curves &curves_id = *curves_info.curves;
@@ -1348,6 +1403,16 @@ static void execute_realize_curve_task(const RealizeInstancesOptions &options,
     curves_info.resolution.materialize(all_resolutions.slice(dst_curve_range));
   }
 
+  if (all_curves_info.create_custom_normal_attribute) {
+    if (curves_info.custom_normal.is_empty()) {
+      all_custom_normals.slice(dst_point_range).fill(float3(0, 0, 1));
+    }
+    else {
+      copy_transformed_normals(
+          curves_info.custom_normal, task.transform, all_custom_normals.slice(dst_point_range));
+    }
+  }
+
   /* Copy curve offsets. */
   const Span<int> src_offsets = curves.offsets();
   const MutableSpan<int> dst_offsets = dst_curves.offsets_for_write().slice(dst_curve_range);
@@ -1366,11 +1431,11 @@ static void execute_realize_curve_task(const RealizeInstancesOptions &options,
       curves_info.attributes,
       task.attribute_fallbacks,
       ordered_attributes,
-      [&](const eAttrDomain domain) {
+      [&](const bke::AttrDomain domain) {
         switch (domain) {
-          case ATTR_DOMAIN_POINT:
+          case bke::AttrDomain::Point:
             return IndexRange(task.start_indices.point, curves.points_num());
-          case ATTR_DOMAIN_CURVE:
+          case bke::AttrDomain::Curve:
             return IndexRange(task.start_indices.curve, curves.curves_num());
           default:
             BLI_assert_unreachable();
@@ -1410,14 +1475,15 @@ static void execute_realize_curve_tasks(const RealizeInstancesOptions &options,
   /* Prepare id attribute. */
   SpanAttributeWriter<int> point_ids;
   if (all_curves_info.create_id_attribute) {
-    point_ids = dst_attributes.lookup_or_add_for_write_only_span<int>("id", ATTR_DOMAIN_POINT);
+    point_ids = dst_attributes.lookup_or_add_for_write_only_span<int>("id",
+                                                                      bke::AttrDomain::Point);
   }
 
   /* Prepare generic output attributes. */
   Vector<GSpanAttributeWriter> dst_attribute_writers;
   for (const int attribute_index : ordered_attributes.index_range()) {
     const AttributeIDRef &attribute_id = ordered_attributes.ids[attribute_index];
-    const eAttrDomain domain = ordered_attributes.kinds[attribute_index].domain;
+    const bke::AttrDomain domain = ordered_attributes.kinds[attribute_index].domain;
     const eCustomDataType data_type = ordered_attributes.kinds[attribute_index].data_type;
     dst_attribute_writers.append(
         dst_attributes.lookup_or_add_for_write_only_span(attribute_id, domain, data_type));
@@ -1428,24 +1494,30 @@ static void execute_realize_curve_tasks(const RealizeInstancesOptions &options,
   SpanAttributeWriter<float3> handle_right;
   if (all_curves_info.create_handle_postion_attributes) {
     handle_left = dst_attributes.lookup_or_add_for_write_only_span<float3>("handle_left",
-                                                                           ATTR_DOMAIN_POINT);
-    handle_right = dst_attributes.lookup_or_add_for_write_only_span<float3>("handle_right",
-                                                                            ATTR_DOMAIN_POINT);
+                                                                           bke::AttrDomain::Point);
+    handle_right = dst_attributes.lookup_or_add_for_write_only_span<float3>(
+        "handle_right", bke::AttrDomain::Point);
   }
 
   SpanAttributeWriter<float> radius;
   if (all_curves_info.create_radius_attribute) {
-    radius = dst_attributes.lookup_or_add_for_write_only_span<float>("radius", ATTR_DOMAIN_POINT);
+    radius = dst_attributes.lookup_or_add_for_write_only_span<float>("radius",
+                                                                     bke::AttrDomain::Point);
   }
   SpanAttributeWriter<float> nurbs_weight;
   if (all_curves_info.create_nurbs_weight_attribute) {
     nurbs_weight = dst_attributes.lookup_or_add_for_write_only_span<float>("nurbs_weight",
-                                                                           ATTR_DOMAIN_POINT);
+                                                                           bke::AttrDomain::Point);
   }
   SpanAttributeWriter<int> resolution;
   if (all_curves_info.create_resolution_attribute) {
     resolution = dst_attributes.lookup_or_add_for_write_only_span<int>("resolution",
-                                                                       ATTR_DOMAIN_CURVE);
+                                                                       bke::AttrDomain::Curve);
+  }
+  SpanAttributeWriter<float3> custom_normal;
+  if (all_curves_info.create_custom_normal_attribute) {
+    custom_normal = dst_attributes.lookup_or_add_for_write_only_span<float3>(
+        "custom_normal", bke::AttrDomain::Point);
   }
 
   /* Actually execute all tasks. */
@@ -1463,7 +1535,8 @@ static void execute_realize_curve_tasks(const RealizeInstancesOptions &options,
                                  handle_right.span,
                                  radius.span,
                                  nurbs_weight.span,
-                                 resolution.span);
+                                 resolution.span,
+                                 custom_normal.span);
     }
   });
 
@@ -1486,6 +1559,7 @@ static void execute_realize_curve_tasks(const RealizeInstancesOptions &options,
   nurbs_weight.finish();
   handle_left.finish();
   handle_right.finish();
+  custom_normal.finish();
 }
 
 /** \} */
@@ -1498,7 +1572,7 @@ static void remove_id_attribute_from_instances(bke::GeometrySet &geometry_set)
 {
   geometry_set.modify_geometry_sets([&](bke::GeometrySet &sub_geometry) {
     if (Instances *instances = sub_geometry.get_instances_for_write()) {
-      instances->custom_data_attributes().remove("id");
+      instances->attributes_for_write().remove("id");
     }
   });
 }
