@@ -2,6 +2,7 @@
  *
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
+#include "BLI_array_utils.hh"
 #include "BLI_math_color.hh"
 #include "BLI_math_quaternion.hh"
 
@@ -99,13 +100,13 @@ static void retrieve_attribute_spans(const Span<bke::AttributeIDRef> ids,
 {
   const bke::AttributeAccessor src_attributes = src_curves.attributes();
   for (const int i : ids.index_range()) {
-    const GVArray src_attribute = *src_attributes.lookup(ids[i], ATTR_DOMAIN_POINT);
+    const GVArray src_attribute = *src_attributes.lookup(ids[i], bke::AttrDomain::Point);
     src.append(src_attribute.get_internal_span());
 
     const eCustomDataType data_type = bke::cpp_type_to_custom_data_type(src_attribute.type());
     bke::GSpanAttributeWriter dst_attribute =
         dst_curves.attributes_for_write().lookup_or_add_for_write_only_span(
-            ids[i], ATTR_DOMAIN_POINT, data_type);
+            ids[i], bke::AttrDomain::Point, data_type);
     dst.append(dst_attribute.span);
     dst_attributes.append(std::move(dst_attribute));
   }
@@ -139,7 +140,7 @@ static void gather_point_attributes_to_interpolate(
   VectorSet<bke::AttributeIDRef> ids_no_interpolation;
   src_curves.attributes().for_all(
       [&](const bke::AttributeIDRef &id, const bke::AttributeMetaData meta_data) {
-        if (meta_data.domain != ATTR_DOMAIN_POINT) {
+        if (meta_data.domain != bke::AttrDomain::Point) {
           return true;
         }
         if (meta_data.data_type == CD_PROP_STRING) {
@@ -177,14 +178,14 @@ static void gather_point_attributes_to_interpolate(
   if (output_ids.tangent_id) {
     result.src_evaluated_tangents = src_curves.evaluated_tangents();
     bke::GSpanAttributeWriter dst_attribute = dst_attributes.lookup_or_add_for_write_only_span(
-        output_ids.tangent_id, ATTR_DOMAIN_POINT, CD_PROP_FLOAT3);
+        output_ids.tangent_id, bke::AttrDomain::Point, CD_PROP_FLOAT3);
     result.dst_tangents = dst_attribute.span.typed<float3>();
     result.dst_attributes.append(std::move(dst_attribute));
   }
   if (output_ids.normal_id) {
     result.src_evaluated_normals = src_curves.evaluated_normals();
     bke::GSpanAttributeWriter dst_attribute = dst_attributes.lookup_or_add_for_write_only_span(
-        output_ids.normal_id, ATTR_DOMAIN_POINT, CD_PROP_FLOAT3);
+        output_ids.normal_id, bke::AttrDomain::Point, CD_PROP_FLOAT3);
     result.dst_normals = dst_attribute.span.typed<float3>();
     result.dst_attributes.append(std::move(dst_attribute));
   }
@@ -197,25 +198,25 @@ static void copy_or_defaults_for_unselected_curves(const CurvesGeometry &src_cur
 {
   const OffsetIndices src_points_by_curve = src_curves.points_by_curve();
   const OffsetIndices dst_points_by_curve = dst_curves.points_by_curve();
-  bke::curves::copy_point_data(src_points_by_curve,
-                               dst_points_by_curve,
-                               unselected_curves,
-                               src_curves.positions(),
-                               dst_curves.positions_for_write());
+  array_utils::copy_group_to_group(src_points_by_curve,
+                                   dst_points_by_curve,
+                                   unselected_curves,
+                                   src_curves.positions(),
+                                   dst_curves.positions_for_write());
 
   for (const int i : attributes.src.index_range()) {
-    bke::curves::copy_point_data(src_points_by_curve,
-                                 dst_points_by_curve,
-                                 unselected_curves,
-                                 attributes.src[i],
-                                 attributes.dst[i]);
+    array_utils::copy_group_to_group(src_points_by_curve,
+                                     dst_points_by_curve,
+                                     unselected_curves,
+                                     attributes.src[i],
+                                     attributes.dst[i]);
   }
   for (const int i : attributes.src_no_interpolation.index_range()) {
-    bke::curves::copy_point_data(src_points_by_curve,
-                                 dst_points_by_curve,
-                                 unselected_curves,
-                                 attributes.src_no_interpolation[i],
-                                 attributes.dst_no_interpolation[i]);
+    array_utils::copy_group_to_group(src_points_by_curve,
+                                     dst_points_by_curve,
+                                     unselected_curves,
+                                     attributes.src_no_interpolation[i],
+                                     attributes.dst_no_interpolation[i]);
   }
 
   if (!attributes.dst_tangents.is_empty()) {
@@ -245,6 +246,7 @@ static void normalize_curve_point_data(const IndexMaskSegment curve_selection,
 }
 
 static CurvesGeometry resample_to_uniform(const CurvesGeometry &src_curves,
+                                          const fn::FieldContext &field_context,
                                           const fn::Field<bool> &selection_field,
                                           const fn::Field<int> &count_field,
                                           const ResampleCurvesOutputAttributeIDs &output_ids)
@@ -261,7 +263,6 @@ static CurvesGeometry resample_to_uniform(const CurvesGeometry &src_curves,
   CurvesGeometry dst_curves = bke::curves::copy_only_curve_domain(src_curves);
   MutableSpan<int> dst_offsets = dst_curves.offsets_for_write();
 
-  const bke::CurvesFieldContext field_context{src_curves, ATTR_DOMAIN_CURVE};
   fn::FieldEvaluator evaluator{field_context, src_curves.curves_num()};
   evaluator.set_selection(selection_field);
   evaluator.add_with_destination(count_field, dst_offsets.drop_back(1));
@@ -394,24 +395,33 @@ static CurvesGeometry resample_to_uniform(const CurvesGeometry &src_curves,
 }
 
 CurvesGeometry resample_to_count(const CurvesGeometry &src_curves,
+                                 const fn::FieldContext &field_context,
                                  const fn::Field<bool> &selection_field,
                                  const fn::Field<int> &count_field,
                                  const ResampleCurvesOutputAttributeIDs &output_ids)
 {
-  return resample_to_uniform(
-      src_curves, selection_field, get_count_input_max_one(count_field), output_ids);
+  return resample_to_uniform(src_curves,
+                             field_context,
+                             selection_field,
+                             get_count_input_max_one(count_field),
+                             output_ids);
 }
 
 CurvesGeometry resample_to_length(const CurvesGeometry &src_curves,
+                                  const fn::FieldContext &field_context,
                                   const fn::Field<bool> &selection_field,
                                   const fn::Field<float> &segment_length_field,
                                   const ResampleCurvesOutputAttributeIDs &output_ids)
 {
-  return resample_to_uniform(
-      src_curves, selection_field, get_count_input_from_length(segment_length_field), output_ids);
+  return resample_to_uniform(src_curves,
+                             field_context,
+                             selection_field,
+                             get_count_input_from_length(segment_length_field),
+                             output_ids);
 }
 
 CurvesGeometry resample_to_evaluated(const CurvesGeometry &src_curves,
+                                     const fn::FieldContext &field_context,
                                      const fn::Field<bool> &selection_field,
                                      const ResampleCurvesOutputAttributeIDs &output_ids)
 {
@@ -422,7 +432,6 @@ CurvesGeometry resample_to_evaluated(const CurvesGeometry &src_curves,
   const OffsetIndices src_evaluated_points_by_curve = src_curves.evaluated_points_by_curve();
   const Span<float3> evaluated_positions = src_curves.evaluated_positions();
 
-  const bke::CurvesFieldContext field_context{src_curves, ATTR_DOMAIN_CURVE};
   fn::FieldEvaluator evaluator{field_context, src_curves.curves_num()};
   evaluator.set_selection(selection_field);
   evaluator.evaluate();
