@@ -10,7 +10,7 @@
 
 #pragma once
 
-#include "GPU_material.h"
+#include "GPU_material.hh"
 #include "GPU_shader.h"
 
 #ifdef __cplusplus
@@ -25,6 +25,7 @@ typedef struct GPUPass GPUPass;
 
 GPUPass *GPU_generate_pass(GPUMaterial *material,
                            struct GPUNodeGraph *graph,
+                           eGPUMaterialEngine engine,
                            GPUCodegenCallbackFn finalize_source_cb,
                            void *thunk,
                            bool optimize_graph);
