@@ -1,5 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2005 Blender Foundation */
+/* SPDX-FileCopyrightText: 2005 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 /** \file
  * \ingroup gpu
@@ -8,7 +9,8 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_math_matrix.h"
-#include "BLI_string_utils.h"
+#include "BLI_string.h"
+#include "BLI_string_utils.hh"
 
 #include "GPU_capabilities.h"
 #include "GPU_debug.h"
@@ -62,9 +64,11 @@ Shader::~Shader()
 
 static void standard_defines(Vector<const char *> &sources)
 {
-  BLI_assert(sources.size() == 0);
-  /* Version needs to be first. Exact values will be added by implementation. */
+  BLI_assert(sources.is_empty());
+  /* Version and specialization constants needs to be first.
+   * Exact values will be added by implementation. */
   sources.append("version");
+  sources.append("/* specialization_constants */");
   /* Define to identify code usage in shading language. */
   sources.append("#define GPU_SHADER\n");
   /* some useful defines to detect GPU type */
@@ -273,8 +277,8 @@ GPUShader *GPU_shader_create_from_info_name(const char *info_name)
   const GPUShaderCreateInfo *_info = gpu_shader_create_info_get(info_name);
   const ShaderCreateInfo &info = *reinterpret_cast<const ShaderCreateInfo *>(_info);
   if (!info.do_static_compilation_) {
-    printf("Warning: Trying to compile \"%s\" which was not marked for static compilation.\n",
-           info.name_.c_str());
+    std::cerr << "Warning: Trying to compile \"" << info.name_.c_str()
+              << "\" which was not marked for static compilation.\n";
   }
   return GPU_shader_create_from_info(_info);
 }
@@ -290,11 +294,13 @@ GPUShader *GPU_shader_create_from_info(const GPUShaderCreateInfo *_info)
 
   const std::string error = info.check_error();
   if (!error.empty()) {
-    printf("%s\n", error.c_str());
+    std::cerr << error.c_str() << "\n";
     BLI_assert(false);
   }
 
   Shader *shader = GPUBackend::get()->shader_alloc(info.name_.c_str());
+  shader->init(info);
+  shader->specialization_constants_init(info);
 
   std::string defines = shader->defines_declare(info);
   std::string resources = shader->resources_declare(info);
@@ -441,6 +447,12 @@ GPUShader *GPU_shader_create_from_python(const char *vertcode,
   return sh;
 }
 
+void GPU_shader_compile_static()
+{
+  printf("Compiling all static GPU shaders. This process takes a while.\n");
+  gpu_shader_create_info_compile("");
+}
+
 /** \} */
 
 /* -------------------------------------------------------------------- */
@@ -458,8 +470,13 @@ void GPU_shader_bind(GPUShader *gpu_shader)
     shader->bind();
     GPU_matrix_bind(gpu_shader);
     Shader::set_srgb_uniform(gpu_shader);
+    shader->constants.is_dirty = false;
   }
   else {
+    if (shader->constants.is_dirty) {
+      shader->bind();
+      shader->constants.is_dirty = false;
+    }
     if (Shader::srgb_uniform_dirty_get()) {
       Shader::set_srgb_uniform(gpu_shader);
     }
@@ -542,6 +559,68 @@ void GPU_shader_transform_feedback_disable(GPUShader *shader)
 
 /** \} */
 
+/* -------------------------------------------------------------------- */
+/** \name Assign specialization constants.
+ * \{ */
+
+void Shader::specialization_constants_init(const shader::ShaderCreateInfo &info)
+{
+  using namespace shader;
+  for (const ShaderCreateInfo::SpecializationConstant &sc : info.specialization_constants_) {
+    constants.types.append(sc.type);
+    constants.values.append(sc.default_value);
+  }
+  constants.is_dirty = true;
+}
+
+void GPU_shader_constant_int_ex(GPUShader *sh, int location, int value)
+{
+  Shader &shader = *unwrap(sh);
+  BLI_assert(shader.constants.types[location] == gpu::shader::Type::INT);
+  shader.constants.values[location].i = value;
+  shader.constants.is_dirty = true;
+}
+void GPU_shader_constant_uint_ex(GPUShader *sh, int location, uint value)
+{
+  Shader &shader = *unwrap(sh);
+  BLI_assert(shader.constants.types[location] == gpu::shader::Type::UINT);
+  shader.constants.values[location].u = value;
+  shader.constants.is_dirty = true;
+}
+void GPU_shader_constant_float_ex(GPUShader *sh, int location, float value)
+{
+  Shader &shader = *unwrap(sh);
+  BLI_assert(shader.constants.types[location] == gpu::shader::Type::FLOAT);
+  shader.constants.values[location].f = value;
+  shader.constants.is_dirty = true;
+}
+void GPU_shader_constant_bool_ex(GPUShader *sh, int location, bool value)
+{
+  Shader &shader = *unwrap(sh);
+  BLI_assert(shader.constants.types[location] == gpu::shader::Type::BOOL);
+  shader.constants.values[location].u = value;
+  shader.constants.is_dirty = true;
+}
+
+void GPU_shader_constant_int(GPUShader *sh, const char *name, int value)
+{
+  GPU_shader_constant_int_ex(sh, unwrap(sh)->interface->constant_get(name)->location, value);
+}
+void GPU_shader_constant_uint(GPUShader *sh, const char *name, uint value)
+{
+  GPU_shader_constant_uint_ex(sh, unwrap(sh)->interface->constant_get(name)->location, value);
+}
+void GPU_shader_constant_float(GPUShader *sh, const char *name, float value)
+{
+  GPU_shader_constant_float_ex(sh, unwrap(sh)->interface->constant_get(name)->location, value);
+}
+void GPU_shader_constant_bool(GPUShader *sh, const char *name, bool value)
+{
+  GPU_shader_constant_bool_ex(sh, unwrap(sh)->interface->constant_get(name)->location, value);
+}
+
+/** \} */
+
 /* -------------------------------------------------------------------- */
 /** \name Uniforms / Resource location
  * \{ */
@@ -553,6 +632,13 @@ int GPU_shader_get_uniform(GPUShader *shader, const char *name)
   return uniform ? uniform->location : -1;
 }
 
+int GPU_shader_get_constant(GPUShader *shader, const char *name)
+{
+  const ShaderInterface *interface = unwrap(shader)->interface;
+  const ShaderInput *constant = interface->constant_get(name);
+  return constant ? constant->location : -1;
+}
+
 int GPU_shader_get_builtin_uniform(GPUShader *shader, int builtin)
 {
   const ShaderInterface *interface = unwrap(shader)->interface;
@@ -634,6 +720,16 @@ int GPU_shader_get_program(GPUShader *shader)
   return unwrap(shader)->program_handle_get();
 }
 
+int GPU_shader_get_ssbo_vertex_fetch_num_verts_per_prim(GPUShader *shader)
+{
+  return unwrap(shader)->get_ssbo_vertex_fetch_output_num_verts();
+}
+
+bool GPU_shader_uses_ssbo_vertex_fetch(GPUShader *shader)
+{
+  return unwrap(shader)->get_uses_ssbo_vertex_fetch();
+}
+
 /** \} */
 
 /* -------------------------------------------------------------------- */
@@ -724,6 +820,12 @@ void GPU_shader_uniform_mat3_as_mat4(GPUShader *sh, const char *name, const floa
   GPU_shader_uniform_mat4(sh, name, matrix);
 }
 
+void GPU_shader_uniform_1f_array(GPUShader *sh, const char *name, int len, const float *val)
+{
+  const int loc = GPU_shader_get_uniform(sh, name);
+  GPU_shader_uniform_float_ex(sh, loc, 1, len, val);
+}
+
 void GPU_shader_uniform_2fv_array(GPUShader *sh, const char *name, int len, const float (*val)[2])
 {
   const int loc = GPU_shader_get_uniform(sh, name);
