@@ -6,17 +6,18 @@
  * \ingroup imbuf
  */
 
+#include <algorithm>
 #include <cstddef>
 
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
 
-#include "IMB_allocimbuf.h"
-#include "IMB_colormanagement_intern.h"
-#include "IMB_filetype.h"
-#include "IMB_metadata.h"
+#include "IMB_allocimbuf.hh"
+#include "IMB_colormanagement_intern.hh"
+#include "IMB_filetype.hh"
+#include "IMB_metadata.hh"
 
-#include "imbuf.h"
+#include "imbuf.hh"
 
 #include "MEM_guardedalloc.h"
 
@@ -100,7 +101,7 @@ static void imb_stereo3d_write_anaglyph(const Stereo3DData *s3d, enum eStereo3dA
           to[0] = from[r][0];
           to[1] = from[g][1];
           to[2] = from[b][2];
-          to[3] = MAX2(from[0][3], from[1][3]);
+          to[3] = std::max(from[0][3], from[1][3]);
         }
       }
     }
@@ -137,7 +138,7 @@ static void imb_stereo3d_write_anaglyph(const Stereo3DData *s3d, enum eStereo3dA
           to[0] = from[r][0];
           to[1] = from[g][1];
           to[2] = from[b][2];
-          to[3] = MAX2(from[0][3], from[1][3]);
+          to[3] = std::max(from[0][3], from[1][3]);
         }
       }
     }
