@@ -1,4 +1,4 @@
-/* SPDX-FileCopyrightText: 2023 Blender Authors
+/* SPDX-FileCopyrightText: 2024 Blender Authors
  *
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
@@ -17,15 +17,15 @@
 #include "BLO_blend_defs.hh"
 #include "BLO_readfile.h"
 
-#include "BKE_idtype.h"
-#include "BKE_main.h"
+#include "BKE_idtype.hh"
+#include "BKE_main.hh"
 #include "BKE_preview_image.hh"
 
 #include "DNA_ID.h" /* For preview images... */
 
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
-#include "IMB_thumbs.h"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
+#include "IMB_thumbs.hh"
 
 #include "MEM_guardedalloc.h"
 
