@@ -3,9 +3,9 @@
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "ABC_alembic.h"
+#include "IO_subdiv_disabler.hh"
 #include "abc_archive.h"
 #include "abc_hierarchy_iterator.h"
-#include "abc_subdiv_disabler.h"
 
 #include "MEM_guardedalloc.h"
 
@@ -17,9 +17,9 @@
 #include "DNA_scene_types.h"
 
 #include "BKE_blender_version.h"
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_global.h"
-#include "BKE_main.h"
+#include "BKE_main.hh"
 #include "BKE_scene.h"
 
 #include "BLI_fileops.h"
@@ -70,12 +70,7 @@ static void report_job_duration(const ExportJobData *data)
   std::cout << '\n';
 }
 
-static void export_startjob(void *customdata,
-                            /* Cannot be const, this function implements wm_jobs_start_callback.
-                             * NOLINTNEXTLINE: readability-non-const-parameter. */
-                            bool *stop,
-                            bool *do_update,
-                            float *progress)
+static void export_startjob(void *customdata, wmJobWorkerStatus *worker_status)
 {
   ExportJobData *data = static_cast<ExportJobData *>(customdata);
   data->was_canceled = false;
@@ -85,10 +80,9 @@ static void export_startjob(void *customdata,
   WM_set_locked_interface(data->wm, true);
   G.is_break = false;
 
-  *progress = 0.0f;
-  *do_update = true;
+  worker_status->progress = 0.0f;
+  worker_status->do_update = true;
 
-  build_depsgraph(data->depsgraph, data->params.visible_objects_only);
   SubdivModifierDisabler subdiv_disabler(data->depsgraph);
   if (!data->params.apply_subdiv) {
     subdiv_disabler.disable_modifiers();
@@ -140,7 +134,7 @@ static void export_startjob(void *customdata,
     for (; frame_it != frames_end; frame_it++) {
       double frame = *frame_it;
 
-      if (G.is_break || (stop != nullptr && *stop)) {
+      if (G.is_break || worker_status->stop) {
         break;
       }
 
@@ -154,8 +148,8 @@ static void export_startjob(void *customdata,
       iter.set_export_subset(export_subset);
       iter.iterate_and_write();
 
-      *progress += progress_per_frame;
-      *do_update = true;
+      worker_status->progress += progress_per_frame;
+      worker_status->do_update = true;
     }
   }
   else {
@@ -173,8 +167,8 @@ static void export_startjob(void *customdata,
 
   data->export_ok = !data->was_canceled;
 
-  *progress = 1.0f;
-  *do_update = true;
+  worker_status->progress = 1.0f;
+  worker_status->do_update = true;
 }
 
 static void export_endjob(void *customdata)
@@ -213,6 +207,12 @@ bool ABC_export(Scene *scene,
   job->depsgraph = DEG_graph_new(job->bmain, scene, view_layer, params->evaluation_mode);
   job->params = *params;
 
+  /* Construct the depsgraph for exporting.
+   *
+   * Has to be done from main thread currently, as it may affect Main original data (e.g. when
+   * doing deferred update of the view-layers, see #112534 for details). */
+  blender::io::alembic::build_depsgraph(job->depsgraph, job->params.visible_objects_only);
+
   bool export_ok = false;
   if (as_background_job) {
     wmJob *wm_job = WM_jobs_get(
@@ -230,11 +230,8 @@ bool ABC_export(Scene *scene,
     WM_jobs_start(CTX_wm_manager(C), wm_job);
   }
   else {
-    /* Fake a job context, so that we don't need null pointer checks while exporting. */
-    bool stop = false, do_update = false;
-    float progress = 0.0f;
-
-    blender::io::alembic::export_startjob(job, &stop, &do_update, &progress);
+    wmJobWorkerStatus worker_status = {};
+    blender::io::alembic::export_startjob(job, &worker_status);
     blender::io::alembic::export_endjob(job);
     export_ok = job->export_ok;
 
