@@ -1,5 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2020 Blender Foundation */
+/* SPDX-FileCopyrightText: 2020 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "abc_hierarchy_iterator.h"
 #include "abc_writer_abstract.h"
@@ -18,7 +19,7 @@
 
 #include "BLI_assert.h"
 
-#include "DEG_depsgraph_query.h"
+#include "DEG_depsgraph_query.hh"
 
 #include "DNA_ID.h"
 #include "DNA_layer_types.h"
