@@ -2,10 +2,10 @@
  *
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
-#include "usd_reader_skeleton.h"
-#include "usd_skel_convert.h"
+#include "usd_reader_skeleton.hh"
+#include "usd_skel_convert.hh"
 
-#include "BKE_armature.h"
+#include "BKE_armature.hh"
 #include "BKE_idprop.h"
 #include "BKE_object.hh"
 
@@ -16,8 +16,6 @@
 
 #include "WM_api.hh"
 
-#include <iostream>
-
 namespace blender::io::usd {
 
 bool USDSkeletonReader::valid() const
@@ -39,7 +37,7 @@ void USDSkeletonReader::read_object_data(Main *bmain, const double motionSampleT
     return;
   }
 
-  import_skeleton(bmain, object_, skel_);
+  import_skeleton(bmain, object_, skel_, reports());
 
   USDXformReader::read_object_data(bmain, motionSampleTime);
 }
