@@ -20,9 +20,9 @@
 #include "DNA_material_types.h"
 #include "DNA_node_types.h"
 
-#include "BKE_context.h"
-#include "BKE_lib_id.h"
-#include "BKE_main.h"
+#include "BKE_context.hh"
+#include "BKE_lib_id.hh"
+#include "BKE_main.hh"
 #include "BKE_mesh.hh"
 #include "BKE_node.hh"
 #include "BLI_fileops.h"
@@ -36,8 +36,8 @@
 
 #include "WM_api.hh"
 
-#include "usd.h"
-#include "usd_writer_material.h"
+#include "usd.hh"
+#include "usd_writer_material.hh"
 
 namespace blender::io::usd {
 
@@ -191,10 +191,10 @@ class UsdExportTest : public BlendfileLoadingBaseTest {
     mesh_prim.GetPointsAttr().Get(&positions, 0.0);
     mesh_prim.GetNormalsAttr().Get(&normals, 0.0);
 
-    EXPECT_EQ(mesh->totvert, positions.size());
+    EXPECT_EQ(mesh->verts_num, positions.size());
     EXPECT_EQ(mesh->faces_num, face_counts.size());
-    EXPECT_EQ(mesh->totloop, face_indices.size());
-    EXPECT_EQ(mesh->totloop, normals.size());
+    EXPECT_EQ(mesh->corners_num, face_indices.size());
+    EXPECT_EQ(mesh->corners_num, normals.size());
   }
 };
 
@@ -214,7 +214,7 @@ TEST_F(UsdExportTest, usd_export_rain_mesh)
   params.export_uvmaps = false;
   params.visible_objects_only = true;
 
-  bool result = USD_export(context, output_filename.c_str(), &params, false);
+  bool result = USD_export(context, output_filename.c_str(), &params, false, nullptr);
   ASSERT_TRUE(result) << "Writing to " << output_filename << " failed!";
 
   pxr::UsdStageRefPtr stage = pxr::UsdStage::Open(output_filename);
@@ -280,7 +280,7 @@ TEST_F(UsdExportTest, usd_export_material)
   params.generate_preview_surface = true;
   params.relative_paths = false;
 
-  const bool result = USD_export(context, output_filename.c_str(), &params, false);
+  const bool result = USD_export(context, output_filename.c_str(), &params, false, nullptr);
   ASSERT_TRUE(result) << "Unable to export stage to " << output_filename;
 
   pxr::UsdStageRefPtr stage = pxr::UsdStage::Open(output_filename);
