@@ -9,7 +9,7 @@
 #include "BKE_mesh.hh"
 #include "BKE_object.hh"
 
-#include "BLI_delaunay_2d.h"
+#include "BLI_delaunay_2d.hh"
 #include "BLI_math_geom.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_rotation.h"
@@ -24,49 +24,38 @@
 
 namespace blender::io::obj {
 
-Vector<Vector<int>> fixup_invalid_polygon(Span<float3> vertex_coords,
-                                          Span<int> face_vertex_indices)
+Vector<Vector<int>> fixup_invalid_face(Span<float3> vert_positions, Span<int> face_verts)
 {
   using namespace blender::meshintersect;
-  if (face_vertex_indices.size() < 3) {
+  if (face_verts.size() < 3) {
     return {};
   }
 
-  /* Calculate face normal, to project verts to 2D. */
-  float normal[3] = {0, 0, 0};
-  float3 co_prev = vertex_coords[face_vertex_indices.last()];
-  for (int idx : face_vertex_indices) {
-    BLI_assert(idx >= 0 && idx < vertex_coords.size());
-    float3 co_curr = vertex_coords[idx];
-    add_newell_cross_v3_v3v3(normal, co_prev, co_curr);
-    co_prev = co_curr;
-  }
-  if (UNLIKELY(normalize_v3(normal) == 0.0f)) {
-    normal[2] = 1.0f;
-  }
+  const float3 normal = bke::mesh::face_normal_calc(vert_positions, face_verts);
   float axis_mat[3][3];
   axis_dominant_v3_to_m3(axis_mat, normal);
 
-  /* Prepare data for CDT. */
-  CDT_input<double> input;
-  input.vert.reinitialize(face_vertex_indices.size());
-  input.face.reinitialize(1);
-  input.face[0].resize(face_vertex_indices.size());
-  for (int64_t i = 0; i < face_vertex_indices.size(); ++i) {
-    input.face[0][i] = i;
-  }
-  input.epsilon = 1.0e-6f;
-  input.need_ids = true;
   /* Project vertices to 2D. */
-  for (size_t i = 0; i < face_vertex_indices.size(); ++i) {
-    int idx = face_vertex_indices[i];
-    BLI_assert(idx >= 0 && idx < vertex_coords.size());
-    float3 coord = vertex_coords[idx];
+  Array<double2> input_verts(face_verts.size());
+  for (const int i : face_verts.index_range()) {
+    int idx = face_verts[i];
+    BLI_assert(idx >= 0 && idx < vert_positions.size());
     float2 coord2d;
-    mul_v2_m3v3(coord2d, axis_mat, coord);
-    input.vert[i] = double2(coord2d.x, coord2d.y);
+    mul_v2_m3v3(coord2d, axis_mat, vert_positions[idx]);
+    input_verts[i] = double2(coord2d.x, coord2d.y);
   }
 
+  Array<Vector<int>> input_faces(1);
+  input_faces.first().resize(input_verts.size());
+
+  std::iota(input_faces.first().begin(), input_faces.first().end(), 0);
+
+  /* Prepare data for CDT. */
+  CDT_input<double> input;
+  input.vert = std::move(input_verts);
+  input.face = std::move(input_faces);
+  input.epsilon = 1.0e-6f;
+  input.need_ids = true;
   CDT_result<double> res = delaunay_2d_calc(input, CDT_CONSTRAINTS_VALID_BMESH_WITH_HOLES);
 
   /* Emit new face information from CDT result. */
@@ -79,16 +68,14 @@ Vector<Vector<int>> fixup_invalid_polygon(Span<float3> vertex_coords,
       int idx = f[i];
       BLI_assert(idx >= 0 && idx < res.vert_orig.size());
       if (res.vert_orig[idx].is_empty()) {
-        /* If we have a whole new vertex in the tessellated result,
-         * we won't quite know what to do with it (how to create normal/UV
-         * for it, for example). Such vertices are often due to
-         * self-intersecting polygons. Just skip them from the output
-         * face. */
+        /* If we have a whole new vertex in the tessellated result, we won't quite know what to do
+         * with it (how to create normal/UV for it, for example). Such vertices are often due to
+         * self-intersecting faces. Just skip them from the output face. */
       }
       else {
         /* Vertex corresponds to one or more of the input vertices, use it. */
         idx = res.vert_orig[idx][0];
-        BLI_assert(idx >= 0 && idx < face_vertex_indices.size());
+        BLI_assert(idx >= 0 && idx < face_verts.size());
         face_verts.append(idx);
       }
     }
@@ -114,17 +101,11 @@ void transform_object(Object *object, const OBJImportParams &import_params)
   BKE_object_apply_mat4(object, obmat, true, false);
 
   if (import_params.clamp_size != 0.0f) {
-    float3 max_coord(-INT_MAX);
-    float3 min_coord(INT_MAX);
-    BoundBox *bb = BKE_mesh_boundbox_get(object);
-    for (const float(&vertex)[3] : bb->vec) {
-      for (int axis = 0; axis < 3; axis++) {
-        max_coord[axis] = max_ff(max_coord[axis], vertex[axis]);
-        min_coord[axis] = min_ff(min_coord[axis], vertex[axis]);
-      }
-    }
-    const float max_diff = max_fff(
-        max_coord[0] - min_coord[0], max_coord[1] - min_coord[1], max_coord[2] - min_coord[2]);
+    BLI_assert(object->type == OB_MESH);
+    const Mesh *mesh = static_cast<const Mesh *>(object->data);
+    const Bounds<float3> bounds = *mesh->bounds_min_max();
+    const float max_diff = math::reduce_max(bounds.max - bounds.min);
+
     float scale = 1.0f;
     while (import_params.clamp_size < max_diff * scale) {
       scale = scale / 10;
@@ -133,4 +114,16 @@ void transform_object(Object *object, const OBJImportParams &import_params)
   }
 }
 
+std::string get_geometry_name(const std::string &full_name, char separator)
+{
+  if (separator == 0) {
+    return full_name;
+  }
+  size_t pos = full_name.find_last_of(separator);
+  if (pos == std::string::npos) {
+    return full_name;
+  }
+  return full_name.substr(pos + 1);
+}
+
 }  // namespace blender::io::obj
