@@ -6,6 +6,8 @@
  * \ingroup obj
  */
 
+#include "BKE_report.h"
+
 #include "BLI_map.hh"
 #include "BLI_math_color.h"
 #include "BLI_math_vector.h"
@@ -229,7 +231,7 @@ static void geom_add_polygon(Geometry *geom,
                              const int group_index,
                              const bool shaded_smooth)
 {
-  PolyElem curr_face;
+  FaceElem curr_face;
   curr_face.shaded_smooth = shaded_smooth;
   curr_face.material_index = material_index;
   if (group_index >= 0) {
@@ -243,10 +245,16 @@ static void geom_add_polygon(Geometry *geom,
   bool face_valid = true;
   p = drop_whitespace(p, end);
   while (p < end && face_valid) {
-    PolyCorner corner;
+    FaceCorner corner;
     bool got_uv = false, got_normal = false;
     /* Parse vertex index. */
     p = parse_int(p, end, INT32_MAX, corner.vert_index, false);
+
+    /* Skip parsing when we reach start of the comment. */
+    if (*p == '#') {
+      break;
+    }
+
     face_valid &= corner.vert_index != INT32_MAX;
     if (p < end && *p == '/') {
       /* Parse UV index. */
@@ -313,7 +321,7 @@ static void geom_add_polygon(Geometry *geom,
 
   if (face_valid) {
     geom->face_elements_.append(curr_face);
-    geom->total_loops_ += curr_face.corner_count_;
+    geom->total_corner_ += curr_face.corner_count_;
   }
   else {
     /* Remove just-added corners for the invalid face. */
@@ -441,6 +449,10 @@ OBJParser::OBJParser(const OBJImportParams &import_params, size_t read_buffer_si
   obj_file_ = BLI_fopen(import_params_.filepath, "rb");
   if (!obj_file_) {
     fprintf(stderr, "Cannot read from OBJ file:'%s'.\n", import_params_.filepath);
+    BKE_reportf(import_params_.reports,
+                RPT_ERROR,
+                "OBJ Import: Cannot open file '%s'",
+                import_params_.filepath);
     return;
   }
 }
