@@ -1,5 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2005 Blender Foundation */
+/* SPDX-FileCopyrightText: 2005 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 /** \file
  * \ingroup DNA
@@ -13,10 +14,6 @@
 #include "DNA_defs.h"
 #include "DNA_texture_types.h" /* for MTex */
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 struct CurveMapping;
 struct Image;
 struct MTex;
@@ -93,7 +90,7 @@ typedef struct BrushGpencilSettings {
   int flag;
 
   /** gradient control along y for color */
-  float hardeness;
+  float hardness;
   /** factor xy of shape for dots gradients */
   float aspect_ratio[2];
   /** Simplify adaptive factor */
@@ -166,6 +163,8 @@ typedef struct BrushCurvesSculptSettings {
   struct CurveMapping *curve_parameter_falloff;
 } BrushCurvesSculptSettings;
 
+/** Max number of propagation steps for automasking settings.*/
+#define AUTOMASKING_BOUNDARY_EDGES_MAX_PROPAGATION_STEPS 20
 typedef struct Brush {
   DNA_DEFINE_CXX_METHODS(Brush)
 
@@ -205,6 +204,9 @@ typedef struct Brush {
   int flag2;
   int sampling_flag;
 
+  /** Number of samples used to smooth the stroke. */
+  int input_samples;
+
   /** Pressure influence for mask. */
   int mask_pressure;
   /** Jitter the position of the brush. */
@@ -261,7 +263,7 @@ typedef struct Brush {
   /** Source for fill tool color gradient application. */
   char gradient_fill_mode;
 
-  char _pad0[5];
+  char _pad0;
 
   /** Projection shape (sphere, circle). */
   char falloff_shape;
@@ -320,6 +322,11 @@ typedef struct Brush {
   int automasking_flags;
   int automasking_boundary_edges_propagation_steps;
 
+  float automasking_start_normal_limit;
+  float automasking_start_normal_falloff;
+  float automasking_view_normal_limit;
+  float automasking_view_normal_falloff;
+
   int elastic_deform_type;
   float elastic_deform_volume_preservation;
 
@@ -440,7 +447,3 @@ typedef struct PaintCurve {
   /** Index where next point will be added. */
   int add_index;
 } PaintCurve;
-
-#ifdef __cplusplus
-}
-#endif
