@@ -8,6 +8,7 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "BLI_array_utils.hh"
 #include "BLI_bitmap.h"
 #include "BLI_linklist.h"
 #include "BLI_math_vector.h"
@@ -22,9 +23,9 @@
 #include "DNA_screen_types.h"
 
 #include "BKE_attribute.hh"
-#include "BKE_context.h"
-#include "BKE_deform.h"
-#include "BKE_lib_id.h"
+#include "BKE_context.hh"
+#include "BKE_deform.hh"
+#include "BKE_lib_id.hh"
 #include "BKE_mesh.hh"
 #include "BKE_mesh_mapping.hh"
 #include "BKE_screen.hh"
@@ -39,7 +40,7 @@
 #include "MOD_ui_common.hh"
 #include "MOD_util.hh"
 
-#include "bmesh.h"
+#include "bmesh.hh"
 
 #define CLNORS_VALID_VEC_LEN (1e-6f)
 
@@ -82,11 +83,10 @@ struct WeightedNormalData {
   blender::Span<int> loop_to_face;
   blender::MutableSpan<blender::short2> clnors;
   bool has_clnors; /* True if clnors already existed, false if we had to create them. */
-  float split_angle;
 
   blender::OffsetIndices<int> faces;
   blender::Span<blender::float3> face_normals;
-  const bool *sharp_faces;
+  blender::VArraySpan<bool> sharp_faces;
   const int *face_strength;
 
   const MDeformVert *dvert;
@@ -149,7 +149,8 @@ static void aggregate_item_normal(WeightedNormalModifierData *wnmd,
                              BKE_defvert_find_index(&dvert[mv_index], defgrp_index) != nullptr;
 
   if (has_vgroup &&
-      ((vert_of_group && use_invert_vgroup) || (!vert_of_group && !use_invert_vgroup))) {
+      ((vert_of_group && use_invert_vgroup) || (!vert_of_group && !use_invert_vgroup)))
+  {
     return;
   }
 
@@ -206,7 +207,6 @@ static void apply_weights_vertex_normal(WeightedNormalModifierData *wnmd,
   ModePair *mode_pair = wn_data->mode_pair;
 
   const bool has_clnors = wn_data->has_clnors;
-  const float split_angle = wn_data->split_angle;
   bke::mesh::CornerNormalSpaceArray lnors_spacearr;
 
   const bool keep_sharp = (wnmd->flag & MOD_WEIGHTEDNORMAL_KEEP_SHARP) != 0;
@@ -214,28 +214,26 @@ static void apply_weights_vertex_normal(WeightedNormalModifierData *wnmd,
                                   face_strength != nullptr;
   const bool has_vgroup = dvert != nullptr;
 
-  blender::Array<blender::float3> loop_normals;
+  blender::Array<blender::float3> corner_normals;
 
   Array<WeightedNormalDataAggregateItem> items_data;
   if (keep_sharp) {
     /* This will give us loop normal spaces,
-     * we do not actually care about computed loop_normals for now... */
-    loop_normals.reinitialize(corner_verts.size());
-    bke::mesh::normals_calc_loop(positions,
-                                 edges,
-                                 faces,
-                                 corner_verts,
-                                 corner_edges,
-                                 loop_to_face,
-                                 wn_data->vert_normals,
-                                 wn_data->face_normals,
-                                 wn_data->sharp_edges.data(),
-                                 wn_data->sharp_faces,
-                                 has_clnors ? clnors.data() : nullptr,
-                                 true,
-                                 split_angle,
-                                 &lnors_spacearr,
-                                 loop_normals);
+     * we do not actually care about computed corner_normals for now... */
+    corner_normals.reinitialize(corner_verts.size());
+    bke::mesh::normals_calc_corners(positions,
+                                    edges,
+                                    faces,
+                                    corner_verts,
+                                    corner_edges,
+                                    loop_to_face,
+                                    wn_data->vert_normals,
+                                    wn_data->face_normals,
+                                    wn_data->sharp_edges,
+                                    wn_data->sharp_faces,
+                                    has_clnors ? clnors.data() : nullptr,
+                                    &lnors_spacearr,
+                                    corner_normals);
 
     WeightedNormalDataAggregateItem start_item{};
     start_item.curr_strength = FACE_STRENGTH_WEAK;
@@ -246,8 +244,7 @@ static void apply_weights_vertex_normal(WeightedNormalModifierData *wnmd,
     start_item.curr_strength = FACE_STRENGTH_WEAK;
     items_data = Array<WeightedNormalDataAggregateItem>(verts_num, start_item);
     lnors_spacearr.corner_space_indices.reinitialize(corner_verts.size());
-    std::iota(
-        lnors_spacearr.corner_space_indices.begin(), lnors_spacearr.corner_space_indices.end(), 0);
+    array_utils::fill_index_range<int>(lnors_spacearr.corner_space_indices);
   }
   wn_data->items_data = items_data;
 
@@ -257,9 +254,9 @@ static void apply_weights_vertex_normal(WeightedNormalModifierData *wnmd,
         const int face_index = mode_pair[i].index;
         const float mp_val = mode_pair[i].val;
 
-        for (const int ml_index : faces[face_index]) {
-          const int mv_index = corner_verts[ml_index];
-          const int space_index = lnors_spacearr.corner_space_indices[ml_index];
+        for (const int corner : faces[face_index]) {
+          const int mv_index = corner_verts[corner];
+          const int space_index = lnors_spacearr.corner_space_indices[corner];
 
           WeightedNormalDataAggregateItem *item_data = keep_sharp ? &items_data[space_index] :
                                                                     &items_data[mv_index];
@@ -272,12 +269,12 @@ static void apply_weights_vertex_normal(WeightedNormalModifierData *wnmd,
     case MOD_WEIGHTEDNORMAL_MODE_ANGLE:
     case MOD_WEIGHTEDNORMAL_MODE_FACE_ANGLE:
       for (int i = 0; i < corner_verts.size(); i++) {
-        const int ml_index = mode_pair[i].index;
+        const int corner = mode_pair[i].index;
         const float ml_val = mode_pair[i].val;
-        const int space_index = lnors_spacearr.corner_space_indices[ml_index];
+        const int space_index = lnors_spacearr.corner_space_indices[corner];
 
-        const int face_index = loop_to_face[ml_index];
-        const int mv_index = corner_verts[ml_index];
+        const int face_index = loop_to_face[corner];
+        const int mv_index = corner_verts[corner];
         WeightedNormalDataAggregateItem *item_data = keep_sharp ? &items_data[space_index] :
                                                                   &items_data[mv_index];
 
@@ -298,28 +295,28 @@ static void apply_weights_vertex_normal(WeightedNormalModifierData *wnmd,
 
   if (keep_sharp) {
     /* Set loop normals for normal computed for each lnor space (smooth fan).
-     * Note that loop_normals is already populated with clnors
+     * Note that corner_normals is already populated with clnors
      * (before this modifier is applied, at start of this function),
      * so no need to recompute them here. */
-    for (int ml_index = 0; ml_index < corner_verts.size(); ml_index++) {
-      const int space_index = lnors_spacearr.corner_space_indices[ml_index];
+    for (int corner = 0; corner < corner_verts.size(); corner++) {
+      const int space_index = lnors_spacearr.corner_space_indices[corner];
       WeightedNormalDataAggregateItem *item_data = &items_data[space_index];
       if (!is_zero_v3(item_data->normal)) {
-        copy_v3_v3(loop_normals[ml_index], item_data->normal);
+        copy_v3_v3(corner_normals[corner], item_data->normal);
       }
     }
 
-    blender::bke::mesh::normals_loop_custom_set(positions,
-                                                edges,
-                                                faces,
-                                                corner_verts,
-                                                corner_edges,
-                                                wn_data->vert_normals,
-                                                face_normals,
-                                                wn_data->sharp_faces,
-                                                wn_data->sharp_edges,
-                                                loop_normals,
-                                                clnors);
+    blender::bke::mesh::normals_corner_custom_set(positions,
+                                                  edges,
+                                                  faces,
+                                                  corner_verts,
+                                                  corner_edges,
+                                                  wn_data->vert_normals,
+                                                  face_normals,
+                                                  wn_data->sharp_faces,
+                                                  wn_data->sharp_edges,
+                                                  corner_normals,
+                                                  clnors);
   }
   else {
     /* TODO: Ideally, we could add an option to `BKE_mesh_normals_loop_custom_[from_verts_]set()`
@@ -333,58 +330,56 @@ static void apply_weights_vertex_normal(WeightedNormalModifierData *wnmd,
        * and it makes code simpler & cleaner. */
       blender::Array<blender::float3> vert_normals(verts_num, float3(0.0f));
 
-      for (int ml_index = 0; ml_index < corner_verts.size(); ml_index++) {
-        const int mv_index = corner_verts[ml_index];
+      for (int corner = 0; corner < corner_verts.size(); corner++) {
+        const int mv_index = corner_verts[corner];
         copy_v3_v3(vert_normals[mv_index], items_data[mv_index].normal);
       }
 
-      blender::bke::mesh::normals_loop_custom_set_from_verts(positions,
-                                                             edges,
-                                                             faces,
-                                                             corner_verts,
-                                                             corner_edges,
-                                                             wn_data->vert_normals,
-                                                             face_normals,
-                                                             wn_data->sharp_faces,
-                                                             wn_data->sharp_edges,
-                                                             vert_normals,
-                                                             clnors);
+      blender::bke::mesh::normals_corner_custom_set_from_verts(positions,
+                                                               edges,
+                                                               faces,
+                                                               corner_verts,
+                                                               corner_edges,
+                                                               wn_data->vert_normals,
+                                                               face_normals,
+                                                               wn_data->sharp_faces,
+                                                               wn_data->sharp_edges,
+                                                               vert_normals,
+                                                               clnors);
     }
     else {
-      loop_normals.reinitialize(corner_verts.size());
-      blender::bke::mesh::normals_calc_loop(positions,
-                                            edges,
-                                            faces,
-                                            corner_verts,
-                                            corner_edges,
-                                            loop_to_face,
-                                            wn_data->vert_normals,
-                                            face_normals,
-                                            wn_data->sharp_edges.data(),
-                                            wn_data->sharp_faces,
-                                            has_clnors ? clnors.data() : nullptr,
-                                            true,
-                                            split_angle,
-                                            nullptr,
-                                            loop_normals);
+      corner_normals.reinitialize(corner_verts.size());
+      blender::bke::mesh::normals_calc_corners(positions,
+                                               edges,
+                                               faces,
+                                               corner_verts,
+                                               corner_edges,
+                                               loop_to_face,
+                                               wn_data->vert_normals,
+                                               face_normals,
+                                               wn_data->sharp_edges,
+                                               wn_data->sharp_faces,
+                                               has_clnors ? clnors.data() : nullptr,
+                                               nullptr,
+                                               corner_normals);
 
-      for (int ml_index = 0; ml_index < corner_verts.size(); ml_index++) {
-        const int item_index = corner_verts[ml_index];
+      for (int corner = 0; corner < corner_verts.size(); corner++) {
+        const int item_index = corner_verts[corner];
         if (!is_zero_v3(items_data[item_index].normal)) {
-          copy_v3_v3(loop_normals[ml_index], items_data[item_index].normal);
+          copy_v3_v3(corner_normals[corner], items_data[item_index].normal);
         }
       }
-      blender::bke::mesh::normals_loop_custom_set(positions,
-                                                  edges,
-                                                  faces,
-                                                  corner_verts,
-                                                  corner_edges,
-                                                  wn_data->vert_normals,
-                                                  face_normals,
-                                                  wn_data->sharp_faces,
-                                                  wn_data->sharp_edges,
-                                                  loop_normals,
-                                                  clnors);
+      blender::bke::mesh::normals_corner_custom_set(positions,
+                                                    edges,
+                                                    faces,
+                                                    corner_verts,
+                                                    corner_edges,
+                                                    wn_data->vert_normals,
+                                                    face_normals,
+                                                    wn_data->sharp_faces,
+                                                    wn_data->sharp_edges,
+                                                    corner_normals,
+                                                    clnors);
     }
   }
 }
@@ -428,11 +423,11 @@ static void wn_corner_angle(WeightedNormalModifierData *wnmd, WeightedNormalData
 
     ModePair *c_angl = &corner_angle[face.start()];
     float *angl = index_angle;
-    for (int ml_index = face.start(); ml_index < face.start() + face.size();
-         ml_index++, c_angl++, angl++)
+    for (int corner = face.start(); corner < face.start() + face.size();
+         corner++, c_angl++, angl++)
     {
       c_angl->val = float(M_PI) - *angl;
-      c_angl->index = ml_index;
+      c_angl->index = corner;
     }
     MEM_freeN(index_angle);
   }
@@ -462,12 +457,11 @@ static void wn_face_with_angle(WeightedNormalModifierData *wnmd, WeightedNormalD
 
     ModePair *cmbnd = &combined[face.start()];
     float *angl = index_angle;
-    for (int ml_index = face.start(); ml_index < face.start() + face.size();
-         ml_index++, cmbnd++, angl++)
+    for (int corner = face.start(); corner < face.start() + face.size(); corner++, cmbnd++, angl++)
     {
       /* In this case val is product of corner angle and face area. */
       cmbnd->val = (float(M_PI) - *angl) * face_area;
-      cmbnd->index = ml_index;
+      cmbnd->index = corner;
     }
     MEM_freeN(index_angle);
   }
@@ -482,28 +476,11 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
 {
   using namespace blender;
   WeightedNormalModifierData *wnmd = (WeightedNormalModifierData *)md;
-  Object *ob = ctx->object;
-
-  /* XXX TODO(Rohan Rathi):
-   * Once we fully switch to Mesh evaluation of modifiers,
-   * we can expect to get that flag from the COW copy.
-   * But for now, it is lost in the DM intermediate step,
-   * so we need to directly check orig object's data. */
-#if 0
-  if (!(mesh->flag & ME_AUTOSMOOTH))
-#else
-  if (!(((Mesh *)ob->data)->flag & ME_AUTOSMOOTH))
-#endif
-  {
-    BKE_modifier_set_error(
-        ctx->object, (ModifierData *)wnmd, "Enable 'Auto Smooth' in Object Data Properties");
-    return mesh;
-  }
 
   Mesh *result;
   result = (Mesh *)BKE_id_copy_ex(nullptr, &mesh->id, nullptr, LIB_ID_COPY_LOCALIZE);
 
-  const int verts_num = result->totvert;
+  const int verts_num = result->verts_num;
   const blender::Span<blender::float3> positions = mesh->vert_positions();
   const blender::Span<int2> edges = mesh->edges();
   const OffsetIndices faces = result->faces();
@@ -527,16 +504,15 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
     weight = (weight - 1) * 25;
   }
 
-  const float split_angle = mesh->smoothresh;
-  blender::short2 *clnors = static_cast<blender::short2 *>(
-      CustomData_get_layer_for_write(&result->loop_data, CD_CUSTOMLOOPNORMAL, mesh->totloop));
+  blender::short2 *clnors = static_cast<blender::short2 *>(CustomData_get_layer_for_write(
+      &result->corner_data, CD_CUSTOMLOOPNORMAL, mesh->corners_num));
 
   /* Keep info whether we had clnors,
    * it helps when generating clnor spaces and default normals. */
   const bool has_clnors = clnors != nullptr;
   if (!clnors) {
     clnors = static_cast<blender::short2 *>(CustomData_add_layer(
-        &result->loop_data, CD_CUSTOMLOOPNORMAL, CD_SET_DEFAULT, corner_verts.size()));
+        &result->corner_data, CD_CUSTOMLOOPNORMAL, CD_SET_DEFAULT, corner_verts.size()));
   }
 
   const MDeformVert *dvert;
@@ -547,7 +523,7 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
 
   bke::MutableAttributeAccessor attributes = result->attributes_for_write();
   bke::SpanAttributeWriter<bool> sharp_edges = attributes.lookup_or_add_for_write_span<bool>(
-      "sharp_edge", ATTR_DOMAIN_EDGE);
+      "sharp_edge", bke::AttrDomain::Edge);
 
   WeightedNormalData wn_data{};
   wn_data.verts_num = verts_num;
@@ -560,14 +536,12 @@ static Mesh *modify_mesh(ModifierData *md, const ModifierEvalContext *ctx, Mesh
   wn_data.corner_verts = corner_verts;
   wn_data.corner_edges = corner_edges;
   wn_data.loop_to_face = loop_to_face_map;
-  wn_data.clnors = {clnors, mesh->totloop};
+  wn_data.clnors = {clnors, mesh->corners_num};
   wn_data.has_clnors = has_clnors;
-  wn_data.split_angle = split_angle;
 
   wn_data.faces = faces;
   wn_data.face_normals = mesh->face_normals();
-  wn_data.sharp_faces = static_cast<const bool *>(
-      CustomData_get_layer_named(&mesh->face_data, CD_PROP_BOOL, "sharp_face"));
+  wn_data.sharp_faces = *attributes.lookup<bool>("sharp_face", bke::AttrDomain::Face);
   wn_data.face_strength = static_cast<const int *>(CustomData_get_layer_named(
       &result->face_data, CD_PROP_INT32, MOD_WEIGHTEDNORMALS_FACEWEIGHT_CDLAYER_ID));
 
@@ -663,7 +637,7 @@ ModifierTypeInfo modifierType_WeightedNormal = {
     /*struct_name*/ "WeightedNormalModifierData",
     /*struct_size*/ sizeof(WeightedNormalModifierData),
     /*srna*/ &RNA_WeightedNormalModifier,
-    /*type*/ eModifierTypeType_Constructive,
+    /*type*/ ModifierTypeType::Constructive,
     /*flags*/ eModifierTypeFlag_AcceptsMesh | eModifierTypeFlag_SupportsMapping |
         eModifierTypeFlag_SupportsEditmode | eModifierTypeFlag_EnableInEditmode,
     /*icon*/ ICON_MOD_NORMALEDIT,
@@ -690,4 +664,5 @@ ModifierTypeInfo modifierType_WeightedNormal = {
     /*panel_register*/ panel_register,
     /*blend_write*/ nullptr,
     /*blend_read*/ nullptr,
+    /*foreach_cache*/ nullptr,
 };
