@@ -2,6 +2,7 @@
  *
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
+#include "BLI_array_utils.hh"
 #include "BLI_task.hh"
 
 #include "BKE_curves.hh"
@@ -178,13 +179,13 @@ class CurveParameterFieldInput final : public bke::CurvesFieldInput {
   }
 
   GVArray get_varray_for_context(const bke::CurvesGeometry &curves,
-                                 const eAttrDomain domain,
+                                 const AttrDomain domain,
                                  const IndexMask & /*mask*/) const final
   {
     switch (domain) {
-      case ATTR_DOMAIN_POINT:
+      case AttrDomain::Point:
         return VArray<float>::ForContainer(calculate_point_parameters(curves));
-      case ATTR_DOMAIN_CURVE:
+      case AttrDomain::Curve:
         return VArray<float>::ForContainer(calculate_curve_parameters(curves));
       default:
         BLI_assert_unreachable();
@@ -212,14 +213,14 @@ class CurveLengthParameterFieldInput final : public bke::CurvesFieldInput {
   }
 
   GVArray get_varray_for_context(const bke::CurvesGeometry &curves,
-                                 const eAttrDomain domain,
+                                 const AttrDomain domain,
                                  const IndexMask & /*mask*/) const final
   {
     switch (domain) {
-      case ATTR_DOMAIN_POINT:
+      case AttrDomain::Point:
         return VArray<float>::ForContainer(calculate_point_lengths(
             curves, [](MutableSpan<float> /*lengths*/, const float /*total*/) {}));
-      case ATTR_DOMAIN_CURVE:
+      case AttrDomain::Curve:
         return VArray<float>::ForContainer(accumulated_lengths_curve_domain(curves));
       default:
         BLI_assert_unreachable();
@@ -246,10 +247,10 @@ class IndexOnSplineFieldInput final : public bke::CurvesFieldInput {
   }
 
   GVArray get_varray_for_context(const bke::CurvesGeometry &curves,
-                                 const eAttrDomain domain,
+                                 const AttrDomain domain,
                                  const IndexMask & /*mask*/) const final
   {
-    if (domain != ATTR_DOMAIN_POINT) {
+    if (domain != AttrDomain::Point) {
       return {};
     }
     Array<int> result(curves.points_num());
@@ -257,9 +258,7 @@ class IndexOnSplineFieldInput final : public bke::CurvesFieldInput {
     threading::parallel_for(curves.curves_range(), 1024, [&](IndexRange range) {
       for (const int i_curve : range) {
         MutableSpan<int> indices = result.as_mutable_span().slice(points_by_curve[i_curve]);
-        for (const int i : indices.index_range()) {
-          indices[i] = i;
-        }
+        array_utils::fill_index_range(indices);
       }
     });
     return VArray<int>::ForContainer(std::move(result));
@@ -275,9 +274,9 @@ class IndexOnSplineFieldInput final : public bke::CurvesFieldInput {
     return dynamic_cast<const IndexOnSplineFieldInput *>(&other) != nullptr;
   }
 
-  std::optional<eAttrDomain> preferred_domain(const CurvesGeometry & /*curves*/) const
+  std::optional<AttrDomain> preferred_domain(const CurvesGeometry & /*curves*/) const
   {
-    return ATTR_DOMAIN_POINT;
+    return AttrDomain::Point;
   }
 };
 
