@@ -1,4 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later */
+/* SPDX-FileCopyrightText: 2023 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 #ifdef WITH_OPENVDB
 #  include <openvdb/openvdb.h>
@@ -9,13 +11,18 @@
 #include "DNA_node_types.h"
 #include "DNA_pointcloud_types.h"
 
-#include "BKE_pointcloud.h"
-#include "BKE_volume.h"
+#include "BKE_pointcloud.hh"
+#include "BKE_volume.hh"
+#include "BKE_volume_grid.hh"
 
-#include "UI_interface.h"
-#include "UI_resources.h"
+#include "NOD_rna_define.hh"
 
-#include "DEG_depsgraph_query.h"
+#include "UI_interface.hh"
+#include "UI_resources.hh"
+
+#include "DEG_depsgraph_query.hh"
+
+#include "GEO_randomize.hh"
 
 #include "node_geometry_util.hh"
 
@@ -26,7 +33,7 @@ NODE_STORAGE_FUNCS(NodeGeometryDistributePointsInVolume)
 static void node_declare(NodeDeclarationBuilder &b)
 {
   b.add_input<decl::Geometry>("Volume")
-      .supported_type(GEO_COMPONENT_TYPE_VOLUME)
+      .supported_type(GeometryComponent::Type::Volume)
       .translation_context(BLT_I18NCONTEXT_ID_ID);
   b.add_input<decl::Float>("Density")
       .default_value(1.0f)
@@ -51,7 +58,7 @@ static void node_declare(NodeDeclarationBuilder &b)
 
 static void node_layout(uiLayout *layout, bContext * /*C*/, PointerRNA *ptr)
 {
-  uiItemR(layout, ptr, "mode", 0, "", ICON_NONE);
+  uiItemR(layout, ptr, "mode", UI_ITEM_NONE, "", ICON_NONE);
 }
 
 static void node_init(bNodeTree * /*tree*/, bNode *node)
@@ -203,39 +210,35 @@ static void node_geo_exec(GeoNodeExecParams params)
 
   geometry_set.modify_geometry_sets([&](GeometrySet &geometry_set) {
     if (!geometry_set.has_volume()) {
-      geometry_set.keep_only({GEO_COMPONENT_TYPE_POINT_CLOUD, GEO_COMPONENT_TYPE_INSTANCES});
+      geometry_set.keep_only_during_modify({GeometryComponent::Type::PointCloud});
       return;
     }
-    const VolumeComponent *component = geometry_set.get_component_for_read<VolumeComponent>();
-    const Volume *volume = component->get_for_read();
+    const VolumeComponent *component = geometry_set.get_component<VolumeComponent>();
+    const Volume *volume = component->get();
     BKE_volume_load(volume, DEG_get_bmain(params.depsgraph()));
 
     Vector<float3> positions;
 
     for (const int i : IndexRange(BKE_volume_num_grids(volume))) {
-      const VolumeGrid *volume_grid = BKE_volume_grid_get_for_read(volume, i);
+      const bke::VolumeGridData *volume_grid = BKE_volume_grid_get(volume, i);
       if (volume_grid == nullptr) {
         continue;
       }
 
-      openvdb::GridBase::ConstPtr base_grid = BKE_volume_grid_openvdb_for_read(volume,
-                                                                               volume_grid);
-      if (!base_grid) {
+      bke::VolumeTreeAccessToken tree_token;
+      const openvdb::GridBase &base_grid = volume_grid->grid(tree_token);
+
+      if (!base_grid.isType<openvdb::FloatGrid>()) {
         continue;
       }
 
-      if (!base_grid->isType<openvdb::FloatGrid>()) {
-        continue;
-      }
-
-      const openvdb::FloatGrid::ConstPtr grid = openvdb::gridConstPtrCast<openvdb::FloatGrid>(
-          base_grid);
+      const openvdb::FloatGrid &grid = static_cast<const openvdb::FloatGrid &>(base_grid);
 
       if (mode == GEO_NODE_DISTRIBUTE_POINTS_IN_VOLUME_DENSITY_RANDOM) {
-        point_scatter_density_random(*grid, density, seed, positions);
+        point_scatter_density_random(grid, density, seed, positions);
       }
       else if (mode == GEO_NODE_DISTRIBUTE_POINTS_IN_VOLUME_DENSITY_GRID) {
-        point_scatter_density_grid(*grid, spacing, threshold, positions);
+        point_scatter_density_grid(grid, spacing, threshold, positions);
       }
     }
 
@@ -243,29 +246,51 @@ static void node_geo_exec(GeoNodeExecParams params)
     bke::MutableAttributeAccessor point_attributes = pointcloud->attributes_for_write();
     pointcloud->positions_for_write().copy_from(positions);
     bke::SpanAttributeWriter<float> point_radii =
-        point_attributes.lookup_or_add_for_write_only_span<float>("radius", ATTR_DOMAIN_POINT);
+        point_attributes.lookup_or_add_for_write_only_span<float>("radius", AttrDomain::Point);
 
     point_radii.span.fill(0.05f);
     point_radii.finish();
 
+    geometry::debug_randomize_point_order(pointcloud);
+
     geometry_set.replace_pointcloud(pointcloud);
-    geometry_set.keep_only_during_modify({GEO_COMPONENT_TYPE_POINT_CLOUD});
+    geometry_set.keep_only_during_modify({GeometryComponent::Type::PointCloud});
   });
 
   params.set_output("Points", std::move(geometry_set));
 
 #else
-  params.set_default_remaining_outputs();
-  params.error_message_add(NodeWarningType::Error,
-                           TIP_("Disabled, Blender was compiled without OpenVDB"));
+  node_geo_exec_with_missing_openvdb(params);
 #endif
 }
-}  // namespace blender::nodes::node_geo_distribute_points_in_volume_cc
 
-void register_node_type_geo_distribute_points_in_volume()
+static void node_rna(StructRNA *srna)
 {
-  namespace file_ns = blender::nodes::node_geo_distribute_points_in_volume_cc;
+  static const EnumPropertyItem mode_items[] = {
+      {GEO_NODE_DISTRIBUTE_POINTS_IN_VOLUME_DENSITY_RANDOM,
+       "DENSITY_RANDOM",
+       0,
+       "Random",
+       "Distribute points randomly inside of the volume"},
+      {GEO_NODE_DISTRIBUTE_POINTS_IN_VOLUME_DENSITY_GRID,
+       "DENSITY_GRID",
+       0,
+       "Grid",
+       "Distribute the points in a grid pattern inside of the volume"},
+      {0, nullptr, 0, nullptr, nullptr},
+  };
 
+  RNA_def_node_enum(srna,
+                    "mode",
+                    "Distribution Method",
+                    "Method to use for scattering points",
+                    mode_items,
+                    NOD_storage_enum_accessors(mode),
+                    GEO_NODE_DISTRIBUTE_POINTS_IN_VOLUME_DENSITY_RANDOM);
+}
+
+static void node_register()
+{
   static bNodeType ntype;
   geo_node_type_base(&ntype,
                      GEO_NODE_DISTRIBUTE_POINTS_IN_VOLUME,
@@ -275,11 +300,16 @@ void register_node_type_geo_distribute_points_in_volume()
                     "NodeGeometryDistributePointsInVolume",
                     node_free_standard_storage,
                     node_copy_standard_storage);
-  ntype.initfunc = file_ns::node_init;
-  ntype.updatefunc = file_ns::node_update;
+  ntype.initfunc = node_init;
+  ntype.updatefunc = node_update;
   blender::bke::node_type_size(&ntype, 170, 100, 320);
-  ntype.declare = file_ns::node_declare;
-  ntype.geometry_node_execute = file_ns::node_geo_exec;
-  ntype.draw_buttons = file_ns::node_layout;
+  ntype.declare = node_declare;
+  ntype.geometry_node_execute = node_geo_exec;
+  ntype.draw_buttons = node_layout;
   nodeRegisterType(&ntype);
+
+  node_rna(ntype.rna_ext.srna);
 }
+NOD_REGISTER_NODE(node_register)
+
+}  // namespace blender::nodes::node_geo_distribute_points_in_volume_cc
