@@ -1,4 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0-or-later */
+/* SPDX-FileCopyrightText: 2023 Blender Authors
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "node_geometry_util.hh"
 
@@ -17,13 +19,13 @@ class InstanceScaleFieldInput final : public bke::InstancesFieldInput {
  public:
   InstanceScaleFieldInput() : bke::InstancesFieldInput(CPPType::get<float3>(), "Scale") {}
 
-  GVArray get_varray_for_context(const bke::Instances &instances, IndexMask /*mask*/) const final
+  GVArray get_varray_for_context(const bke::Instances &instances,
+                                 const IndexMask & /*mask*/) const final
   {
-    auto scale_fn = [&](const int i) -> float3 {
-      return math::to_scale(instances.transforms()[i]);
-    };
-
-    return VArray<float3>::ForFunc(instances.instances_num(), scale_fn);
+    const Span<float4x4> transforms = instances.transforms();
+    return VArray<float3>::ForFunc(instances.instances_num(), [transforms](const int i) {
+      return math::to_scale<true>(transforms[i]);
+    });
   }
 
   uint64_t hash() const override
@@ -43,15 +45,14 @@ static void node_geo_exec(GeoNodeExecParams params)
   params.set_output("Scale", std::move(scale));
 }
 
-}  // namespace blender::nodes::node_geo_input_instance_scale_cc
-
-void register_node_type_geo_input_instance_scale()
+static void node_register()
 {
-  namespace file_ns = blender::nodes::node_geo_input_instance_scale_cc;
-
   static bNodeType ntype;
   geo_node_type_base(&ntype, GEO_NODE_INPUT_INSTANCE_SCALE, "Instance Scale", NODE_CLASS_INPUT);
-  ntype.geometry_node_execute = file_ns::node_geo_exec;
-  ntype.declare = file_ns::node_declare;
+  ntype.geometry_node_execute = node_geo_exec;
+  ntype.declare = node_declare;
   nodeRegisterType(&ntype);
 }
+NOD_REGISTER_NODE(node_register)
+
+}  // namespace blender::nodes::node_geo_input_instance_scale_cc
