@@ -8,7 +8,7 @@
 
 #include "DNA_mesh_types.h"
 
-#include "BKE_bvhutils.h"
+#include "BKE_bvhutils.hh"
 #include "BKE_geometry_set.hh"
 
 #include "NOD_rna_define.hh"
@@ -24,9 +24,11 @@ NODE_STORAGE_FUNCS(NodeGeometryProximity)
 
 static void node_declare(NodeDeclarationBuilder &b)
 {
-  b.add_input<decl::Geometry>("Target").only_realized_data().supported_type(
-      {GeometryComponent::Type::Mesh, GeometryComponent::Type::PointCloud});
-  b.add_input<decl::Vector>("Source Position").implicit_field(implicit_field_inputs::position);
+  b.add_input<decl::Geometry>("Geometry", "Target")
+      .only_realized_data()
+      .supported_type({GeometryComponent::Type::Mesh, GeometryComponent::Type::PointCloud});
+  b.add_input<decl::Vector>("Sample Position", "Source Position")
+      .implicit_field(implicit_field_inputs::position);
   b.add_output<decl::Vector>("Position").dependent_field().reference_pass_all();
   b.add_output<decl::Float>("Distance").dependent_field().reference_pass_all();
 }
@@ -59,7 +61,7 @@ static bool calculate_mesh_proximity(const VArray<float3> &positions,
       BKE_bvhtree_from_mesh_get(&bvh_data, &mesh, BVHTREE_FROM_EDGES, 2);
       break;
     case GEO_NODE_PROX_TARGET_FACES:
-      BKE_bvhtree_from_mesh_get(&bvh_data, &mesh, BVHTREE_FROM_LOOPTRI, 2);
+      BKE_bvhtree_from_mesh_get(&bvh_data, &mesh, BVHTREE_FROM_CORNER_TRIS, 2);
       break;
   }
 
