@@ -24,24 +24,24 @@
 #include "DNA_anim_types.h"
 #include "DNA_sound_types.h"
 
-#include "IMB_imbuf.h"
+#include "IMB_imbuf.hh"
 
 #include "RNA_prototypes.h"
 
-#include "SEQ_channels.h"
-#include "SEQ_iterator.h"
-#include "SEQ_relations.h"
-#include "SEQ_render.h"
+#include "SEQ_channels.hh"
+#include "SEQ_iterator.hh"
+#include "SEQ_relations.hh"
+#include "SEQ_render.hh"
 #include "SEQ_retiming.hh"
-#include "SEQ_sequencer.h"
-#include "SEQ_time.h"
-#include "SEQ_transform.h"
+#include "SEQ_sequencer.hh"
+#include "SEQ_time.hh"
+#include "SEQ_transform.hh"
 
-#include "sequencer.h"
-#include "strip_time.h"
-#include "utils.h"
+#include "sequencer.hh"
+#include "strip_time.hh"
+#include "utils.hh"
 
-float seq_time_media_playback_rate_factor_get(const Scene *scene, const Sequence *seq)
+float SEQ_time_media_playback_rate_factor_get(const Scene *scene, const Sequence *seq)
 {
   if ((seq->flag & SEQ_AUTO_PLAYBACK_RATE) == 0) {
     return 1.0f;
@@ -60,7 +60,7 @@ int seq_time_strip_original_content_length_get(const Scene *scene, const Sequenc
     return seq->len;
   }
 
-  return seq->len / seq_time_media_playback_rate_factor_get(scene, seq);
+  return seq->len / SEQ_time_media_playback_rate_factor_get(scene, seq);
 }
 
 float SEQ_give_frame_index(const Scene *scene, Sequence *seq, float timeline_frame)
@@ -91,7 +91,7 @@ float SEQ_give_frame_index(const Scene *scene, Sequence *seq, float timeline_fra
 
   frame_index = max_ff(frame_index, 0);
 
-  frame_index *= seq_time_media_playback_rate_factor_get(scene, seq);
+  frame_index *= SEQ_time_media_playback_rate_factor_get(scene, seq);
 
   if (SEQ_retiming_is_active(seq)) {
     const float retiming_factor = seq_retiming_evaluate(seq, frame_index);
@@ -200,8 +200,9 @@ void SEQ_time_update_meta_strip_range(const Scene *scene, Sequence *seq_meta)
   seq_meta->enddisp = strip_end; /* Only to make files usable in older versions. */
 
   seq_update_sound_bounds_recursive(scene, seq_meta);
+  blender::Span effects = seq_sequence_lookup_effects_by_seq(scene, seq_meta);
+  seq_time_update_effects_strip_range(scene, effects);
   SEQ_time_update_meta_strip_range(scene, seq_sequence_lookup_meta_by_seq(scene, seq_meta));
-  seq_time_update_effects_strip_range(scene, seq_sequence_lookup_effects_by_seq(scene, seq_meta));
 }
 
 void seq_time_effect_range_set(const Scene *scene, Sequence *seq)
@@ -226,7 +227,7 @@ void seq_time_effect_range_set(const Scene *scene, Sequence *seq)
   }
 
   if (seq->startdisp > seq->enddisp) {
-    SWAP(int, seq->startdisp, seq->enddisp);
+    std::swap(seq->startdisp, seq->enddisp);
   }
 
   /* Values unusable for effects, these should be always 0. */
@@ -235,22 +236,18 @@ void seq_time_effect_range_set(const Scene *scene, Sequence *seq)
   seq->len = seq->enddisp - seq->startdisp;
 }
 
-void seq_time_update_effects_strip_range(const Scene *scene, SeqCollection *effects)
+void seq_time_update_effects_strip_range(const Scene *scene, blender::Span<Sequence *> &effects)
 {
-  if (effects == nullptr) {
-    return;
-  }
-
-  Sequence *seq;
   /* First pass: Update length of immediate effects. */
-  SEQ_ITERATOR_FOREACH (seq, effects) {
+  for (Sequence *seq : effects) {
     seq_time_effect_range_set(scene, seq);
   }
 
   /* Second pass: Recursive call to update effects in chain and in order, so they inherit length
    * correctly. */
-  SEQ_ITERATOR_FOREACH (seq, effects) {
-    seq_time_update_effects_strip_range(scene, seq_sequence_lookup_effects_by_seq(scene, seq));
+  for (Sequence *seq : effects) {
+    blender::Span effects = seq_sequence_lookup_effects_by_seq(scene, seq);
+    seq_time_update_effects_strip_range(scene, effects);
   }
 }
 
@@ -345,7 +342,7 @@ float SEQ_time_sequence_get_fps(Scene *scene, Sequence *seq)
       }
       short frs_sec;
       float frs_sec_base;
-      if (IMB_anim_get_fps(strip_anim->anim, &frs_sec, &frs_sec_base, true)) {
+      if (IMB_anim_get_fps(strip_anim->anim, true, &frs_sec, &frs_sec_base)) {
         return float(frs_sec) / frs_sec_base;
       }
       break;
@@ -398,11 +395,10 @@ void SEQ_timeline_boundbox(const Scene *scene, const ListBase *seqbase, rctf *re
 }
 
 static bool strip_exists_at_frame(const Scene *scene,
-                                  SeqCollection *all_strips,
+                                  blender::Span<Sequence *> strips,
                                   const int timeline_frame)
 {
-  Sequence *seq;
-  SEQ_ITERATOR_FOREACH (seq, all_strips) {
+  for (Sequence *seq : strips) {
     if (SEQ_time_strip_intersects_frame(scene, seq, timeline_frame)) {
       return true;
     }
@@ -423,12 +419,12 @@ void seq_time_gap_info_get(const Scene *scene,
   int timeline_frame = initial_frame;
   r_gap_info->gap_exists = false;
 
-  SeqCollection *collection = SEQ_query_all_strips(seqbase);
+  blender::VectorSet strips = SEQ_query_all_strips(seqbase);
 
-  if (!strip_exists_at_frame(scene, collection, initial_frame)) {
+  if (!strip_exists_at_frame(scene, strips, initial_frame)) {
     /* Search backward for gap_start_frame. */
     for (; timeline_frame >= sfra; timeline_frame--) {
-      if (strip_exists_at_frame(scene, collection, timeline_frame)) {
+      if (strip_exists_at_frame(scene, strips, timeline_frame)) {
         break;
       }
     }
@@ -438,7 +434,7 @@ void seq_time_gap_info_get(const Scene *scene,
   else {
     /* Search forward for gap_start_frame. */
     for (; timeline_frame <= efra; timeline_frame++) {
-      if (!strip_exists_at_frame(scene, collection, timeline_frame)) {
+      if (!strip_exists_at_frame(scene, strips, timeline_frame)) {
         r_gap_info->gap_start_frame = timeline_frame;
         break;
       }
@@ -446,7 +442,7 @@ void seq_time_gap_info_get(const Scene *scene,
   }
   /* Search forward for gap_end_frame. */
   for (; timeline_frame <= efra; timeline_frame++) {
-    if (strip_exists_at_frame(scene, collection, timeline_frame)) {
+    if (strip_exists_at_frame(scene, strips, timeline_frame)) {
       const int gap_end_frame = timeline_frame;
       r_gap_info->gap_length = gap_end_frame - r_gap_info->gap_start_frame;
       r_gap_info->gap_exists = true;
@@ -481,13 +477,13 @@ bool SEQ_time_has_still_frames(const Scene *scene, const Sequence *seq)
 int SEQ_time_strip_length_get(const Scene *scene, const Sequence *seq)
 {
   if (SEQ_retiming_is_active(seq)) {
-    const SeqRetimingKey *key_start = seq->retiming_keys;
-    const SeqRetimingKey *key_end = seq->retiming_keys + (SEQ_retiming_keys_count(seq) - 1);
-    return (key_end->strip_frame_index + 1) / seq_time_media_playback_rate_factor_get(scene, seq) -
-           key_start->strip_frame_index / seq_time_media_playback_rate_factor_get(scene, seq);
+    const int last_key_frame = SEQ_retiming_key_timeline_frame_get(
+        scene, seq, SEQ_retiming_last_key_get(seq));
+    /* Last key is mapped to last frame index. Numbering starts from 0. */
+    return last_key_frame + 1 - SEQ_time_start_frame_get(seq);
   }
 
-  return seq->len / seq_time_media_playback_rate_factor_get(scene, seq);
+  return seq->len / SEQ_time_media_playback_rate_factor_get(scene, seq);
 }
 
 float SEQ_time_start_frame_get(const Sequence *seq)
@@ -498,8 +494,9 @@ float SEQ_time_start_frame_get(const Sequence *seq)
 void SEQ_time_start_frame_set(const Scene *scene, Sequence *seq, int timeline_frame)
 {
   seq->start = timeline_frame;
+  blender::Span effects = seq_sequence_lookup_effects_by_seq(scene, seq);
+  seq_time_update_effects_strip_range(scene, effects);
   SEQ_time_update_meta_strip_range(scene, seq_sequence_lookup_meta_by_seq(scene, seq));
-  seq_time_update_effects_strip_range(scene, seq_sequence_lookup_effects_by_seq(scene, seq));
 }
 
 float SEQ_time_content_end_frame_get(const Scene *scene, const Sequence *seq)
@@ -547,8 +544,9 @@ void SEQ_time_left_handle_frame_set(const Scene *scene, Sequence *seq, int timel
 
   seq->startdisp = timeline_frame; /* Only to make files usable in older versions. */
 
+  blender::Span effects = seq_sequence_lookup_effects_by_seq(scene, seq);
+  seq_time_update_effects_strip_range(scene, effects);
   SEQ_time_update_meta_strip_range(scene, seq_sequence_lookup_meta_by_seq(scene, seq));
-  seq_time_update_effects_strip_range(scene, seq_sequence_lookup_effects_by_seq(scene, seq));
 }
 
 void SEQ_time_right_handle_frame_set(const Scene *scene, Sequence *seq, int timeline_frame)
@@ -562,8 +560,9 @@ void SEQ_time_right_handle_frame_set(const Scene *scene, Sequence *seq, int time
   seq->endofs = SEQ_time_content_end_frame_get(scene, seq) - timeline_frame;
   seq->enddisp = timeline_frame; /* Only to make files usable in older versions. */
 
+  blender::Span effects = seq_sequence_lookup_effects_by_seq(scene, seq);
+  seq_time_update_effects_strip_range(scene, effects);
   SEQ_time_update_meta_strip_range(scene, seq_sequence_lookup_meta_by_seq(scene, seq));
-  seq_time_update_effects_strip_range(scene, seq_sequence_lookup_effects_by_seq(scene, seq));
 }
 
 void seq_time_translate_handles(const Scene *scene, Sequence *seq, const int offset)
@@ -573,6 +572,55 @@ void seq_time_translate_handles(const Scene *scene, Sequence *seq, const int off
   seq->startdisp += offset; /* Only to make files usable in older versions. */
   seq->enddisp -= offset;   /* Only to make files usable in older versions. */
 
+  blender::Span effects = seq_sequence_lookup_effects_by_seq(scene, seq);
+  seq_time_update_effects_strip_range(scene, effects);
   SEQ_time_update_meta_strip_range(scene, seq_sequence_lookup_meta_by_seq(scene, seq));
-  seq_time_update_effects_strip_range(scene, seq_sequence_lookup_effects_by_seq(scene, seq));
+}
+
+static void seq_time_slip_strip_ex(const Scene *scene, Sequence *seq, int delta, bool recursed)
+{
+  if (delta == 0) {
+    return;
+  }
+
+  /* Skip effect strips where the length is dependent on another strip,
+   * as they are calculated with #seq_time_update_effects_strip_range. */
+  if (seq->seq1 != nullptr || seq->seq2 != nullptr) {
+    return;
+  }
+
+  /* Effects only have a start frame and a length, so unless we're inside
+   * a meta strip, there's no need to do anything. */
+  if (!recursed && (seq->type & SEQ_TYPE_EFFECT)) {
+    return;
+  }
+
+  /* Move strips inside meta strip. */
+  if (seq->type == SEQ_TYPE_META) {
+    /* If the meta strip has no contents, don't do anything. */
+    if (BLI_listbase_is_empty(&seq->seqbase)) {
+      return;
+    }
+    LISTBASE_FOREACH (Sequence *, seq_child, &seq->seqbase) {
+      seq_time_slip_strip_ex(scene, seq_child, delta, true);
+    }
+  }
+
+  seq->start = seq->start + delta;
+  if (!recursed) {
+    seq->startofs = seq->startofs - delta;
+    seq->endofs = seq->endofs + delta;
+  }
+
+  /* Only to make files usable in older versions. */
+  seq->startdisp = SEQ_time_left_handle_frame_get(scene, seq);
+  seq->enddisp = SEQ_time_right_handle_frame_get(scene, seq);
+
+  blender::Span effects = seq_sequence_lookup_effects_by_seq(scene, seq);
+  seq_time_update_effects_strip_range(scene, effects);
+}
+
+void SEQ_time_slip_strip(const Scene *scene, Sequence *seq, int delta)
+{
+  seq_time_slip_strip_ex(scene, seq, delta, false);
 }
