@@ -14,6 +14,7 @@
  * - Links to web sites.
  */
 
+#include <algorithm>
 #include <cstring>
 
 #include "CLG_log.h"
@@ -27,17 +28,17 @@
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
-#include "BKE_appdir.h"
+#include "BKE_appdir.hh"
 #include "BKE_blender_version.h"
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_screen.hh"
 
 #include "BLT_translation.h"
 
-#include "BLF_api.h"
+#include "BLF_api.hh"
 
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
 
 #include "ED_datafiles.h"
 #include "ED_screen.hh"
@@ -51,7 +52,11 @@
 
 #include "wm.hh"
 
-static void wm_block_close(bContext *C, void *arg_block, void * /*arg*/)
+/* -------------------------------------------------------------------- */
+/** \name Splash Screen
+ * \{ */
+
+static void wm_block_splash_close(bContext *C, void *arg_block, void * /*arg*/)
 {
   wmWindow *win = CTX_wm_window(C);
   UI_popup_block_close(C, win, static_cast<uiBlock *>(arg_block));
@@ -127,7 +132,7 @@ static void wm_block_splash_image_roundcorners_add(ImBuf *ibuf)
     }
   }
 }
-#endif /* WITH_HEADLESS */
+#endif /* !WITH_HEADLESS */
 
 static ImBuf *wm_block_splash_image(int width, int *r_height)
 {
@@ -153,6 +158,7 @@ static ImBuf *wm_block_splash_image(int width, int *r_height)
   }
 
   if (ibuf) {
+    ibuf->planes = 32; /* The image might not have an alpha channel. */
     height = (width * ibuf->y) / ibuf->x;
     if (width != ibuf->x || height != ibuf->y) {
       IMB_scaleImBuf(ibuf, width, height);
@@ -169,7 +175,31 @@ static ImBuf *wm_block_splash_image(int width, int *r_height)
   return ibuf;
 }
 
-static uiBlock *wm_block_create_splash(bContext *C, ARegion *region, void * /*arg*/)
+/**
+ * Close the splash when opening a file-selector.
+ */
+static void wm_block_splash_close_on_fileselect(bContext *C, void *arg1, void * /*arg2*/)
+{
+  wmWindow *win = CTX_wm_window(C);
+  if (!win) {
+    return;
+  }
+
+  /* Check for the event as this will run before the new window/area has been created. */
+  bool has_fileselect = false;
+  LISTBASE_FOREACH (const wmEvent *, event, &win->event_queue) {
+    if (event->type == EVT_FILESELECT) {
+      has_fileselect = true;
+      break;
+    }
+  }
+
+  if (has_fileselect) {
+    wm_block_splash_close(C, arg1, nullptr);
+  }
+}
+
+static uiBlock *wm_block_splash_create(bContext *C, ARegion *region, void * /*arg*/)
 {
   const uiStyle *style = UI_style_get_dpi();
 
@@ -181,7 +211,7 @@ static uiBlock *wm_block_create_splash(bContext *C, ARegion *region, void * /*ar
   UI_block_flag_enable(block, UI_BLOCK_LOOP | UI_BLOCK_KEEP_OPEN | UI_BLOCK_NO_WIN_CLIP);
   UI_block_theme_style_set(block, UI_BLOCK_THEME_STYLE_POPUP);
 
-  const int text_points_max = MAX2(style->widget.points, style->widgetlabel.points);
+  const int text_points_max = std::max(style->widget.points, style->widgetlabel.points);
   int splash_width = text_points_max * 45 * UI_SCALE_FAC;
   CLAMP_MAX(splash_width, CTX_wm_window(C)->sizex * 0.7f);
   int splash_height;
@@ -194,7 +224,7 @@ static uiBlock *wm_block_create_splash(bContext *C, ARegion *region, void * /*ar
     uiBut *but = uiDefButImage(
         block, ibuf, 0, 0.5f * U.widget_unit, splash_width, splash_height, nullptr);
 
-    UI_but_func_set(but, wm_block_close, block, nullptr);
+    UI_but_func_set(but, wm_block_splash_close, block, nullptr);
 
     wm_block_splash_add_label(block,
                               BKE_blender_version_string(),
@@ -215,10 +245,10 @@ static uiBlock *wm_block_create_splash(bContext *C, ARegion *region, void * /*ar
 
   MenuType *mt;
   char userpref[FILE_MAX];
-  const char *const cfgdir = BKE_appdir_folder_id(BLENDER_USER_CONFIG, nullptr);
+  const std::optional<std::string> cfgdir = BKE_appdir_folder_id(BLENDER_USER_CONFIG, nullptr);
 
-  if (cfgdir) {
-    BLI_path_join(userpref, sizeof(userpref), cfgdir, BLENDER_USERPREF_FILE);
+  if (cfgdir.has_value()) {
+    BLI_path_join(userpref, sizeof(userpref), cfgdir->c_str(), BLENDER_USERPREF_FILE);
   }
 
   /* Draw setup screen if no preferences have been saved yet. */
@@ -233,6 +263,8 @@ static uiBlock *wm_block_create_splash(bContext *C, ARegion *region, void * /*ar
     mt = WM_menutype_find("WM_MT_splash", true);
   }
 
+  UI_block_func_set(block, wm_block_splash_close_on_fileselect, block, nullptr);
+
   if (mt) {
     UI_menutype_draw(C, mt, layout);
   }
@@ -244,7 +276,7 @@ static uiBlock *wm_block_create_splash(bContext *C, ARegion *region, void * /*ar
 
 static int wm_splash_invoke(bContext *C, wmOperator * /*op*/, const wmEvent * /*event*/)
 {
-  UI_popup_block_invoke(C, wm_block_create_splash, nullptr, nullptr);
+  UI_popup_block_invoke(C, wm_block_splash_create, nullptr, nullptr);
 
   return OPERATOR_FINISHED;
 }
@@ -259,10 +291,16 @@ void WM_OT_splash(wmOperatorType *ot)
   ot->poll = WM_operator_winactive;
 }
 
-static uiBlock *wm_block_create_about(bContext *C, ARegion *region, void * /*arg*/)
+/** \} */
+
+/* -------------------------------------------------------------------- */
+/** \name Splash Screen: About
+ * \{ */
+
+static uiBlock *wm_block_about_create(bContext *C, ARegion *region, void * /*arg*/)
 {
   const uiStyle *style = UI_style_get_dpi();
-  const int text_points_max = MAX2(style->widget.points, style->widgetlabel.points);
+  const int text_points_max = std::max(style->widget.points, style->widgetlabel.points);
   const int dialog_width = text_points_max * 42 * UI_SCALE_FAC;
 
   uiBlock *block = UI_block_begin(C, region, "about", UI_EMBOSS);
@@ -304,7 +342,7 @@ static uiBlock *wm_block_create_about(bContext *C, ARegion *region, void * /*arg
     row = uiLayoutRow(layout, false);
     uiItemS_ex(row, 2.7f);
   }
-#endif /* WITH_HEADLESS */
+#endif /* !WITH_HEADLESS */
 
   uiLayout *col = uiLayoutColumn(layout, true);
 
@@ -320,9 +358,9 @@ static uiBlock *wm_block_create_about(bContext *C, ARegion *region, void * /*arg
   return block;
 }
 
-static int wm_about_invoke(bContext *C, wmOperator * /*op*/, const wmEvent * /*event*/)
+static int wm_splash_about_invoke(bContext *C, wmOperator * /*op*/, const wmEvent * /*event*/)
 {
-  UI_popup_block_invoke(C, wm_block_create_about, nullptr, nullptr);
+  UI_popup_block_invoke(C, wm_block_about_create, nullptr, nullptr);
 
   return OPERATOR_FINISHED;
 }
@@ -333,6 +371,8 @@ void WM_OT_splash_about(wmOperatorType *ot)
   ot->idname = "WM_OT_splash_about";
   ot->description = "Open a window with information about Blender";
 
-  ot->invoke = wm_about_invoke;
+  ot->invoke = wm_splash_about_invoke;
   ot->poll = WM_operator_winactive;
 }
+
+/** \} */
