@@ -53,7 +53,6 @@ class TestQueue:
 
     def __init__(self, filepath: pathlib.Path):
         self.filepath = filepath
-        self.has_multiple_revisions_to_build = False
         self.has_multiple_categories = False
         self.entries = []
 
@@ -117,6 +116,7 @@ class TestConfig:
         self.name = name
         self.base_dir = env.base_dir / name
         self.logs_dir = self.base_dir / 'logs'
+        self.builds_dir = self.base_dir / 'builds'
 
         config = TestConfig._read_config_module(self.base_dir)
         self.tests = TestCollection(env,
@@ -206,13 +206,6 @@ class TestConfig:
             date = env.git_hash_date(git_hash)
             entries += self._get_entries(revision_name, git_hash, '', environment, date)
 
-        # Optimization to avoid rebuilds.
-        revisions_to_build = set()
-        for entry in entries:
-            if entry.status in {'queued', 'outdated'}:
-                revisions_to_build.add(entry.git_hash)
-        self.queue.has_multiple_revisions_to_build = len(revisions_to_build) > 1
-
         # Get entries for revisions based on existing builds.
         for revision_name, executable in self.builds.items():
             executable, environment = self._split_environment_variables(executable)
