@@ -1,3 +1,5 @@
+# SPDX-FileCopyrightText: 2017-2022 Blender Authors
+#
 # SPDX-License-Identifier: GPL-2.0-or-later
 
 # ############################################################
@@ -5,8 +7,6 @@
 # ############################################################
 
 import unittest
-import os
-import sys
 
 from view_layer_common import *
 
